@@ -3,7 +3,7 @@
 
 /*
  * filter_ops.h
- * $Id: filter_ops.h,v 1.9.6.2 2004/10/23 13:14:19 richi Exp $
+ * $Id: filter_ops.h,v 1.12 2004/10/23 13:09:22 richi Exp $
  *
  * Copyright (C) 1999, 2000, 2001, 2003 Richard Guenther
  *
@@ -51,5 +51,13 @@ struct filter_launchcontext {
 	glame_atomic_t result;
 };
 
+struct filter_operations {
+	int (*init)(filter_t *n);
+	int (*launch)(filter_t *n);
+	void (*postprocess)(filter_t *n);
+	int (*wait)(filter_t *n);
+};
+
+
 
 #endif
