@@ -36,7 +36,6 @@ struct gpsmitem_smob {
 	gpsm_item_t *item;
 };
 #define SCM2GPSMITEMSMOB(s) ((struct gpsmitem_smob *)SCM_SMOB_DATA(s))
-#define gpsmitem_p(s) (SCM_NIMP(s) && SCM_CAR(s) == gpsmitem_smob_tag)
 SCM gpsmitem2scm(gpsm_item_t *item);
 gpsm_item_t *scm2gpsmitem(SCM gpsmitem_smob);
 
@@ -312,14 +311,17 @@ static SCM gls_gpsm_swfile_set_position(SCM s_item, SCM s_pos)
  * Global control.
  */
 
-static SCM gls_gpsm_init(SCM s_swapfile)
+static SCM gls_gpsm_init(SCM s_swapfile, SCM s_maxundo)
 {
 	char *swapfile;
-	int res, swapfile_len;
+	int res, swapfile_len, maxundo;
 	SCM_ASSERT(gh_string_p(s_swapfile), s_swapfile,
 		   SCM_ARG1, "gpsm-init");
+	SCM_ASSERT(gh_exact_p(s_maxundo), s_maxundo,
+		   SCM_ARG2, "gpsm-init");
 	swapfile = gh_scm2newstr(s_swapfile, &swapfile_len);
-	res = gpsm_init(swapfile);
+	maxundo = glame_scm2long(s_maxundo);
+	res = gpsm_init(swapfile, maxundo);
 	if (res == -1)
 		GLAME_THROW_ERRNO();
 	return SCM_UNSPECIFIED;
@@ -707,7 +709,7 @@ static SCM gls_gpsm_op_forget(SCM s_item)
 
 
 
-int glscript_init_gpsm(void)
+int glscript_init_gpsm()
 {
 	/* Register gpsmitem, gpsmgrp and gpsmswfile SMOBs to guile. */
 	gpsmitem_smob_tag = scm_make_smob_type("gpsmitem",
