@@ -26,8 +26,13 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#ifdef HAVE_LIBXML
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
+#else
 #include <xmlmemory.h>
 #include <parser.h>
+#endif
 #include "glame_types.h"
 #include "swapfile.h"
 #include "gpsm.h"
@@ -428,7 +433,7 @@ static void scan_swap(void)
  * Initialization API.
  */
 
-int gpsm_init(const char *swapfile)
+int gpsm_init(const char *swapfile, int maxundo)
 {
 	xmlDocPtr doc;
 	char *xml;
@@ -489,7 +494,7 @@ empty:
 	/* Init the op list. */
 	GLAME_INIT_LIST_HEAD(&oplist);
 	op_listsize = 0;
-	glame_config_get_long("swapfile/maxundo", &op_max_listsize);
+	op_max_listsize = maxundo;
 
 	/* Create the tree root group and recurse down the xml tree. */
         root = (gpsm_grp_t *)gpsm_newitem(GPSM_ITEM_TYPE_GRP);
@@ -2591,3 +2596,27 @@ void gpsm_position_transform(gpsm_item_t *source, gpsm_grp_t *dest,
 		grp = gpsm_item_parent(grp);
 	}
 }
+
+
+int gpsm_group_common_samplerate(gpsm_grp_t *grp)
+{
+	gpsm_item_t *item;
+	int samplerate = -1;
+
+	gpsm_grp_foreach_item(grp, item) {
+		int sr;
+		if (GPSM_ITEM_IS_GRP(item)) {
+			sr = gpsm_group_common_samplerate((gpsm_grp_t *)item);
+			if (sr == -1)
+				continue;
+		} else
+			sr = gpsm_swfile_samplerate(item);
+
+		if (samplerate == -1)
+			samplerate = sr;
+		else if (sr != samplerate)
+			return -1;
+	}
+
+	return samplerate;
+}
