@@ -102,10 +102,14 @@
 #endif
 
 #include <sys/time.h>
+#include <time.h>
 #include <unistd.h>
 #include "list.h"
 #include "glsignal.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 struct gpsm_item_s;
 typedef struct gpsm_item_s gpsm_item_t;
@@ -243,12 +247,15 @@ struct gpsm_swfile_s {
 #define GPSM_GRP_CLIPBOARD_LABEL "[clipboard]"
 #define GPSM_GRP_CLIPBOARD_VPOS 980000
 
+/* Type for PLUGIN_GPSMOP callback functions. */
+typedef int (*gpsmop_func_t)(gpsm_item_t *, long, long);
+
 
 /* Initializes the gpsm subsystem using the specified swapfile
  * (the gpsm subsystem metadata is stored in swapfile file 0).
  * Returns 0 on success and -1 on error (which is the sign for
  * swapfile corruption or already initialized gpsm). */
-int gpsm_init(const char *swapfile);
+int gpsm_init(const char *swapfile, int maxundo);
 
 /* Changes (or just queries, if max < 0) the maximum number of states
  * saved for undo/redo. Returns the actual set value. */
@@ -508,5 +515,13 @@ gpsm_grp_t *gpsm_flatten(gpsm_item_t *item);
 void gpsm_position_transform(gpsm_item_t *source, gpsm_grp_t *dest,
 			     long *hpos, long *vpos);
 
+/* Queries a possibly common samplerate of a group.  Returns -1,
+ * if there are different samplerates (or no swfiles) in the group. */
+int gpsm_group_common_samplerate(gpsm_grp_t *grp);
+
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
