@@ -1,7 +1,7 @@
 /*
  * ladspa.c
  *
- * $Id: ladspa.c,v 1.18.4.4 2004/10/23 13:14:20 richi Exp $
+ * $Id: ladspa.c,v 1.33 2004/10/23 13:09:23 richi Exp $
  * 
  * Copyright (C) 2000, 2001, 2002, 2003 Richard Furse, Alexander Ehlert,
  * 	Richard Guenther
@@ -32,6 +32,10 @@
 #include <ctype.h>
 #include <limits.h>
 #include <ladspa.h>
+#ifdef HAVE_LRDF
+#include <stdio.h>
+#include <lrdf.h>
+#endif
 #include "filter.h"
 #include "util.h"
 #include "glplugin.h"
@@ -58,7 +62,6 @@ static int ladspa_f(filter_t * n)
 	nto1_state_t *psNTo1_State = NULL;
 	int iNTo1_NR, iNTo1_Index, i;
 	filter_pipe_t **ppsAudioPorts;
-	filter_pipe_t *psPipe;
 	filter_param_t *psParam;
 	filter_buffer_t **ppsBuffers;
 	LADSPA_Data *pfControlValues;
@@ -81,21 +84,24 @@ static int ladspa_f(filter_t * n)
 	iNTo1_NR = 0;
 	lSampleRate = 0;
 	for (lPortIndex = 0; lPortIndex < lPortCount; lPortIndex++) {
-		iPortDescriptor =
-		    psDescriptor->PortDescriptors[lPortIndex];
-		if (LADSPA_IS_PORT_AUDIO(iPortDescriptor)) {
-			if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
+		filter_port_t *psPort;
+		filter_pipe_t *psPipe;
+		psPort = filterportdb_get_port
+			(filter_portdb(n), psDescriptor->PortNames[lPortIndex]);
+		if (!psPort)
+			continue;
+		if (filterport_is_input(psPort)
+		    && filterport_get_property(psPort, "!CONTROL")) {
+			if (filterport_get_pipe(psPort))
+				iNTo1_NR++;
+		} else {
+			psPipe = filterport_get_pipe(psPort);
+			if (!psPipe && filterport_is_input(psPort))
+				FILTER_ERROR_RETURN
+					("LADSPA plugins require all "
+					 "inputs be connected");
+			if (filterport_is_input(psPort)) {
 				iNTo1_NR++;
-				psPipe =
-				    filterport_get_pipe
-				    (filterportdb_get_port
-				     (filter_portdb(n),
-				      psDescriptor->
-				      PortNames[lPortIndex]));
-				if (!psPipe)
-					FILTER_ERROR_RETURN
-					    ("LADSPA plugins require all "
-					     "inputs be connected");
 				lNewSampleRate =
 				    filterpipe_sample_rate(psPipe);
 				if (lSampleRate != 0
@@ -105,18 +111,6 @@ static int ladspa_f(filter_t * n)
 					     "to be at the same sample rate");
 				else
 					lSampleRate = lNewSampleRate;
-			} else {	/* LADSPA_IS_PORT_OUTPUT */
-
-				psPipe =
-				    filterport_get_pipe
-				    (filterportdb_get_port
-				     (filter_portdb(n),
-				      psDescriptor->
-				      PortNames[lPortIndex]));
-				if (!psPipe)
-					FILTER_ERROR_RETURN
-					    ("LADSPA plugins require all "
-					     "outputs be connected");
 			}
 		}
 	}
@@ -191,25 +185,30 @@ static int ladspa_f(filter_t * n)
 
 	iNTo1_Index = 0;
 	for (lPortIndex = 0; lPortIndex < lPortCount; lPortIndex++) {
+		filter_port_t *psPort;
 
 		iPortDescriptor =
 		    psDescriptor->PortDescriptors[lPortIndex];
 
-		if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)) {
-			if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
-				/* Lookup the control value. */
-				psParam =
-				    filterparamdb_get_param(filter_paramdb
-							    (n),
-							    psDescriptor->
-							    PortNames
-							    [lPortIndex]);
-				/* psParam == NULL does not happen if params were registered
-				 * appropriately. [richi] */
+		/* First handle control ports as params. Always. */
+		if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
+		    && LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
+			/* Lookup the control value. */
+			psParam =
+				filterparamdb_get_param(filter_paramdb
+							(n),
+							psDescriptor->
+							PortNames
+							[lPortIndex]);
+			/* psParam == NULL does not happen if params were registered
+			 * appropriately. [richi] */
+			if (FILTER_PARAM_IS_DOUBLE(psParam))
 				pfControlValues[lPortIndex] =
-				    filterparam_val_double(psParam);
-			}
-			
+					filterparam_val_double(psParam);
+			else if (FILTER_PARAM_IS_LONG(psParam))
+				pfControlValues[lPortIndex] =
+					filterparam_val_long(psParam);
+
 /* 
  * We now need to wire up the control port on the LADSPA plugin
  * to the point in the control ports array. For an input port
@@ -218,44 +217,42 @@ static int ladspa_f(filter_t * n)
  * relevant spot in the array but never read again. Less than
  * ideal, perhaps a FIXME for a later date... 
  */
-			
 			psDescriptor->connect_port(psLADSPAPluginInstance,
 						   lPortIndex,
 						   pfControlValues +
 						   lPortIndex);
-		} else {	/* i.e. LADSPA_IS_PORT_AUDIO(iPortDescriptor) */
+		}
+
+		/* Second handle audio ports and control ports as streams. */
+		psPort = filterportdb_get_port
+			(filter_portdb(n), psDescriptor->PortNames[lPortIndex]);
+		if (!psPort)
+			continue;
+
+		if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
+		    && !filterport_get_pipe(psPort))
+			continue;
+
+		/* now LADSPA_IS_PORT_AUDIO(iPortDescriptor) or
+		 * a connected control port. */
 
 /* 
  * We simply want to hang on to the port input/output channel
  * for use further along. 
  */
-			if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
-				DPRINTF("Mapping audio input port %ld",
-						lPortIndex);
-				psNTo1_State[iNTo1_Index++].in
-				    = ppsAudioPorts[lPortIndex]
-				    =
-				    filterport_get_pipe
-				    (filterportdb_get_port
-				     (filter_portdb(n),
-				      psDescriptor->
-				      PortNames[lPortIndex]));
-			} 
-			else	/* i.e. LADSPA_IS_PORT_OUTPUT(iPortDescriptor) */
-			{
-				DPRINTF("Mapping audio output port %ld\n",
-						lPortIndex);
-				ppsAudioPorts[lPortIndex] =
-					filterport_get_pipe
-				    (filterportdb_get_port
-				     (filter_portdb(n),
-				      psDescriptor->
-				      PortNames[lPortIndex]));
-			}
-			if (!ppsAudioPorts[lPortIndex]) {
-				DPRINTF("port %ld not connected\n", lPortIndex);
-				FILTER_ERROR_CLEANUP("port not connected");
-			}
+		if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
+			DPRINTF("Mapping audio input port %ld",
+				lPortIndex);
+			psNTo1_State[iNTo1_Index++].in
+				= ppsAudioPorts[lPortIndex]
+				= filterport_get_pipe(psPort);
+		} 
+		else	/* i.e. LADSPA_IS_PORT_OUTPUT(iPortDescriptor) */
+		{
+			DPRINTF("Mapping audio output port %ld\n",
+				lPortIndex);
+			ppsAudioPorts[lPortIndex] =
+				filterport_get_pipe(psPort);
 		}
 	}
 
@@ -284,21 +281,29 @@ static int ladspa_f(filter_t * n)
 			iPortDescriptor =
 			    psDescriptor->PortDescriptors[lPortIndex];
 
-			if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)) {
-				if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
-					/* Lookup the control value. */
-					psParam =
-					    filterparamdb_get_param(filter_paramdb
-								    (n),
-								    psDescriptor->
-								    PortNames
-								    [lPortIndex]);
+			if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
+			    && LADSPA_IS_PORT_INPUT(iPortDescriptor)
+			    && !ppsAudioPorts[lPortIndex]) {
+				/* Lookup the control value - if not connected through a pipe. */
+				psParam = filterparamdb_get_param(
+					filter_paramdb(n),
+					psDescriptor->PortNames[lPortIndex]);
 				/* psParam == NULL does not happen if params were registered
 				 * appropriately. [richi] */
+				if (FILTER_PARAM_IS_DOUBLE(psParam))
 					pfControlValues[lPortIndex] =
-					    filterparam_val_double(psParam);
-				}
-			
+						filterparam_val_double(psParam);
+				else if (FILTER_PARAM_IS_LONG(psParam))
+					pfControlValues[lPortIndex] =
+						filterparam_val_long(psParam);
+				psDescriptor->connect_port(psLADSPAPluginInstance,
+							   lPortIndex,
+							   pfControlValues +
+							   lPortIndex);
+
+			} else if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
+				   && LADSPA_IS_PORT_OUTPUT(iPortDescriptor)) {
+				/* Output controls get just written to and ignored then... */
 				psDescriptor->connect_port(psLADSPAPluginInstance,
 							   lPortIndex,
 							   pfControlValues +
@@ -316,11 +321,12 @@ static int ladspa_f(filter_t * n)
 			for (lPortIndex = 0; lPortIndex < lPortCount;
 			     lPortIndex++) {
 				iPortDescriptor =
-				    psDescriptor->
-				    PortDescriptors[lPortIndex];
-				if (LADSPA_IS_PORT_AUDIO(iPortDescriptor)
-				    &&
-				    LADSPA_IS_PORT_INPUT(iPortDescriptor))
+				    psDescriptor->PortDescriptors[lPortIndex];
+				if ((LADSPA_IS_PORT_AUDIO(iPortDescriptor)
+				     && LADSPA_IS_PORT_INPUT(iPortDescriptor))
+				    || (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
+					&& LADSPA_IS_PORT_INPUT(iPortDescriptor)
+					&& ppsAudioPorts[lPortIndex]))
 				{
 
 					if (!psNTo1_State[iNTo1_Index].buf) {
@@ -331,8 +337,7 @@ static int ladspa_f(filter_t * n)
 					}
 
 					/* We have audio coming in on this port. Link to it. */
-					if (psNTo1_State[iNTo1_Index].s !=
-					    NULL) {
+					if (psNTo1_State[iNTo1_Index].s != NULL) {
 						psDescriptor->
 						    connect_port
 						    (psLADSPAPluginInstance,
@@ -366,7 +371,7 @@ static int ladspa_f(filter_t * n)
 			}
 		} else {
 	/* We have no buffers coming in. Therefore we can choose our own
-			   sample count. Large counts are good. */
+	   sample count. Large counts are good. */
 			lRunSampleCount = GLAME_WBUFSIZE;
 			if  (iNTo1_NR==0)
 				glame_timer -= GLAME_WBUFSIZE;
@@ -416,24 +421,21 @@ static int ladspa_f(filter_t * n)
 		   we wrote to and release the input buffers we read from. If were
 		   supporting control outputs we would handle these here too. */
 		for (lPortIndex = 0; lPortIndex < lPortCount; lPortIndex++) {
-
 			iPortDescriptor =
 			    psDescriptor->PortDescriptors[lPortIndex];
 			if (LADSPA_IS_PORT_AUDIO(iPortDescriptor)
 			    && LADSPA_IS_PORT_OUTPUT(iPortDescriptor))
 				sbuf_queue(ppsAudioPorts[lPortIndex],
 					   ppsBuffers[lPortIndex]);
-
-#if 0
-			/* Test code: output the data in any control outputs to stdout: */
-			if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
-			    && LADSPA_IS_PORT_OUTPUT(iPortDescriptor))
-				printf("Control output (%s/%s): %g\n",
-				       psDescriptor->Name,
-				       psDescriptor->PortNames[lPortIndex],
-				       pfControlValues[lPortIndex]);
-#endif
-
+			else if (LADSPA_IS_PORT_CONTROL(iPortDescriptor)
+				 && LADSPA_IS_PORT_OUTPUT(iPortDescriptor)
+				 && ppsAudioPorts[lPortIndex]) {
+				filter_buffer_t *sbuf = sbuf_make_private(sbuf_alloc(lRunSampleCount, n));
+				SAMPLE *s = sbuf_buf(sbuf);
+				for (i=0; i<lRunSampleCount; ++i)
+					s[i] = pfControlValues[lPortIndex];
+				sbuf_queue(ppsAudioPorts[lPortIndex], sbuf);
+			}
 		}
 	}
 
@@ -509,37 +511,31 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 		    psDescriptor->PortDescriptors[lPortIndex];
 
 		/* LADSPA audio ports are translated directly to GLAME sample
-		   ports. */
-		if (LADSPA_IS_PORT_AUDIO(iPortDescriptor)) {
-			if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
-				psPort =
-				    filterportdb_add_port(filter_portdb(psFilter),
-							  psDescriptor->PortNames[lPortIndex],
-							  FILTER_PORTTYPE_SAMPLE,
-							  FILTER_PORTFLAG_INPUT,
-							  FILTERPORT_DESCRIPTION,
-							  psDescriptor->PortNames[lPortIndex],
-							  FILTERPORT_END);
+		 * ports. Control ports get transformed to audio ports marked
+		 * with the !CONTROL property. */
+		if (LADSPA_IS_PORT_INPUT(iPortDescriptor)) {
+			psPort = filterportdb_add_port(filter_portdb(psFilter),
+						       psDescriptor->PortNames[lPortIndex],
+						       FILTER_PORTTYPE_SAMPLE,
+						       FILTER_PORTFLAG_INPUT,
+						       FILTERPORT_DESCRIPTION,
+						       psDescriptor->PortNames[lPortIndex],
+						       FILTERPORT_END);
+			if (LADSPA_IS_PORT_AUDIO(iPortDescriptor))
 				bHasAudioInput = 1;
-			} else {	/* LADSPA_IS_PORT_OUTPUT(iPortDescriptor) */
-
-				psPort =
-				    filterportdb_add_port(filter_portdb
-							  (psFilter),
-							  psDescriptor->
-							  PortNames
-							  [lPortIndex],
-							  FILTER_PORTTYPE_SAMPLE,
-							  FILTER_PORTFLAG_OUTPUT,
-							  FILTERPORT_DESCRIPTION,
-							  psDescriptor->
-							  PortNames
-							  [lPortIndex],
-							  FILTERPORT_END);
-			}
-			if (!psPort)
-				return -1;
+		} else /* if (LADSPA_IS_PORT_OUTPUT(iPortDescriptor)) */ {
+			psPort = filterportdb_add_port(filter_portdb(psFilter),
+						       psDescriptor->PortNames[lPortIndex],
+						       FILTER_PORTTYPE_SAMPLE,
+						       FILTER_PORTFLAG_OUTPUT,
+						       FILTERPORT_DESCRIPTION,
+						       psDescriptor->PortNames[lPortIndex],
+						       FILTERPORT_END);
 		}
+		if (!psPort)
+			return -1;
+		if (LADSPA_IS_PORT_CONTROL(iPortDescriptor))
+			filterport_set_property(psPort, "!CONTROL", "true");
 
 		/* Interpret input controls as parameters. In fact they could be
 		   varied, but we're using a simple model for now. Perhaps this is
@@ -649,82 +645,42 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 					lpage++;
 				step = pow(10.0, lstep);
 				page = pow(10.0, lpage);
-#if 1
 				snprintf(xml, 1023,
-"<?xml version=\"1.0\"?><GTK-Interface>"
-"  <widget>"
-"    <class>GtkHScale</class>"
-"    <name>widget</name>"
-"    <can_focus>True</can_focus>"
-"    <draw_value>True</draw_value>"
-"    <value_pos>GTK_POS_LEFT</value_pos>"
-"    <digits>2</digits>"
-"    <policy>GTK_UPDATE_CONTINUOUS</policy>"
-"    <value>%.3f</value>"
-"    <lower>%.3f</lower>"
-"    <upper>%.3f</upper>"
-"    <step>%.3f</step>"
-"    <page>%.3f</page>"
-"    <page_size>%.3f</page_size>"
+"<?xml version=\"1.0\" standalone=\"no\"?>"
+"<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-2.0.dtd\">"
+"<glade-interface>"
+"  <widget class=\"GtkHScale\" id=\"widget\">"
+"    <property name=\"visible\">True</property>"
+"    <property name=\"can_focus\">True</property>"
+"    <property name=\"draw_value\">True</property>"
+"    <property name=\"value_pos\">GTK_POS_LEFT</property>"
+"    <property name=\"digits\">2</property>"
+"    <property name=\"update_policy\">GTK_UPDATE_CONTINUOUS</property>"
+"    <property name=\"inverted\">False</property>"
+"    <property name=\"adjustment\">%.3f %.3f %.3f %.3f %.3f 0.0</property>"
 "  </widget>"
-"</GTK-Interface>",
-					 fRecommendation, fBound1, fBound2, step, page, 0.0);
-#else
-				snprintf(xml, 1023,
-"<?xml version=\"1.0\"?><GTK-Interface>"
-" <widget>"
-"  <class>GtkHBox</class>"
-"  <name>root</name>"
-"  <widget>"
-"    <class>GtkKnob</class>"
-"    <name>widget</name>"
-"    <value>%.3f</value>"
-"    <lower>%.3f</lower>"
-"    <upper>%.3f</upper>"
-"  </widget>"
-"  <widget>"
-"    <class>GtkHScale</class>"
-"    <name>widget2</name>"
-"    <can_focus>True</can_focus>"
-"    <draw_value>True</draw_value>"
-"    <value_pos>GTK_POS_LEFT</value_pos>"
-"    <digits>3</digits>"
-"    <policy>GTK_UPDATE_CONTINUOUS</policy>"
-"    <value>0.0</value>"
-"    <lower>0.0</lower>"
-"    <upper>1.0</upper>"
-"    <step>0.1</step>"
-"    <page>0.1</page>"
-"    <page_size>0.0</page_size>"
-"  </widget>"
-" </widget>"
-"</GTK-Interface>",
-					 fRecommendation, fBound1, fBound2);
-#endif
+"</glade-interface>",
+					 fRecommendation, fBound1, fBound2, step, page);
 				filterparam_set_property(param, FILTERPARAM_GLADEXML, strdup(xml));
 			} else if ((bound_below || bound_above) && !isint) {
 				/* Use GtkSpinButton */
 				char xml[1024];
 				snprintf(xml, 1023,
-"<?xml version=\"1.0\"?><GTK-Interface>"
-"  <widget>"
-"    <class>GtkSpinButton</class>"
-"    <name>widget</name>"
-"    <can_focus>True</can_focus>"
-"    <climb_rate>1</climb_rate>"
-"    <digits>2</digits>"
-"    <numeric>True</numeric>"
-"    <update_policy>GTK_UPDATE_ALWAYS</update_policy>"
-"    <snap>False</snap>"
-"    <wrap>False</wrap>"
-"    <value>%.3f</value>"
-"    <lower>%.3f</lower>"
-"    <upper>%.3f</upper>"
-"    <step>%.3f</step>"
-"    <page>%.3f</page>"
-"    <page_size>0.0</page_size>"
+"<?xml version=\"1.0\" standalone=\"no\"?>"
+"<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-2.0.dtd\">"
+"<glade-interface>"
+"  <widget class=\"GtkSpinButton\" id=\"widget\">"
+"    <property name=\"visible\">True</property>"
+"    <property name=\"can_focus\">True</property>"
+"    <property name=\"climb_rate\">1</property>"
+"    <property name=\"digits\">2</property>"
+"    <property name=\"numeric\">True</property>"
+"    <property name=\"update_policy\">GTK_UPDATE_ALWAYS</property>"
+"    <property name=\"snap_to_ticks\">False</property>"
+"    <property name=\"wrap\">False</property>"
+"    <property name=\"adjustment\">%.3f %.3f %.3f %.3f %.3f 0.0</property>"
 "  </widget>"
-"</GTK-Interface>",
+"</glade-interface>",
 					 fRecommendation,
 					 bound_below ? fBound1 : -1e19,
 					 bound_above ? fBound2 : 1e19, 0.01, 0.1);
@@ -733,25 +689,21 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 				/* Use GtkSpinButton */
 				char xml[1024];
 				snprintf(xml, 1023,
-"<?xml version=\"1.0\"?><GTK-Interface>"
-"  <widget>"
-"    <class>GtkSpinButton</class>"
-"    <name>widget</name>"
-"    <can_focus>True</can_focus>"
-"    <climb_rate>1</climb_rate>"
-"    <digits>0</digits>"
-"    <numeric>True</numeric>"
-"    <update_policy>GTK_UPDATE_ALWAYS</update_policy>"
-"    <snap>False</snap>"
-"    <wrap>False</wrap>"
-"    <value>%.0f</value>"
-"    <lower>%.0f</lower>"
-"    <upper>%.0f</upper>"
-"    <step>%.0f</step>"
-"    <page>%.0f</page>"
-"    <page_size>0.0</page_size>"
+"<?xml version=\"1.0\" standalone=\"no\"?>"
+"<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-2.0.dtd\">"
+"<glade-interface>"
+"  <widget class=\"GtkSpinButton\" id=\"widget\">"
+"    <property name=\"visible\">True</property>"
+"    <property name=\"can_focus\">True</property>"
+"    <property name=\"climb_rate\">1</property>"
+"    <property name=\"digits\">0</property>"
+"    <property name=\"numeric\">True</property>"
+"    <property name=\"update_policy\">GTK_UPDATE_ALWAYS</property>"
+"    <property name=\"snap_to_ticks\">False</property>"
+"    <property name=\"wrap\">False</property>"
+"    <property name=\"adjustment\">%.3f %.3f %.3f %.3f %.3f 0.0</property>"
 "  </widget>"
-"</GTK-Interface>",
+"</glade-interface>",
 					 fRecommendation,
 					 bound_below ? fBound1 : -1000,
 					 bound_above ? fBound2 : 1000,
@@ -761,25 +713,21 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 				/* Use GtkSpinButton */
 				char xml[1024];
 				snprintf(xml, 1023,
-"<?xml version=\"1.0\"?><GTK-Interface>"
-"  <widget>"
-"    <class>GtkSpinButton</class>"
-"    <name>widget</name>"
-"    <can_focus>True</can_focus>"
-"    <climb_rate>1</climb_rate>"
-"    <digits>0</digits>"
-"    <numeric>True</numeric>"
-"    <update_policy>GTK_UPDATE_ALWAYS</update_policy>"
-"    <snap>False</snap>"
-"    <wrap>False</wrap>"
-"    <value>%.0f</value>"
-"    <lower>%.0f</lower>"
-"    <upper>%.0f</upper>"
-"    <step>%.0f</step>"
-"    <page>%.0f</page>"
-"    <page_size>0.0</page_size>"
+"<?xml version=\"1.0\" standalone=\"no\"?>"
+"<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-2.0.dtd\">"
+"<glade-interface>"
+"  <widget class=\"GtkSpinButton\" id=\"widget\">"
+"    <property name=\"visible\">True</property>"
+"    <property name=\"can_focus\">True</property>"
+"    <property name=\"climb_rate\">1</property>"
+"    <property name=\"digits\">0</property>"
+"    <property name=\"numeric\">True</property>"
+"    <property name=\"update_policy\">GTK_UPDATE_ALWAYS</property>"
+"    <property name=\"snap_to_ticks\">False</property>"
+"    <property name=\"wrap\">False</property>"
+"    <property name=\"adjustment\">%.3f %.3f %.3f %.3f %.3f 0.0</property>"
 "  </widget>"
-"</GTK-Interface>",
+"</glade-interface>",
 					 fRecommendation,
 					 0.0,
 					 1.0,
@@ -840,7 +788,14 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 		tolower(psDescriptor->Name ? psDescriptor->Name[0]
                                            : psPlugin->name[0]));
 	plugin_set(psPlugin, PLUGIN_CATEGORY, strdup(category));
-
+#if 0 && defined HAVE_LRDF
+	{
+		uri = lrdf_get_default_uri(psDescriptor->UniqueID);
+		if (uri) {
+			lrdf_get_setting_metadata(uri, "");
+		}
+	}
+#endif
 
 	filter_register(psFilter, psPlugin);
 
