@@ -1,7 +1,7 @@
 /*
  * canvasport.c
  *
- * $Id: canvasport.c,v 1.31.2.1 2004/10/23 13:14:20 richi Exp $
+ * $Id: canvasport.c,v 1.38 2005/03/19 22:14:51 richi Exp $
  *
  * Copyright (C) 2001, 2002 Johannes Hirche
  *
@@ -36,6 +36,8 @@
 #include "util/glame_gui_utils.h"
 #include "hash.h"
 #include <X11/bitmaps/hlines3>
+#include "edit_filter_marshal.h"
+
 extern long bMac;
 extern long nPopupTimeout;
 
@@ -98,34 +100,37 @@ glame_canvas_port_connections_changed_cb(GlameCanvasPort *p, gpointer userdata)
 }
 
 static void
-glame_canvas_port_class_init(GlameCanvasPortClass* class)
+glame_canvas_port_class_init(GlameCanvasPortClass* klass)
 {
 	
 	GtkObjectClass *object_class;
-	object_class = GTK_OBJECT_CLASS(class);
+	object_class = GTK_OBJECT_CLASS(klass);
 	object_class->destroy = glame_canvas_port_destroy;
 
-	port_signals[MOVED] = gtk_signal_new("moved",
-					     GTK_RUN_LAST,
-					     object_class->type,
-					     GTK_SIGNAL_OFFSET(GlameCanvasPortClass, moved),
-					     glame_canvas_marshal_NONE__DOUBLE_DOUBLE,
-					     GTK_TYPE_NONE,
-					     2,
-					     GTK_TYPE_DOUBLE,
-					     GTK_TYPE_DOUBLE);
-	port_signals[CONNECTIONS_CHANGED] = gtk_signal_new("connections_changed",
-							   GTK_RUN_FIRST,
-							   object_class->type,
-							   GTK_SIGNAL_OFFSET(GlameCanvasPortClass,connections_changed),
-							   gtk_marshal_NONE__NONE,
-							   GTK_TYPE_NONE,
-							   0);
+	port_signals[MOVED] = g_signal_new("moved",
+					   G_OBJECT_CLASS_TYPE(object_class),
+					   GTK_RUN_LAST,
+					   GTK_SIGNAL_OFFSET(GlameCanvasPortClass, moved),
+					   NULL,NULL,
+					   edit_filter_marshal_VOID__DOUBLE_DOUBLE,
+					   GTK_TYPE_NONE,
+					   2,
+					   GTK_TYPE_DOUBLE,
+					   GTK_TYPE_DOUBLE);
 	
-	gtk_object_class_add_signals(object_class,port_signals,LAST_SIGNAL);
+	port_signals[CONNECTIONS_CHANGED] = g_signal_new("connections_changed",
+							 G_OBJECT_CLASS_TYPE(object_class),
+							 GTK_RUN_FIRST,
+							 GTK_SIGNAL_OFFSET(GlameCanvasPortClass,connections_changed),
+							 NULL,NULL,
+							 edit_filter_marshal_VOID__VOID,
+							 GTK_TYPE_NONE,
+							 0);
 	
-	class->moved = NULL;
-	class->connections_changed = (void(*)(GlameCanvasPort*))glame_canvas_port_connections_changed_cb;
+	//	gtk_object_class_add_signals(object_class,port_signals,LAST_SIGNAL);
+	
+	klass->moved = NULL;
+	klass->connections_changed = (void(*)(GlameCanvasPort*))glame_canvas_port_connections_changed_cb;
 }
 
 static void
@@ -158,7 +163,6 @@ glame_canvas_port_get_type(void)
 			NULL,NULL,(GtkClassInitFunc)NULL,};
 		canvas_port_type = gtk_type_unique(GNOME_TYPE_CANVAS_RECT,
 						   &canvas_port_info);
-		gtk_type_set_chunk_alloc(canvas_port_type,8);
 	}
 	
 	return canvas_port_type;
@@ -166,8 +170,10 @@ glame_canvas_port_get_type(void)
 
 void glame_canvas_port_pipe_deleted_cb(GlameCanvasPipe* pipe, GlameCanvasPort* port)
 {
-	/* remove pipe from sighandlers */
-	if(is_hashed_gcport(port)){
+	/* Check if port is a life object (if it is hashed).
+	   Using is_hashed_gcport(port) is not safe! */
+	if (hash_find_gcport(filterpipe_source(pipe->pipe)) == port
+	    || hash_find_gcport(filterpipe_dest(pipe->pipe)) == port){
 		gtk_signal_disconnect_by_data(GTK_OBJECT(port),pipe);
 		glame_canvas_port_connections_changed_cb(port,NULL);
 	}
@@ -285,7 +291,7 @@ glame_canvas_port_grabbing_cb(GlameCanvasPort* port, GdkEvent* event, GlameCanva
 		if(bMac){
 			if(event->button.button == 1){
 				menu = gnome_popup_menu_new(port_menu);
-				gnome_popup_menu_do_popup_modal(menu, NULL,NULL,&event->button, port);
+				gnome_popup_menu_do_popup_modal(menu, NULL,NULL,&event->button, port,GTK_WIDGET(CANVAS_ITEM_CANVAS(port)));
 			}
 		}
 		return TRUE;
@@ -387,8 +393,8 @@ glame_canvas_port_redirect_cb(GtkWidget* foo, GlameCanvasPort *port)
 	vbox = GNOME_DIALOG(dialog)->vbox;
 
 	nameEntry = gtk_entry_new();
-	gtk_signal_connect(GTK_OBJECT(nameEntry),"changed",
-			   update_string_from_editable,&filenamebuffer);
+	gtk_signal_connect(GTK_OBJECT(nameEntry), "changed",
+			   GTK_SIGNAL_FUNC(update_string_from_editable), &filenamebuffer);
 	sprintf(nameBuffer,"%s_%s",filter_name(filterport_filter(port->port)),filterport_label(port->port));
 	gtk_entry_set_text(GTK_ENTRY(nameEntry),nameBuffer);
 	create_label_widget_pair(vbox,_("New port name"),nameEntry);
@@ -520,7 +526,7 @@ glame_canvas_port_event_cb(GnomeCanvasItem* item, GdkEvent* event, GlameCanvasPo
 			break;
 		case 3:
 			menu = gnome_popup_menu_new(port_menu);
-			gnome_popup_menu_do_popup_modal(menu, NULL,NULL,&event->button, port);
+			gnome_popup_menu_do_popup_modal(menu, NULL,NULL,&event->button, port,GTK_WIDGET(CANVAS_ITEM_CANVAS(port)));
 			return TRUE;
 			break;
 		default:
@@ -547,6 +553,19 @@ GlameCanvasPort* glame_canvas_port_new(GnomeCanvasGroup* group, filter_port_t *p
 	GlameCanvasPort *p;
 	char * buffer;
 	filter_t* red_node;
+	unsigned int color;
+
+	if (filterport_is_input(port)) {
+		if (filterport_get_property(port, "!CONTROL"))
+			color = 0x00ffff00;
+		else
+			color = 0xff000000;
+	} else {
+		if (filterport_get_property(port, "!CONTROL"))
+			color = 0x00ffff00;
+		else
+			color = 0x0000ff00;
+	}
 	p = GLAME_CANVAS_PORT(gnome_canvas_item_new(group,
 						    glame_canvas_port_get_type(),
 						    "x1",x,
@@ -556,7 +575,7 @@ GlameCanvasPort* glame_canvas_port_new(GnomeCanvasGroup* group, filter_port_t *p
 						    "outline_color","black",
 						    "width_units",1.0,
 						    "fill_color_rgba",
-						    (filterport_is_input(port)?0xff000000:0x0000ff00),
+						    color,
 						    NULL));
 
 
