@@ -1,7 +1,7 @@
 /*
  * filtereditgui.c
  *
- * $Id: filtereditgui.c,v 1.59.2.2 2004/10/23 13:14:20 richi Exp $
+ * $Id: filtereditgui.c,v 1.69 2005/03/11 20:14:23 richi Exp $
  *
  * Copyright (C) 2001, 2002, 2003 Johannes Hirche
  *
@@ -30,11 +30,11 @@
 #include <stdio.h>
 #include <math.h>
 #include <gnome.h>
-#ifdef HAVE_LIBSTROKE
-#include <stroke.h>
-#endif
+#include <gtk/gtkstock.h>
+#include <bonobo.h>
 #include "glscript.h"
 #include "util/glame_gui_utils.h"
+#include "util/glame_dnd.h"
 #include "glame_accelerator.h"
 #include "glconfig.h"
 #include "glamecanvas.h"
@@ -64,9 +64,13 @@ static void glame_canvas_zoom_out_cb(GtkObject*foo, FiltereditGui *gui);
 static void glame_canvas_view_all_cb(GtkObject*foo, FiltereditGui *gui);
 static void glame_canvas_add_last_cb(GtkObject* foo,  FiltereditGui *gui);
 static void window_close(GtkWidget *dummy, GtkWidget* window);
+static void filteredit_gui_data_received(GtkWidget *widget, GdkDragContext *drag_context,
+					 gint x,gint y, GtkSelectionData *data,
+					 guint info,guint time, gpointer user_data);
 
-
-
+static void filteredit_gui_data_get(GtkWidget *widget, GdkDragContext *drag_context,
+				    GtkSelectionData *data,
+				    guint info, guint time, gpointer user_data);
 
 
 void
@@ -83,32 +87,6 @@ add_filter_by_plugin_cb(GtkWidget*wid, plugin_t *plugin)
 /* 	switch(event->button.button){ */
 /* 	case 1: */
 
-#ifdef HAVE_LIBSTROKE		
-static guint button_down_sighandlerid;
-
-static gboolean 
-button_down_cb(GnomeCanvas * canvas, GdkEvent *event, GlameCanvas* glCanv)
-{
-	char translation[50];
-	switch(event->type){
-	case GDK_BUTTON_RELEASE:
-		if(stroke_trans(translation)){
-			DPRINTF("translation: %s\n",translation);
-		}else{
-			DPRINTF("failed translation: %s\n",translation);
-		}
-		gtk_signal_disconnect(canvas,button_down_sighandlerid);
-		break;
-	case GDK_MOTION_NOTIFY:
-		stroke_record(event->button.x, event->button.y);
-		break;
-	default:
-		break;
-	}
-	return FALSE;
-}
-#endif
-		
 #if 0
 static void group_all(GlameCanvas* canv)
 {
@@ -211,7 +189,7 @@ root_event(GnomeCanvas * canvas, GdkEvent *event, GlameCanvas* glCanv)
 			if(bMac){
 				if(!onItem){
 					menu = GTK_WIDGET(glame_gui_build_plugin_menu(NULL, add_filter_by_plugin_cb));
-					gnome_popup_menu_do_popup(menu,NULL,NULL,&event->button,NULL);
+					gnome_popup_menu_do_popup(menu,NULL,NULL,&event->button,NULL,GTK_WIDGET(glcanvas));
 					return TRUE;
 				}
 			}else{
@@ -240,11 +218,6 @@ root_event(GnomeCanvas * canvas, GdkEvent *event, GlameCanvas* glCanv)
 			}
 			break;
 		case 2:
-			if(!onItem){
-#ifdef HAVE_LIBSTROKE			  
-			button_down_sighandlerid = gtk_signal_connect(GTK_OBJECT(canvas),"event",GTK_SIGNAL_FUNC(button_down_cb),glCanv);
-#endif
-			}
 			break;
 		case 3:
 			if(!onItem){
@@ -253,10 +226,10 @@ root_event(GnomeCanvas * canvas, GdkEvent *event, GlameCanvas* glCanv)
 				if (glCanv->clipBoard) {
 					edit = gtk_menu_item_new_with_label(_("_Paste"));
 					gtk_widget_show(edit);
-					gtk_signal_connect(GTK_OBJECT(edit),"activate",filtereditgui_paste_cb,glCanv);
+					gtk_signal_connect(GTK_OBJECT(edit),"activate",GTK_SIGNAL_FUNC(filtereditgui_paste_cb),glCanv);
 					gtk_menu_append(GTK_MENU(menu), edit);
 				}
-				gnome_popup_menu_do_popup(menu,NULL,NULL,&event->button,NULL);
+				gnome_popup_menu_do_popup(menu,NULL,NULL,&event->button,NULL,GTK_WIDGET(glCanv));
 				return TRUE;
 			}
 			break;
@@ -308,7 +281,7 @@ static void redirection_list_port_cb(GtkCList *list, gint row, gint column,
 	port = g_list_nth_data(reds,row);
 	gtk_clist_unselect_row(list,row,0);
 	if(port)
-		if(gnome_popup_menu_do_popup_modal(menu,NULL,NULL,&event->button,port)>=0)
+		if(gnome_popup_menu_do_popup_modal(menu,NULL,NULL,&event->button,port,GTK_WIDGET(CANVAS_ITEM_CANVAS(port)))>=0)
 			gtk_clist_remove(list,row);
 
 }
@@ -321,7 +294,7 @@ static void redirection_list_param_cb(GtkCList *list, gint row, gint column,
 	foo = g_list_nth_data(reds,row);
 	gtk_clist_unselect_row(list,row,0);
 	if(foo)
-		if(gnome_popup_menu_do_popup_modal(menu,NULL,NULL,&event->button,foo)>=0)
+		if(gnome_popup_menu_do_popup_modal(menu,NULL,NULL,&event->button,foo,NULL)>=0)
 			gtk_clist_remove(list,row);
 
 }
@@ -383,7 +356,7 @@ void glame_canvas_property_dialog_cb(GtkObject* foo, FiltereditGui *window)
 		gtk_clist_append(list,line);
 	}
 	//if(!canvas->net->openedUp)
-		gtk_signal_connect(GTK_OBJECT(list),"select-row",redirection_list_port_cb,redPorts);
+		gtk_signal_connect(GTK_OBJECT(list),"select-row",GTK_SIGNAL_FUNC(redirection_list_port_cb),redPorts);
 	gtk_widget_show(GTK_WIDGET(list));
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(list),tablabel);
 
@@ -419,7 +392,7 @@ void glame_canvas_property_dialog_cb(GtkObject* foo, FiltereditGui *window)
 		gtk_clist_append(list,line);
 	}
 	gtk_widget_show(GTK_WIDGET(list));
-	gtk_signal_connect(GTK_OBJECT(list),"select-row",redirection_list_param_cb,redParms);
+	gtk_signal_connect(GTK_OBJECT(list),"select-row",GTK_SIGNAL_FUNC(redirection_list_param_cb),redParms);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(list),tablabel);
 	
 	gtk_widget_show(notebook);
@@ -607,11 +580,15 @@ static void filteredit_gui_destroy(GtkObject *filteredit)
 	GTK_OBJECT_CLASS(parent_class)->destroy(filteredit);
 }
 
-static void filteredit_gui_class_init(FiltereditGuiClass *class)
+static void filteredit_gui_class_init(FiltereditGuiClass *klass)
 {
 	GtkObjectClass *object_class;
-	object_class = GTK_OBJECT_CLASS(class);
+	GtkWidgetClass *widget_class;
+	object_class = GTK_OBJECT_CLASS(klass);
 	object_class->destroy = filteredit_gui_destroy;
+	widget_class = GTK_WIDGET_CLASS(klass);
+	widget_class->drag_data_received = filteredit_gui_data_received;
+	widget_class->drag_data_get = filteredit_gui_data_get;
 }
 
 static void filteredit_gui_init(FiltereditGui *filteredit)
@@ -635,13 +612,11 @@ GtkType filteredit_gui_get_type(void)
 			NULL,NULL,(GtkClassInitFunc)NULL,};
 		filteredit_gui_type = gtk_type_unique(
 			gnome_app_get_type(), &filteredit_gui_info);
-		gtk_type_set_chunk_alloc(filteredit_gui_type, 8);
 	}
 
 	return filteredit_gui_type;
 }	
 
-
 static void glame_canvas_copy_selected_w_cb(GtkWidget *w, FiltereditGui *gui)
 {
 	glame_accel_widget_data_cb(NULL, "filteredit/y");
@@ -664,7 +639,7 @@ static void glame_canvas_collapse_selected_w_cb(GtkWidget *w, FiltereditGui *gui
 }
 static void help_cb(GtkWidget *menu, void *blah)
 {
-	gnome_help_goto(NULL, "info:glame#The_Filternetwork_Editor");
+	glame_help_goto(NULL, "info:glame#The_Filternetwork_Editor");
 }
 static void list_keybindings_cb(GtkWidget *menu, void *blah)
 {
@@ -729,12 +704,16 @@ static GnomeUIInfo window_menu[] = {
 
 
 GtkWidget * 
-glame_filtereditgui_new(filter_t *net, gboolean protected)
+glame_filtereditgui_new(filter_t *net, gboolean prot)
 {
 	FiltereditGui *window;
 	GtkWidget *canvas, *sw, *toolbar;
-	GnomeDock *dock;
+	BonoboDock *dock;
 	const char *name;
+	GtkTargetEntry desttargets[] = { GLAME_DND_TARGET_STRING_SCM, 
+					 GLAME_DND_TARGET_STRING_SCM_NETWORK,
+					 GLAME_DND_TARGET_POINTER_FILTER_T,
+					 };
 	
 	if(net && filter_name(net))
 		name = filter_name(net);
@@ -745,11 +724,11 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 	gnome_app_construct(GNOME_APP(window), "glame0.7", name);
 
 	glame_filtereditgui_install_accels(GTK_WIDGET(window));
-	dock = GNOME_DOCK(GNOME_APP(window)->dock);
+	dock = BONOBO_DOCK(GNOME_APP(window)->dock);
 	gtk_widget_ref(GTK_WIDGET(dock));
 	
 	gtk_widget_show(GTK_WIDGET(dock));
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_BOTH);
+	toolbar = gtk_toolbar_new();
 	window->toolbar = GTK_TOOLBAR(toolbar);
 
 	sw = gtk_scrolled_window_new(NULL,NULL);
@@ -764,41 +743,36 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 	gtk_widget_pop_visual();
 	gtk_widget_pop_colormap();
 //	GNOME_CANVAS(canvas)->aa = 1;
-	GLAME_CANVAS(canvas)->openedUp = protected;
+	GLAME_CANVAS(canvas)->openedUp = prot;
 	//gnome_canvas_set_scroll_region(GNOME_CANVAS(canvas),0,0,600,400);
 	
 
 	gtk_container_add(GTK_CONTAINER(sw),canvas);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_EXECUTE,_("Execute"),_("Executes Filternetwork"),GTK_SIGNAL_FUNC(glame_canvas_execute_cb),window,-1);
+
+	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_CONVERT,_("Register"),_("Registers actual filternetwork"),GTK_SIGNAL_FUNC(glame_canvas_register_cb),window,-1);
 	
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Execute"),_("Executes Filternetwork"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_EXEC),
-		glame_canvas_execute_cb, window);
-	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Register"),_("Registers actual filternetwork"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_CONVERT),
-		glame_canvas_register_cb, window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
+
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Save"),_("Saves Filternetwork"),"foo",
 				glame_load_icon_widget("save.png",24,24),
-				glame_canvas_save_as_cb, window);
+				GTK_SIGNAL_FUNC(glame_canvas_save_as_cb),window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Properties"),_("Edit Filternetwork Properties"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_PROPERTIES),
-		glame_canvas_property_dialog_cb, window);
-	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Zoom in"),_("Zooms in"),"foo",
-				glame_load_icon_widget("zoom_in.png",24,24),
-				glame_canvas_zoom_in_cb, window);
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Zoom out"),_("Zooms out"),"foo",
-				glame_load_icon_widget("zoom_out.png",24,24),
-				glame_canvas_zoom_out_cb, window);
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("View all"),_("Adjusts scroll region"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_REFRESH),glame_canvas_view_all_cb, window);
-	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Add last"),_("Adds last filter"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_MULTIPLE),
-				glame_canvas_add_last_cb, window);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_PROPERTIES,_("Properties"),_("Edit Filternetwork Properties"),GTK_SIGNAL_FUNC(glame_canvas_property_dialog_cb),window,-1);
 
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Close"),_("Close"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_CLOSE),window_close,window);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_ZOOM_IN,_("Zoom in"),_("Adjust scroll region"),GTK_SIGNAL_FUNC(glame_canvas_zoom_in_cb),window,-1);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_ZOOM_OUT,_("Zoom out"),_("Adjust scroll region"),GTK_SIGNAL_FUNC(glame_canvas_zoom_out_cb),window,-1);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_ZOOM_FIT,_("View all"),_("Adjust scroll region"),GTK_SIGNAL_FUNC(glame_canvas_view_all_cb),window,-1);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Help"),_("Help"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_HELP),gnome_help_goto,"info:glame#The_Filternetwork_Editor");
+	
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_COPY,_("Add last"),_("Adds last filter"),GTK_SIGNAL_FUNC(glame_canvas_add_last_cb), window,-1);
+
+	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_CLOSE,_("Close"),_("Close"),GTK_SIGNAL_FUNC(window_close),window,-1);
+	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_HELP,_("Help"),_("Help"),GTK_SIGNAL_FUNC(glame_help_goto),"info:glame#The_Filternetwork_Editor",-1);
 
         /* Create menubar - FIXME copy all uiinfos, restructure to
 	 * match nice menu layout, etc.
@@ -810,8 +784,8 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 #if 0
 	gnome_app_add_toolbar(GNOME_APP(window), GTK_TOOLBAR(toolbar),
 			      "canvas::toolbar",
-			      GNOME_DOCK_ITEM_BEH_EXCLUSIVE|GNOME_DOCK_ITEM_BEH_NEVER_FLOATING,
-			      GNOME_DOCK_TOP, 0, 0, 0);
+			      BONOBO_DOCK_ITEM_BEH_EXCLUSIVE|BONOBO_DOCK_ITEM_BEH_NEVER_FLOATING,
+			      BONOBO_DOCK_TOP, 0, 0, 0);
 #endif
 
 	gnome_app_set_contents(GNOME_APP(window),sw);	
@@ -820,9 +794,12 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 	gtk_window_set_default_size(GTK_WINDOW(window),400,300);
 	gtk_widget_show_all(GTK_WIDGET(window));
 	gtk_signal_connect(GTK_OBJECT(canvas),"event",GTK_SIGNAL_FUNC(root_event),canvas);
-#ifdef HAVE_LIBSTROKE
-	stroke_init();
-#endif
+
+	
+	gtk_drag_source_set(GTK_WIDGET(window),GDK_BUTTON2_MASK,desttargets,3,GDK_ACTION_MOVE);
+	gtk_drag_dest_set(GTK_WIDGET(window),GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP, desttargets,3,GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_DEFAULT);
+
+
 	return GTK_WIDGET(window);
 }
 
@@ -833,26 +810,31 @@ glame_load_network(GtkWidget *foo, gpointer bla)
 {
 	GtkWidget *dialog;
 	filter_t *filter;
-	char filenamebuffer[256];
+	char *filename;
 
-	filenamebuffer[0] = '\0';
-	dialog = glame_dialog_file_request(_("Load filternetwork"),
-					   "editfilter:load", _("Filename"),
-					   NULL, filenamebuffer);
-	if (!gnome_dialog_run_and_close(GNOME_DIALOG(dialog))
-	    || !filenamebuffer[0])
+	dialog = gtk_file_chooser_dialog_new(
+		_("Load filternetwork"), NULL,
+		GTK_FILE_CHOOSER_ACTION_OPEN,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+	if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_ACCEPT) {
+		gtk_widget_destroy(dialog);
 		return;
-	
-	filter = glame_load_instance(filenamebuffer);
+	}
+	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+
+	filter = glame_load_instance(filename);
 	if (filter) {
 		GtkWidget *feg;
 		feg = glame_filtereditgui_new(filter, FALSE);
 		gtk_quit_add_destroy(1, GTK_OBJECT(feg));
 		gtk_widget_show(feg);
 	} else {
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("Error in loading network\nCheck out the glame-console output for more information"))));
+		glame_error_dialog(_("Error in loading network\nCheck out the glame-console output for more information"), NULL);
 	} 
+
+	g_free(filename);
+	gtk_widget_destroy(dialog);
 }
 
 static void execute_cleanup(glsig_handler_t *handler, long sig, va_list va)
@@ -888,10 +870,10 @@ static void execute_cleanup(glsig_handler_t *handler, long sig, va_list va)
 
 	gtk_widget_destroy(g_list_nth(gtk_container_children(
 		GTK_CONTAINER(gui->toolbar)), 0)->data);
-	gtk_toolbar_insert_item(GTK_TOOLBAR(gui->toolbar),
-				_("Execute"), _("Executes Filternetwork"), _("Execute"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_EXEC),
-				glame_canvas_execute_cb, gui, 0);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(gui->toolbar),
+				_("Execute"), _("Executes Filternetwork"),
+				GNOME_STOCK_PIXMAP_EXEC,
+				GTK_SIGNAL_FUNC(glame_canvas_execute_cb), gui, 0);
 }
 
 static void glame_canvas_execute_cb(GtkObject* foo, FiltereditGui *gui)
@@ -924,10 +906,10 @@ static void glame_canvas_execute_cb(GtkObject* foo, FiltereditGui *gui)
 
 	gtk_widget_destroy(g_list_nth(gtk_container_children(
 		GTK_CONTAINER(gui->toolbar)), 0)->data);
-	gtk_toolbar_insert_item(GTK_TOOLBAR(gui->toolbar),
-				_("Stop"), _("Stop"), _("Stop"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_STOP),
-				glame_canvas_execute_cb, gui, 0);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(gui->toolbar),
+				 _("Stop"), _("Stop"), 
+				 GNOME_STOCK_PIXMAP_STOP,
+				 GTK_SIGNAL_FUNC(glame_canvas_execute_cb), gui, 0);
 }
 
 static void glame_canvas_save_as_cb(GtkWidget*ignore, FiltereditGui *window)
@@ -935,7 +917,7 @@ static void glame_canvas_save_as_cb(GtkWidget*ignore, FiltereditGui *window)
 	GlameCanvas *canvas = window->canvas;
 	GtkWidget *dialog;
 	GtkWidget *dialogVbox;
-	char filenamebuffer[256] = "";
+	char *filename;
 	char filternamebuffer[256] = "unnamed";
 	char categorynamebuffer[256] = "unnamed";
 	char *buffer;
@@ -943,27 +925,33 @@ static void glame_canvas_save_as_cb(GtkWidget*ignore, FiltereditGui *window)
 
 	/* Open a file request dialog with additional fields for
 	 * filter name and category. */
-	filenamebuffer[0] = '\0';
-	dialog = glame_dialog_file_request(_("Save network as..."),
-					   "editfilter:saveas", _("Filename"),
-					   NULL, filenamebuffer);
-	dialogVbox = GTK_WIDGET(GTK_VBOX(GNOME_DIALOG(dialog)->vbox));
+	dialog = gtk_file_chooser_dialog_new(
+		_("Save network as..."), NULL,
+		GTK_FILE_CHOOSER_ACTION_SAVE,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
+	dialogVbox = GTK_DIALOG(dialog)->vbox;
 	create_label_edit_pair(dialogVbox, _("Filter name"),
 			       "editfilter:saveas:name", filternamebuffer);
 	create_label_edit_pair(dialogVbox, _("Category"),
 			       "editfilter:saveas:category", categorynamebuffer);
-	if (!gnome_dialog_run_and_close(GNOME_DIALOG(dialog)))
+	gtk_widget_show_all(dialog);
+	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy(dialog);
 		return;
+	}
+	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+	gtk_widget_destroy(dialog);
 
-	if (!filenamebuffer[0]
+	if (!filename
 	    || !filternamebuffer[0]
 	    || !categorynamebuffer[0]) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("Empty file/filter or category name"))));
+		glame_error_dialog(_("Empty file/filter or category name"), NULL);
 		return;
 	}
 
-	outf = fopen(filenamebuffer,"w");
+	outf = fopen(filename,"w");
+	g_free(filename);
 	buffer = filter_to_string(GLAME_CANVAS(canvas)->net);
 	DPRINTF("Network .scm is:\n%s\n", buffer);
 	fprintf(outf, "(let ((newplugin (glame_plugin_define %s \"%s\")\n)) (if (filter? newplugin) newplugin (plugin-set! newplugin PLUGIN_CATEGORY \"%s\")))", buffer, filternamebuffer, categorynamebuffer);
@@ -1020,3 +1008,30 @@ static void window_close(GtkWidget *dummy, GtkWidget* window)
 		filter_terminate(FILTEREDIT_GUI(window)->canvas->net->launch_context);
 	gtk_widget_destroy(GTK_WIDGET(window));
 }
+
+static void 
+filteredit_gui_data_received(GtkWidget *widget,
+                                            GdkDragContext *drag_context,
+                                            gint x,
+                                            gint y,
+                                            GtkSelectionData *data,
+                                            guint info,
+                                            guint time,
+                                            gpointer user_data)
+{
+	
+	fprintf(stderr,"data received: %s %d\n",data->data,info);
+}
+
+static void 
+filteredit_gui_data_get      (GtkWidget *widget,
+                                            GdkDragContext *drag_context,
+                                            GtkSelectionData *data,
+                                            guint info,
+                                            guint time,
+                                            gpointer user_data)
+{
+	char * bla=strdup("ficken\0");
+	gtk_selection_data_set(data,data->target,8,bla,strlen(bla)+1);
+	fprintf(stderr,"data get %d\n",info);
+}
