@@ -4,9 +4,9 @@
 /*
  * glame_console.h
  *
- * $Id: glame_console.h,v 1.1 2001/06/11 08:44:12 richi Exp $
+ * $Id: glame_console.h,v 1.3 2003/05/26 18:21:51 richi Exp $
  *
- * Copyright (C) 2001 Richard Guenther
+ * Copyright (C) 2001-2003 Richard Guenther
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,9 @@
 #include <config.h>
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* Init the console and create the widget, but do not show it.
  * Returns 0 on success, -1 on error. */
@@ -44,5 +47,8 @@ void glame_console_show();
  * number of characters written. */
 int glame_console_printf(const char *format, ...);
 
+#ifdef __cplusplus
+}
+#endif
 
 #endif
