@@ -3,7 +3,7 @@
 
 /*
  * importexport.h
- * $Id: importexport.h,v 1.2.4.1 2004/10/23 13:14:20 richi Exp $
+ * $Id: importexport.h,v 1.4 2004/10/23 13:09:24 richi Exp $
  *
  * Copyright (C) 2001, 2002 Alexander Ehlert
  *
@@ -29,10 +29,12 @@
 #include <gtk/gtk.h>
 #include "gpsm.h"
 
+G_BEGIN_DECLS
 
 gpsm_item_t *glame_import_dialog(GtkWindow *parent);
 
 GnomeDialog *glame_export_dialog(gpsm_item_t *item, GtkWindow *parent);
 
+G_END_DECLS
 
 #endif
