@@ -1,5 +1,5 @@
 /*
- * $Id: gtkswapfilebuffer.h,v 1.9.6.1 2004/10/23 13:14:21 richi Exp $
+ * $Id: gtkswapfilebuffer.h,v 1.12 2004/10/23 13:09:25 richi Exp $
  *
  * Copyright (c) 2000, 2001 Richard Guenther
  *
@@ -72,9 +72,15 @@ struct _GtkSwapfileBufferClass
 GtkType      gtk_swapfile_buffer_get_type (void);
 GtkObject   *gtk_swapfile_buffer_new(gpsm_grp_t *item);
 
+/* Returns the original item the swapfile buffer was created with. */
 gpsm_grp_t  *gtk_swapfile_buffer_get_item(GtkSwapfileBuffer *buffer);
-int          gtk_swapfile_buffer_get_swfiles(GtkSwapfileBuffer *buffer,
-					     gpsm_swfile_t ***files);
+
+/* Returns the active tracks in a gpsm group with properties like the
+ * one returned by gpsm_collect_swfiles(). Vertical positions are
+ * corresponding to the track number.
+ * You need to free the returned group by yourself. */
+gpsm_grp_t  *gtk_swapfile_buffer_get_active(GtkSwapfileBuffer *buffer,
+					    guint32            sel_mask);
 
 
 #ifdef __cplusplus
