@@ -28,6 +28,7 @@
 
 #include <string.h>
 #include <gtk/gtk.h>
+#include <math.h>
 #include "util/glame_hruler.h"
 #include "gtkwaveview.h"
 #include "gtkeditablewavebuffer.h"
@@ -37,12 +38,13 @@
 /* static xpms for rec/mute buttons */
 #include "rec.xpm"
 #include "mute.xpm"
+#include "sel.xpm"
 
 
 /* Stuff for new ruler.
  */
 
-static gchar *time_metric_translate(gdouble value)
+static gint time_metric_translate(gdouble value, gchar *buf, gint buf_size)
 {
   gint minutes;
   gdouble seconds;
@@ -50,7 +52,7 @@ static gchar *time_metric_translate(gdouble value)
   minutes = ((gint) (value / 60.0));
   seconds = value - (((gint)value) / 60) * 60;
 
-  return g_strdup_printf ("%02d:%02.3f", minutes, seconds);
+  return snprintf (buf, buf_size, "%02d:%02.3f", minutes, seconds);
 }
 
 static const GlameRulerMetric time_metric = {
@@ -63,9 +65,9 @@ static const GlameRulerMetric time_metric = {
   time_metric_translate
 };
 
-static gchar *frame_metric_translate (gdouble value)
+static gint frame_metric_translate (gdouble value, gchar *buf, gint buf_size)
 {
-  return g_strdup_printf ("%li", (long) value);
+  return snprintf (buf, buf_size, "%li", (long) value);
 }
 
 static const GlameRulerMetric frame_metric = {
@@ -152,9 +154,13 @@ gtk_wave_view_new (void)
 {
   GtkWaveView *waveview;
   
-  waveview = gtk_type_new (GTK_TYPE_WAVEFORM);
+  waveview = (GtkWaveView *)gtk_type_new (GTK_TYPE_WAVEFORM);
   waveview->drawing = 0;
   waveview->destroyed = 0;
+  waveview->drag_cursor = NULL;
+  waveview->normal_cursor = NULL;
+
+  waveview->data = NULL;
 
   return GTK_WIDGET (waveview);
 }
@@ -175,11 +181,22 @@ gtk_wave_view_real_destroy (GtkObject *obj)
   if (waveview->wavebuffer != NULL)
     gtk_wave_view_set_buffer (waveview, NULL);
 
-  gdk_cursor_destroy (waveview->drag_cursor);
-  gdk_cursor_destroy (waveview->normal_cursor);
+  if (waveview->drag_cursor) {
+    gdk_cursor_destroy (waveview->drag_cursor);
+    waveview->drag_cursor = NULL;
+  }
+  if (waveview->normal_cursor) {
+    gdk_cursor_destroy (waveview->normal_cursor);
+    waveview->normal_cursor = NULL;
+  }
 
-  if (waveview->marker_gc != NULL)
+  if (waveview->marker_gc != NULL) {
     gdk_gc_unref (waveview->marker_gc);
+    waveview->marker_gc = NULL;
+  }
+
+  if (waveview->data != NULL)
+    g_free(waveview->data);
 
   GTK_OBJECT_CLASS (parent_class)->destroy (obj); 
 }
@@ -190,20 +207,20 @@ gtk_wave_view_class_init (GtkWaveViewClass *klass)
 {
   GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
 
-  parent_class = gtk_type_class (gtk_vbox_get_type ());
+  parent_class = (GtkVBoxClass *)gtk_type_class (gtk_vbox_get_type ());
   object_class->destroy = gtk_wave_view_real_destroy;
 }
 
 
 /* Unit conversion functions */
 
-static guint32
-calc_scrn_y (gint32 top, gint32 height, gint16 mag, gdouble ampl_zoom)
+static inline guint32
+calc_scrn_y (gint32 top, gint32 height, gint16 mag, gfloat ampl_zoom)
 {
   gint32 pos;
 
-  pos = (guint32) (top + (height * 0.5) + \
-                  (height * mag * ampl_zoom) / 65535.0);
+  pos = (guint32) (top + (height * 0.5f) + \
+                  (height * mag * ampl_zoom) / 65535.0f);
 
   pos = MIN (MAX (pos, top), top + height - 1);
 
@@ -211,44 +228,44 @@ calc_scrn_y (gint32 top, gint32 height, gint16 mag, gdouble ampl_zoom)
 }
 
 
-static gint32
+static inline gint32
 calc_win_pel_pos (GtkWaveView *waveview, gint32 frame_pos)
 {
-  return ((gint32) (frame_pos / waveview->zoom)) - ((gint32) (GTK_ADJUSTMENT (waveview->adjust)->value / waveview->zoom));
+  return ((gint32) (frame_pos / waveview->zoom)) - ((gint32) (waveview->adjust->value / waveview->zoom));
 }
 
 
-static gint32
+static inline gint32
 calc_ext_pel_pos (GtkWaveView *waveview, gint32 frame_pos)
 {
   return (gint32) (frame_pos / waveview->zoom);
 }
 
 
-static gint32
+static inline gint32
 calc_win_pel_ext (GtkWaveView *waveview, gint32 ext_pel_pos)
 {
-  return ext_pel_pos - ((gint32)(GTK_ADJUSTMENT (waveview->adjust)->value / waveview->zoom));
+  return ext_pel_pos - ((gint32)(waveview->adjust->value / waveview->zoom));
 }
 
-static gint32
+static inline gint32
 calc_ext_pel_win (GtkWaveView *waveview, gint32 win_pel_pos)
 {
-  return win_pel_pos + ((gint32)(GTK_ADJUSTMENT (waveview->adjust)->value / waveview->zoom));
+  return win_pel_pos + ((gint32)(waveview->adjust->value / waveview->zoom));
 }
 
 #if 0 /* unused */
-static gint32
+static inline gint32
 calc_frame_pos_ext (GtkWaveView *waveview, gint32 ext_pel_pos)
 {
   return (gint32) (ext_pel_pos * waveview->zoom);
 }
 #endif
 
-static gint32
+static inline gint32
 calc_frame_pos_win (GtkWaveView *waveview, gint32 win_pel_pos)
 {
-	return (((gint32) GTK_ADJUSTMENT (waveview->adjust)->value / waveview->zoom) + win_pel_pos) * waveview->zoom;
+	return (((gint32) waveview->adjust->value / waveview->zoom) + win_pel_pos) * waveview->zoom;
 }
 
 
@@ -256,7 +273,7 @@ calc_frame_pos_win (GtkWaveView *waveview, gint32 win_pel_pos)
 
 static void gtk_wave_view_cache_free (GtkWaveView *waveview);
 static void gtk_wave_view_cache_create (GtkWaveView *waveview);
-static void gtk_wave_view_cache_add (GtkWaveView *waveview, guint32 x, gint16 *min, gint16 *max);
+static void gtk_wave_view_cache_add (GtkWaveView *waveview, guint32 x, gfloat *min, gfloat *max);
 static gint gtk_wave_view_cache_paint (GtkWaveView *waveview, gint32 offset, gint32 x);
 static void gtk_wave_view_cache_invalidate (GtkWaveView *waveview);
 static void gtk_wave_view_cache_invalidate_range (GtkWaveView *waveview, gint32 start, gint32 length);
@@ -296,7 +313,8 @@ gtk_wave_view_cache_create (GtkWaveView *waveview)
 
 
 static void
-gtk_wave_view_cache_add (GtkWaveView *waveview, guint32 x, gint16 *min, gint16 *max)
+gtk_wave_view_cache_add (GtkWaveView *waveview, guint32 x,
+			 gfloat *min, gfloat *max)
 {
   guint32 i, size, offset;
   GtkWaveViewCacheEntry *e;
@@ -310,10 +328,10 @@ gtk_wave_view_cache_add (GtkWaveView *waveview, guint32 x, gint16 *min, gint16 *
   for (i = 0; i < waveview->n_channels; i++)
     {
       e = &waveview->channels [i].cache [offset];
-      e->min = min [i];
-      e->max = max [i];
+      e->min = float_to_s16(min[i]);
+      e->max = float_to_s16(max[i]);
 
-      min [i] = max [i] = 0;
+      min[i] = max[i] = 0.0f;
     }
 }
 
@@ -407,7 +425,7 @@ gtk_wave_view_update_units (GtkWaveView *waveview)
   guint32 width;
   gint32 j;
 
-  adj = GTK_ADJUSTMENT (waveview->adjust);
+  adj = waveview->adjust;
 
   if (waveview->wavebuffer == NULL)
     {
@@ -475,9 +493,9 @@ gtk_wave_view_update_label(GtkWaveView *waveview)
 		 	 waveview->select_left/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer) - mins*60,
 			 mins2,
 		 	 waveview->select_right/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer) - mins2*60);
+		gtk_label_set_text(GTK_LABEL(waveview->selection_label), label);
 	} else
-		strcpy(label, "none");
-	gtk_label_set_text(GTK_LABEL(waveview->selection_label), label);
+		gtk_label_set_text(GTK_LABEL(waveview->selection_label), "none");
 }
 
 
@@ -490,6 +508,10 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
   guint32   i, j, width, start_x, offset;
   gint16    max_val, min_val;
   gboolean  exact;
+  gfloat    max [waveview->n_channels], min [waveview->n_channels];
+  guint32   sample_offset, last_sample_offset, pos, size;
+  guint32   accum, count, oldcount, delta;
+  gfloat    *data;
 
   if (waveview->wavebuffer == NULL)
     return;
@@ -509,9 +531,6 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
   width = MIN(waveview->area->allocation.width, waveview->expose_width);
   start_x = MAX(0, calc_win_pel_ext(waveview, waveview->expose_x));
 
-  /* Hide marker. */
-  gtk_wave_view_draw_marker (waveview);
-
   /* First, paint all cached x coords. */
   /* Keep a range min_val -> max_val that contains all uncached x coords. */
   for (i = start_x; i < start_x + width; i++)
@@ -521,56 +540,42 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
       else
         max_val = i;
     }
+  if (min_val == -1)
+    goto out;
 
-  if (min_val != -1)
-    {
-      gint16    max [waveview->n_channels], min [waveview->n_channels];
-      guint32   sample_offset, last_sample_offset, pos, size;
-      guint32   accum, count, delta;
-      guint32   pos16;
-      gpointer  data;
-      gint16   *data16;
+  /* Now here comes the fun part, if some x coordinates weren't cached,
+   * we gotta do a lot of calculations to figure out their values. */
+  n_channels = waveview->n_channels;
 
-      /* Now here comes the fun part, if some x coordinates weren't cached, */
-      /* we gotta do a lot of calculations to figure out their values.      */
-      n_channels = waveview->n_channels;
-
-      datatype = gtk_wave_buffer_get_datatype (waveview->wavebuffer);
+  /* Datatype will be F4NI - asserted by gtk_wave_view_add_buffer. */
+  datatype = gtk_wave_buffer_get_datatype (waveview->wavebuffer);
 
 #define REDRAW_BUFFER 1024
-      /* Allocate some temp space. */
-      data = g_malloc (g_wavefile_type_width (datatype) * n_channels * REDRAW_BUFFER);
+  /* Temporary space is in waveview. */
+  data = waveview->data;
 
-      /* Allocate more temp space if the data source datatype is not s16.
-         We then have to convert it to s16 data for cache use. */
-      if (datatype != G_WAVEFILE_TYPE_S16)
-        data16 = g_new (gint16, n_channels * REDRAW_BUFFER);
-      else
-        data16 = (gint16*) data;
+  /* Approximate sample # range we need to recalc for the cache. */
+  sample_offset = calc_frame_pos_win (waveview, min_val);
+  last_sample_offset = calc_frame_pos_win (waveview, max_val + 1);
 
-      /* Approximate sample # range we need to recalc for the cache. */
-      sample_offset = calc_frame_pos_win (waveview, min_val);
-      last_sample_offset = calc_frame_pos_win (waveview, max_val + 1);
+  n_samples = gtk_wave_buffer_get_length (waveview->wavebuffer);
 
-      n_samples = gtk_wave_buffer_get_length (waveview->wavebuffer);
+  /* Perform clipping. */
+  if (sample_offset >= n_samples)
+	  goto out;
+  if (last_sample_offset >= n_samples)
+	  last_sample_offset = n_samples - 1;
 
-      /* Perform clipping. */
-      if (sample_offset >= n_samples)
-        goto out;
-      if (last_sample_offset >= n_samples)
-        last_sample_offset = n_samples - 1;
+  for (j = 0; j < waveview->n_channels; j++)
+	  min[j] = max[j] = 0.0f;
 
-      for (j = 0; j < waveview->n_channels; j++)
-        min[j] = max[j] = 0;
+  pos = sample_offset;
+  accum = 1073741824U; /* 2147483648 / 2, round to nearest unit */
+  oldcount = count = 0;
+  delta = (guint32) (2147483648.0 / waveview->zoom);
+  exact = (waveview->zoom < REDRAW_BUFFER);
 
-      pos = sample_offset;
-      accum = 1073741824U; /* 2147483648 / 2, round to nearest unit */
-      count = 0;
-      delta = (guint32) (2147483648.0 / waveview->zoom);
-      exact = (waveview->zoom < REDRAW_BUFFER);
-
-      while (pos <= last_sample_offset)
-        {
+  while (pos <= last_sample_offset) {
           size = MIN (REDRAW_BUFFER, last_sample_offset - pos + 1);
 
 	  /* If by any chance we got destroyed, bail out. */
@@ -583,17 +588,12 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
              /* Read data chunk. */
 	     gtk_wave_buffer_get_samples (waveview->wavebuffer, pos, size, 0xffffffff, data);
 
-	     /* Convert data to s16 if it's not already in that format. */
-	     if (datatype != G_WAVEFILE_TYPE_S16)
-		g_wavefile_type_convert (waveview->n_channels, size, G_WAVEFILE_TYPE_S16, data16, datatype, data);
-
-	     pos16 = 0;
 	     for (i = 0; i < size; i++) {
 		/* For each sample of a track, keep min and max values. */
 		for (j = 0; j < n_channels; j++) {
-		   gint16 val = data16[pos16++];
+		   gfloat val = data[j*size + i];
 		   if (val < min[j]) min[j] = val;
-		   if (val > max[j]) max[j] = val;
+		   else if (val > max[j]) max[j] = val;
                 }
 
 		/* Do Bresenham's algorithm */
@@ -617,15 +617,10 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 		/* Read data chunk. */
 		gtk_wave_buffer_get_samples (waveview->wavebuffer, pos, size, 0xffffffff, data);
 
-		/* Convert data to s16 if it's not already in that format.  */
-		if (datatype != G_WAVEFILE_TYPE_S16)
-		   g_wavefile_type_convert (waveview->n_channels, size, G_WAVEFILE_TYPE_S16, data16, datatype, data);
-
-		pos16 = 0;
-		for (i = 0; i < size; i++) {
-		   /* For each sample of a track, keep min and max values. */
-		   for (j = 0; j < n_channels; j++) {
-		      gint16 val = abs(data16[pos16++]);
+		/* For each sample of a track, keep min and max values. */
+		for (j = 0; j < n_channels; j++) {
+		   for (i = 0; i < size; i++) {
+		      gfloat val = fabsf(data[j*size+i]);
 		      if (val > max[j]) { 
 			 max[j] = val;
 			 min[j] = -val;
@@ -640,10 +635,12 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 	  }
 
 	  /* Be nice to the user. */
-	  if (gtk_events_pending()) {
+	  if (count > oldcount && gtk_events_pending()) {
 		  /* Handle all pending events => stall redrawing. */
+		  gtk_wave_view_draw_marker (waveview);
 		  while (gtk_events_pending())
 			  gtk_main_iteration();
+		  gtk_wave_view_draw_marker (waveview);
 
 		  /* Correcting the exposed region, bail out
 		   * - only if expose event arrived (else redraw error) */
@@ -659,20 +656,17 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 				  waveview->expose_width = notdone_x + notdone_width - waveview->expose_x;
 			  goto out;
 		  }
+
+		  oldcount = count;
 	  }
 
           /* Increment position in data source. */
           pos += size;
-        }
+  }
 
-    out:
-      if (data16 != (void*) data)
-        g_free (data16);
-      g_free (data);
-    }
 
-  /* Show marker. */
-  gtk_wave_view_draw_marker (waveview);
+ out:
+  ;
 }
 
 
@@ -808,6 +802,9 @@ on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdat
   if (event->count > 0 || waveview->drawing)
     return TRUE;
 
+  /* hide marker */
+  gtk_wave_view_draw_marker (waveview);
+
   waveview->drawing = 1;
  again:
   /* Done accumulating sequential expose events, now process them. */
@@ -830,9 +827,6 @@ on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdat
       frame_area.y = 0;
       frame_area.height = widget->allocation.height;
 
-      /* hide marker */
-      //gtk_wave_view_draw_marker (waveview);
-
       /* Set clipping to expose region. */
       gdk_gc_set_clip_rectangle (widget->style->fg_gc [GTK_STATE_NORMAL], &frame_area);
       gdk_gc_set_clip_rectangle (widget->style->fg_gc [GTK_STATE_SELECTED], &frame_area);
@@ -918,15 +912,15 @@ on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdat
 	      return TRUE;
       }
 
-      /* show marker */
-      //gtk_wave_view_draw_marker (waveview);
-
     }
 
   if (waveview->expose_count != 0)
 	  goto again;
   waveview->drawing = 0;
 
+  /* show marker */
+  gtk_wave_view_draw_marker (waveview);
+
   return TRUE;
 }
 
@@ -975,7 +969,6 @@ gtk_wave_view_scroll (GtkWidget *widget, gpointer data)
 {
   GtkWaveView *waveview = GTK_WAVE_VIEW (data);
   guint32 offset, width, height;
-  guint32 shift;
 
   offset = -calc_win_pel_pos (waveview, 0);
 
@@ -987,34 +980,7 @@ gtk_wave_view_scroll (GtkWidget *widget, gpointer data)
   if (!GTK_WIDGET_REALIZED (waveview->area))
     return;
 
-  if (offset > waveview->drawn_offset)
-    {
-      /* Find out how many pixels we moved. */
-      shift = offset - waveview->drawn_offset;
-
-      /* Copy and redraw as needed. */
-      if (shift >= width)
-        gtk_widget_queue_draw (GTK_WIDGET (waveview->area));
-      else
-        {
-          gdk_window_copy_area (waveview->area->window, waveview->area->style->bg_gc [GTK_STATE_NORMAL], 0, 0, waveview->area->window, shift, 0, width - shift, height);
-          gtk_wave_view_redraw_area (waveview, width - 1, width - shift);
-        }
-    }
-  else
-    {
-      /* Find out how many pixels we moved. */
-      shift = waveview->drawn_offset - offset;
-
-      /* Copy and redraw as needed. */
-      if (shift >= width)
-        gtk_widget_queue_draw (GTK_WIDGET (waveview->area));
-      else
-        {
-          gdk_window_copy_area (waveview->area->window, waveview->area->style->bg_gc [GTK_STATE_NORMAL], shift, 0, waveview->area->window, 0, 0, width - shift, height);
-          gtk_wave_view_redraw_area (waveview, 0, shift);
-        }
-    }
+  gdk_window_scroll (waveview->area->window, waveview->drawn_offset - offset, 0);
 }
 
 /* The meaning of the different flags. */
@@ -1266,7 +1232,7 @@ gtk_wave_view_button_release_event (GtkWidget *widget,
     }
  
   /* dragging outside the border */
-  if (x < 0 && GTK_ADJUSTMENT (waveview->adjust)->value >= 0.0)
+  if (x < 0 && waveview->adjust->value >= 0.0)
       x = 0;
   else if (x >= waveview->area->allocation.width)
       x = waveview->area->allocation.width /* - 1  no, round up this way */;
@@ -1346,7 +1312,7 @@ static gboolean
 gtk_wave_view_scroll_notify (gpointer widget)
 {
   GtkWaveView *waveview = GTK_WAVE_VIEW (widget);
-  GtkAdjustment *adjust = GTK_ADJUSTMENT (waveview->adjust);
+  GtkAdjustment *adjust = waveview->adjust;
   gint32 width, pos, frames;
 
   width = GTK_WIDGET (waveview->area)->allocation.width;
@@ -1402,7 +1368,7 @@ gtk_wave_view_motion_notify_event (GtkWidget *widget,
   if (waveview->wavebuffer == NULL)
     return FALSE;
 
-  GTK_WIDGET_CLASS (GTK_OBJECT (waveview->hruler)->klass)->motion_notify_event (GTK_WIDGET (waveview->hruler), event);
+  GTK_WIDGET_CLASS (GTK_OBJECT_GET_CLASS(GTK_OBJECT (waveview->hruler)))->motion_notify_event (GTK_WIDGET (waveview->hruler), event);
 
   /* Get x coordinate on hint. */
   if (event->is_hint)
@@ -1448,13 +1414,14 @@ gtk_wave_view_init (GtkWaveView *waveview)
   gtk_widget_show (waveview->hruler);
 
   waveview->area = gtk_drawing_area_new ();
+  gtk_widget_set_double_buffered (waveview->area, FALSE);
   gtk_drawing_area_size (GTK_DRAWING_AREA (waveview->area), 150, 100);
   gtk_widget_show (waveview->area);
   gtk_box_pack_start (GTK_BOX (vbox2), waveview->area, TRUE, TRUE, 0);
   gtk_widget_show (waveview->area);
 
-  waveview->adjust = gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 1.0, 5.0);
-  waveview->hscroll = gtk_hscrollbar_new (GTK_ADJUSTMENT (waveview->adjust));
+  waveview->adjust = GTK_ADJUSTMENT(gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 1.0, 5.0));
+  waveview->hscroll = gtk_hscrollbar_new (waveview->adjust);
   gtk_box_pack_start (GTK_BOX (vbox2), waveview->hscroll, FALSE, FALSE, 0);
   gtk_widget_show (waveview->hscroll);
 
@@ -1483,19 +1450,19 @@ gtk_wave_view_init (GtkWaveView *waveview)
 	gtk_box_pack_start(GTK_BOX(waveview), hbox, FALSE, FALSE, 0);
 
   gtk_signal_connect (GTK_OBJECT (waveview->adjust),
-                      "value_changed", gtk_wave_view_scroll,
+                      "value_changed", (GtkSignalFunc)gtk_wave_view_scroll,
                       GTK_OBJECT (waveview));
 
   gtk_signal_connect (GTK_OBJECT (waveview),
-                      "size_allocate", gtk_wave_view_resize_event,
+                      "size_allocate", (GtkSignalFunc)gtk_wave_view_resize_event,
                       GTK_OBJECT (waveview));
 
   gtk_signal_connect (GTK_OBJECT (waveview->area),
-                      "realize", on_area_realize,
+                      "realize", (GtkSignalFunc)on_area_realize,
                       GTK_OBJECT (waveview));
 
   gtk_signal_connect (GTK_OBJECT (waveview->area),
-                      "expose_event", on_area_expose_event,
+                      "expose_event", (GtkSignalFunc)on_area_expose_event,
                       GTK_OBJECT (waveview));
 
   gtk_signal_connect (GTK_OBJECT (waveview->area),
@@ -1503,21 +1470,18 @@ gtk_wave_view_init (GtkWaveView *waveview)
                       GTK_OBJECT (waveview));
 
   gtk_signal_connect (GTK_OBJECT (waveview->area),
-                      "button_release_event", gtk_wave_view_button_release_event,
+                      "button_release_event", (GtkSignalFunc)gtk_wave_view_button_release_event,
                       GTK_OBJECT (waveview));
 
   gtk_signal_connect (GTK_OBJECT (waveview->area),
                       "motion_notify_event", GTK_SIGNAL_FUNC (gtk_wave_view_motion_notify_event),
                       GTK_OBJECT (waveview));
 
-  gtk_widget_set_events (waveview->area, GDK_EXPOSURE_MASK |
-                                         GDK_POINTER_MOTION_MASK |
-                                         GDK_BUTTON_PRESS_MASK |
-                                         GDK_BUTTON_RELEASE_MASK |
-                                         GDK_BUTTON_MOTION_MASK |
-                                         GDK_POINTER_MOTION_HINT_MASK |
-	                                 GDK_KEY_PRESS_MASK |
-			                 GDK_KEY_RELEASE_MASK);
+  gtk_widget_set_events (waveview->area,
+			 GDK_EXPOSURE_MASK |
+			 GDK_POINTER_MOTION_MASK | GDK_BUTTON_MOTION_MASK |
+			 GDK_BUTTON_PRESS_MASK |GDK_BUTTON_RELEASE_MASK |
+			 GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
 
   waveview->wavebuffer = NULL;
 
@@ -1563,7 +1527,7 @@ gtk_wave_view_set_amplitude_zoom (GtkWaveView *waveview, gdouble amplzoom)
   if (waveview->ampl_zoom != amplzoom)
     {
       waveview->ampl_zoom = amplzoom;
-      gtk_widget_queue_draw (GTK_WIDGET (waveview));
+      gtk_widget_queue_draw (GTK_WIDGET (waveview->area));
     }
 }
 
@@ -1698,12 +1662,27 @@ void on_tb_realize_cb(GtkWidget *widget, gpointer data)
 	GdkPixmap *pixmap;
 	GdkBitmap *bitmap;
 	pixmap = gdk_pixmap_create_from_xpm_d(
-		widget->window, &bitmap, NULL, data);
+		widget->window, &bitmap, NULL, (gchar**)data);
 	gtk_container_add(GTK_CONTAINER(widget),
 			  gtk_pixmap_new(pixmap, bitmap));
 	gtk_widget_show_all(widget);
 }
 
+void on_select_tb_toggled_cb(GtkToggleButton *tb, gpointer data)
+{
+	GtkWaveView *waveview = data;
+	int track;
+	guint32 sel_mask;
+
+	track = (int)gtk_object_get_user_data(GTK_OBJECT(tb));
+	sel_mask = gtk_wave_view_get_select_channels(waveview);
+	if (gtk_toggle_button_get_active(tb))
+		sel_mask |= 1<<track;
+	else
+		sel_mask &= ~(1<<track);
+	gtk_wave_view_set_select_channels(waveview, sel_mask);
+}
+
 /* Select a new data stream, ref() it, invalidate cache, and update screen. */
 void
 gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
@@ -1713,6 +1692,8 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
 
   gtk_wave_view_cache_free (waveview);
   g_free (waveview->channels);
+  if (waveview->data)
+    g_free(waveview->data);
 
   if (waveview->wavebuffer != NULL)
     {
@@ -1733,6 +1714,8 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
 
   if (wavebuffer != NULL)
     {
+      GWavefileType datatype;
+
       gtk_object_ref (GTK_OBJECT (waveview->wavebuffer));
       gtk_object_sink (GTK_OBJECT (waveview->wavebuffer));
 
@@ -1766,6 +1749,11 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
       waveview->marker = 0;
 
       gtk_wave_view_cache_create (waveview);
+
+      datatype = gtk_wave_buffer_get_datatype (waveview->wavebuffer);
+      if (datatype != G_WAVEFILE_TYPE_F4NI)
+	DERROR("Can only handle float non-interleaved data\n");
+      waveview->data = g_malloc(g_wavefile_type_width (datatype) * waveview->n_channels * REDRAW_BUFFER);
     }
   else
     {
@@ -1773,14 +1761,15 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
       waveview->cache_tag = NULL;
       waveview->n_channels = 0;
       waveview->marker = -1;
+      waveview->data = NULL;
     }
 
-  gtk_widget_queue_draw (GTK_WAVE_VIEW (waveview)->area);
   gtk_wave_view_update_units (waveview);
   gtk_wave_view_calc_channel_locs (waveview);
+  gtk_widget_queue_draw (GTK_WAVE_VIEW (waveview)->area);
 
   if (wavebuffer) {
-	  GtkWidget *tb, *vbox;
+	  GtkWidget *tb, *hbox;
 	  int cnt;
 #if 0
 	  GList *list;
@@ -1792,41 +1781,35 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
 #endif
 	  cnt = gtk_wave_buffer_get_num_channels(wavebuffer);
 	  while (cnt--) {
-#if 0
-		  GdkPixmap *rec_pixmap, *mute_pixmap;
-		  GdkBitmap *rec_bitmap, *mute_bitmap;
-#endif
-		  vbox = gtk_vbox_new(FALSE, 0);
+		  hbox = gtk_hbox_new(FALSE, 0);
+		  /* rec */
 		  tb = gtk_toggle_button_new();
-#if 0
-		  rec_pixmap = gdk_pixmap_create_from_xpm_d(
-			  GTK_WIDGET(tb)->window, &rec_bitmap, NULL, rec_xpm);
-#endif
 		  gtk_container_set_border_width(GTK_CONTAINER(tb), 0);
-#if 0
-		  gtk_container_add(GTK_CONTAINER(tb),
-				    gtk_pixmap_new(rec_pixmap, rec_bitmap));
-#endif
 		  gtk_object_set(GTK_OBJECT(tb), "can_focus", FALSE, NULL);
-		  gtk_box_pack_start(GTK_BOX(vbox), tb, FALSE, FALSE, 0);
+		  gtk_box_pack_start(GTK_BOX(hbox), tb, FALSE, FALSE, 0);
 		  gtk_signal_connect(GTK_OBJECT(tb), "realize",
-				     on_tb_realize_cb, rec_xpm);
+				     (GtkSignalFunc)on_tb_realize_cb, rec_xpm);
+		  /* mute */
 		  tb = gtk_toggle_button_new();
-#if 0
-		  mute_pixmap = gdk_pixmap_create_from_xpm_d(
-			  GTK_WIDGET(tb)->window, &mute_bitmap, NULL, mute_xpm);
-#endif
 		  gtk_container_set_border_width(GTK_CONTAINER(tb), 0);
-#if 0
-		  gtk_container_add(GTK_CONTAINER(tb),
-				    gtk_pixmap_new(mute_pixmap, mute_bitmap));
-#endif
 		  gtk_object_set(GTK_OBJECT(tb), "can_focus", FALSE, NULL);
-		  gtk_box_pack_start(GTK_BOX(vbox), tb, FALSE, FALSE, 0);
+		  gtk_box_pack_start(GTK_BOX(hbox), tb, FALSE, FALSE, 0);
 		  gtk_signal_connect(GTK_OBJECT(tb), "realize",
-				     on_tb_realize_cb, mute_xpm);
-		  gtk_box_pack_start(GTK_BOX(waveview->vbox1), vbox, TRUE, FALSE, 0);
-		  gtk_widget_show_all(vbox);
+				     (GtkSignalFunc)on_tb_realize_cb, mute_xpm);
+		  /* select */
+		  tb = gtk_toggle_button_new();
+		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tb), TRUE);
+		  gtk_container_set_border_width(GTK_CONTAINER(tb), 0);
+		  gtk_object_set(GTK_OBJECT(tb), "can_focus", FALSE, NULL);
+		  gtk_box_pack_start(GTK_BOX(hbox), tb, FALSE, FALSE, 0);
+		  gtk_signal_connect(GTK_OBJECT(tb), "realize",
+				     (GtkSignalFunc)on_tb_realize_cb, sel_xpm);
+		  gtk_signal_connect(GTK_OBJECT(tb), "toggled",
+				     (GtkSignalFunc)on_select_tb_toggled_cb, waveview);
+		  gtk_object_set_user_data(GTK_OBJECT(tb),
+			(gpointer)(gtk_wave_buffer_get_num_channels(wavebuffer)-cnt-1));
+		  gtk_box_pack_start(GTK_BOX(waveview->vbox1), hbox, TRUE, FALSE, 0);
+		  gtk_widget_show_all(hbox);
 	  }
   }
 }
@@ -1969,13 +1952,10 @@ gtk_wave_view_set_marker_scrolling_boundaries (GtkWaveView *waveview,
 }
 
 static void
-scroll_and_draw_marker (GtkWaveView *waveview, 
-                        gint32  frame, 
-                        gint32  win_width,
-                        gint32  wave_length,
-                        gdouble start_offset)
+scroll (GtkWaveView *waveview, 
+        gint32  frame, gint32  win_width, gint32  wave_length, gdouble start_offset)
 {
-  GtkAdjustment *adj = GTK_ADJUSTMENT (waveview->adjust);
+  GtkAdjustment *adj = waveview->adjust;
   gint32 frame_offset, max_offset;
 
   max_offset = wave_length - (win_width * waveview->zoom) - 1;
@@ -1989,10 +1969,6 @@ scroll_and_draw_marker (GtkWaveView *waveview,
   /* this will do the scrolling for us */
   gtk_adjustment_set_value (adj, frame_offset);
 
-  /* remove the old marker, and draw the new one */
-  draw_marker_from_frame_pos (waveview, waveview->marker);
-  draw_marker_from_frame_pos (waveview, frame);
-
   gtk_wave_view_update_units (waveview);
 }
 
@@ -2002,14 +1978,17 @@ gtk_wave_view_set_marker_and_scroll (GtkWaveView *waveview,
                                      gint32      frame)
 {
   gint32 win_pos_new, width, length;
-  //static int count;
 
   if (check_marker (waveview, &frame, &length, &win_pos_new) < 0)
     {
       /* no action necessary */
-      /* WRONG! we might still need to scroll... return; */
+      /* WRONG! we might still need to scroll... */
+      return;
     }
 
+  /* Hide marker before scrolling.  */
+  draw_marker_from_frame_pos (waveview, waveview->marker);
+
   width = waveview->area->allocation.width;
 
   /* calculate the amount of scroll */
@@ -2018,23 +1997,21 @@ gtk_wave_view_set_marker_and_scroll (GtkWaveView *waveview,
        win_pos_new > width * (1.0 - waveview->marker_scroll_stop))) 
     {
       /* we're moving to the right */
-      scroll_and_draw_marker (waveview, frame, width, length,
-                              waveview->marker_scroll_start);
+      scroll (waveview, frame, width, length,
+              waveview->marker_scroll_start);
     }
   else if (win_pos_new >= width ||
            (frame < waveview->marker && 
             win_pos_new < width * waveview->marker_scroll_stop)) 
     {
       /* we're moving to the left */
-      scroll_and_draw_marker (waveview, frame, width, length,
-                              1.0 - waveview->marker_scroll_start);
-    }
-  else
-    {
-      /* redraw the marker */
-      draw_marker_from_frame_pos (waveview, waveview->marker);
-      draw_marker_from_frame_pos (waveview, frame);
+      scroll (waveview, frame, width, length,
+              1.0 - waveview->marker_scroll_start);
     }
+  /* Else just redraw the marker.  */
+
+  /* Show marker at new position.  */
+  draw_marker_from_frame_pos (waveview, frame);
 
   /* set the new marker */
   waveview->marker = frame;
@@ -2085,7 +2062,7 @@ gtk_wave_view_set_zoom (GtkWaveView *waveview,
 
       gtk_wave_view_cache_invalidate (waveview);
       gtk_wave_view_update_units (waveview);
-      gtk_widget_queue_draw (GTK_WIDGET (waveview));
+      gtk_widget_queue_draw (GTK_WIDGET (waveview->area));
     }
 }
 
@@ -2115,7 +2092,7 @@ gtk_wave_view_set_zoom_selection (GtkWaveView *waveview)
   if (waveview->select_left <= waveview->select_right)
     {
       gdouble zoom;
-      GtkAdjustment *adj = GTK_ADJUSTMENT (waveview->adjust);
+      GtkAdjustment *adj = waveview->adjust;
 
       zoom = ((gdouble) (waveview->select_right - waveview->select_left + 1)) / waveview->area->allocation.width;
 
