@@ -1,10 +1,10 @@
 /*
  * main.c
  *
- * $Id: main.c,v 1.114.2.5 2004/10/23 13:14:20 richi Exp $
+ * $Id: main.c,v 1.134 2005/03/11 17:00:16 richi Exp $
  *
  * Copyright (C) 2000, 2001, 2002, 2003, 2004 Johannes Hirche,
- * 	Richard Guenther
+ *	Richard Guenther
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,23 +26,19 @@
 #include <config.h>
 #endif
 
+#include <gnome.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <gnome.h>
 #ifdef HAVE_LIBGLADE
 #include <glade/glade.h>
 #endif
-#include "swapfile.h"
+#include <libintl.h>
 #include "glmid.h"
-#include "swapfilegui.h"
+#include "tree/gltree.h"
 #include "waveeditgui.h"
-#ifdef ENABLE_EXPERIMENTAL
-#include "timeline/timeline.h"
-#endif
 #include "edit_filter/filtereditgui.h"
 #include "util/glame_gui_utils.h"
 #include "util/gtknob.h"
-#include "gltreeitem.h"
 #include "gpsm.h"
 #include "clipboard.h"
 #include "glame_accelerator.h"
@@ -51,230 +47,251 @@
 #include "glconfig.h"
 #include "importexport.h"
 
+
+/* Stuff we reference/export from/to our C modules.  */
 /* HACK */
-extern void swcluster_set_cache(int, int, int, size_t);
+void swcluster_set_cache(int, int, int, size_t);
 
-
-/* Globals. */
-static GtkWidget *swapfile;
-static GtkWidget *app;
 GtkWidget *glame_appbar;
 
 extern long nPopupTimeout;
 extern long bMac;
 
-/* Forward declarations. */
-static void create_new_project_cb(GtkWidget *menu, void * blah);
-static void edit_file_cb(GtkWidget *menu, void *data);
-static void import_cb(GtkWidget *menu, void *data);
-static void show_console_cb(GtkWidget *menu, void *blah);
-static void emptytrash_cb(GtkWidget *menu, void *blah);
-static void sync_cb(GtkWidget *menu, void *blah);
-static void gui_quit(GtkWidget *widget, gpointer data);
-static int preferences_cb(GtkWidget *menu,void *blah);
-static GtkWidget* glame_about(void);
-/* Menus. */
-static GnomeUIInfo swapfile_menu_uiinfo[] = {
-	GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Project"), N_("Creates a new Project group"), create_new_project_cb, NULL),
-	GNOMEUIINFO_ITEM (N_("Edit File..."), N_("Imports a file and opens the waveedit window"), edit_file_cb, NULL),
-	GNOMEUIINFO_ITEM (N_("Import..."), N_("Imports a file"), import_cb, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM (N_("Empty [deleted]"), N_("Kills [deleted] folder"), emptytrash_cb, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM (N_("Show _console"), N_("Shows the GLAME console"), show_console_cb, NULL),
-	GNOMEUIINFO_ITEM (N_("Sync"), N_("Syncs meta to disk"), sync_cb, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_EXIT_ITEM (gui_quit, NULL),
-	GNOMEUIINFO_END
-};
+/* Globals. */
+static GtkWidget *swapfile;
+static GtkWidget *app;
 
-static void new_network_cb(GtkWidget *menu, void *blah);
-extern void glame_load_network(GtkWidget *bla, void *blu);
+/* Forward declarations. */ 
+ static void create_new_project_cb(GtkWidget *menu, void * blah); 
+ static void edit_file_cb(GtkWidget *menu, void *data); 
+ static void record_wave_cb(GtkWidget *menu, void *data);
+ static void import_cb(GtkWidget *menu, void *data); 
+ static void show_console_cb(GtkWidget *menu, void *blah); 
+ static void emptytrash_cb(GtkWidget *menu, void *blah); 
+ static void sync_cb(GtkWidget *menu, void *blah); 
+ static void gui_quit(GtkWidget *widget, gpointer data); 
+ static int preferences_cb(GtkWidget *menu,void *blah); 
+ static GtkWidget* glame_about(void); 
+/* Menus. */
+ static GnomeUIInfo swapfile_menu_uiinfo[] = { 
+ 	GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Project"), N_("Creates a new Project group"), create_new_project_cb, NULL), 
+ 	GNOMEUIINFO_ITEM (N_("Edit File..."), N_("Imports a file and opens the waveedit window"), edit_file_cb, NULL), 
+	GNOMEUIINFO_ITEM (N_("Record Wave..."), N_("Opens a new waveedit window for recording"), record_wave_cb, NULL),
+ 	GNOMEUIINFO_ITEM (N_("Import..."), N_("Imports a file"), import_cb, NULL), 
+ 	GNOMEUIINFO_SEPARATOR, 
+ 	GNOMEUIINFO_ITEM (N_("Empty [deleted]"), N_("Kills [deleted] folder"), emptytrash_cb, NULL), 
+ 	GNOMEUIINFO_SEPARATOR, 
+ 	GNOMEUIINFO_ITEM (N_("Show _console"), N_("Shows the GLAME console"), show_console_cb, NULL), 
+ 	GNOMEUIINFO_ITEM (N_("Sync"), N_("Syncs meta to disk"), sync_cb, NULL), 
+ 	GNOMEUIINFO_SEPARATOR, 
+ 	GNOMEUIINFO_MENU_EXIT_ITEM (gui_quit, NULL), 
+ 	GNOMEUIINFO_END 
+ }; 
+
+static void new_network_cb(GtkWidget *menu, void *blah); 
+void glame_load_network(GtkWidget *bla, void *blu);
 static void load_plugin_cb(GtkWidget* foo, void *bar);
-static GnomeUIInfo filter_menu_uiinfo[] = {
-	GNOMEUIINFO_MENU_NEW_ITEM(N_("New Filternetwork"), N_("Creates a new filternetwork"), new_network_cb, NULL),
-	GNOMEUIINFO_MENU_OPEN_ITEM(glame_load_network, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM(N_("Load Plugin"),N_("Loads and registers a plugin"), load_plugin_cb,NULL),
-	GNOMEUIINFO_END
-};
+static GnomeUIInfo filter_menu_uiinfo[] = { 
+ 	GNOMEUIINFO_MENU_NEW_ITEM(N_("New Filternetwork"), N_("Creates a new filternetwork"), new_network_cb, NULL), 
+	GNOMEUIINFO_MENU_OPEN_ITEM(glame_load_network, NULL), 
+ 	GNOMEUIINFO_SEPARATOR, 
+ 	GNOMEUIINFO_ITEM(N_("Load Plugin"),N_("Loads and registers a plugin"), load_plugin_cb,NULL),
+ 	GNOMEUIINFO_END 
+ };
 
 static GnomeUIInfo help_menu_uiinfo[] =
-{
-	GNOMEUIINFO_ITEM_DATA(N_("_Help"),N_("Opens a gnome help browser"),gnome_help_goto,"info:glame",NULL),
-	GNOMEUIINFO_ITEM_DATA(N_("Quick Start Guide"), N_("Opens a gnome help browser with the quick start guide"), gnome_help_goto, "info:glame#Quick_Start_Guide", NULL),
-	GNOMEUIINFO_ITEM_DATA(N_("List key-bindings"), N_("Lists the current key-bindings"), glame_accel_widget_data_cb, "list_keybindings", NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_ABOUT_ITEM (glame_about, NULL),
-	GNOMEUIINFO_END
-};
+{ 
+	GNOMEUIINFO_ITEM_DATA(N_("_Help"),N_("Opens a gnome help browser"),glame_help_goto,(gpointer)"info:glame",NULL), 
+ 	GNOMEUIINFO_ITEM_DATA(N_("Quick Start Guide"), N_("Opens a gnome help browser with the quick start guide"), glame_help_goto, (gpointer)"info:glame#Quick_Start_Guide", NULL), 
+ 	GNOMEUIINFO_ITEM_DATA(N_("List key-bindings"), N_("Lists the current key-bindings"), glame_accel_widget_data_cb, (gpointer)"list_keybindings", NULL), 
+ 	GNOMEUIINFO_SEPARATOR, 
+ 	GNOMEUIINFO_MENU_ABOUT_ITEM (glame_about, NULL), 
+ 	GNOMEUIINFO_END 
+}; 
 
-static GnomeUIInfo glame_setting_uiinfo[] =
-{
-        GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
-        GNOMEUIINFO_END
-};
+static GnomeUIInfo glame_setting_uiinfo[] = 
+{ 
+        GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL), 
+        GNOMEUIINFO_END 
+}; 
 
 
-static GnomeUIInfo menubar_uiinfo[] =
-{
-  {
-    GNOME_APP_UI_SUBTREE, N_("_Project"),
-    NULL,
-    swapfile_menu_uiinfo, NULL, NULL,
-    GNOME_APP_PIXMAP_NONE, NULL,
-    0, 0, NULL
-  },
-  {
-    GNOME_APP_UI_SUBTREE, N_("_Filternetwork"),
-    NULL,
-    filter_menu_uiinfo, NULL, NULL,
-    GNOME_APP_PIXMAP_NONE, NULL,
-    0, 0, NULL
-  },
-  GNOMEUIINFO_MENU_SETTINGS_TREE (glame_setting_uiinfo),
-  GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
-  GNOMEUIINFO_END
-};
+static GnomeUIInfo menubar_uiinfo[] = 
+{ 
+   { 
+     GNOME_APP_UI_SUBTREE, N_("_Project"), 
+     NULL, 
+     swapfile_menu_uiinfo, NULL, NULL, 
+     GNOME_APP_PIXMAP_NONE, NULL, 
+     0, (GdkModifierType)0, NULL 
+   }, 
+   { 
+     GNOME_APP_UI_SUBTREE, N_("_Filternetwork"), 
+     NULL, 
+     filter_menu_uiinfo, NULL, NULL, 
+     GNOME_APP_PIXMAP_NONE, NULL, 
+     0, (GdkModifierType)0, NULL 
+   }, 
+   GNOMEUIINFO_MENU_SETTINGS_TREE (glame_setting_uiinfo), 
+   GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo), 
+   GNOMEUIINFO_END 
+ }; 
 
 
-static void show_console_cb(GtkWidget *menu, void * blah)
-{
-	glame_console_show();
-}
+static void show_console_cb(GtkWidget *menu, void * blah) 
+{ 
+	glame_console_show(); 
+} 
 
-static void sync_cb(GtkWidget *menu, void * blah)
-{
-	gpsm_sync();
-}
+static void sync_cb(GtkWidget *menu, void * blah) 
+{ 
+ 	gpsm_sync(); 
+} 
 
-static void emptytrash_cb(GtkWidget *menu, void * blah)
-{
-	gpsm_grp_t *deleted;
+static void emptytrash_cb(GtkWidget *menu, void * blah) 
+{ 
+        gpsm_grp_t *deleted; 
 
-	if (!(deleted = gpsm_find_grp_label(gpsm_root(), NULL, GPSM_GRP_DELETED_LABEL)))
-		return;
-	gpsm_item_destroy((gpsm_item_t *)deleted);
-	gpsm_sync();
-}
+ 	if (!(deleted = gpsm_find_grp_label(gpsm_root(), NULL, GPSM_GRP_DELETED_LABEL))) 
+ 		return; 
+	gpsm_item_destroy((gpsm_item_t *)deleted); 
+	gpsm_sync(); 
+} 
 
-static void new_network_cb(GtkWidget *menu, void * blah)
-{
-	GtkWidget *feg;
-	feg = glame_filtereditgui_new(NULL, FALSE);
-	gtk_quit_add_destroy(1, GTK_OBJECT(feg));
-	gtk_widget_show(feg);
-}
+static void new_network_cb(GtkWidget *menu, void * blah) 
+{ 
+ 	GtkWidget *feg; 
+ 	feg = glame_filtereditgui_new(NULL, FALSE); 
+ 	gtk_quit_add_destroy(1, GTK_OBJECT(feg)); 
+ 	gtk_widget_show(feg); 
+} 
 
-extern void edit_tree_label(GlameTreeItem * item);
-static void create_new_project_cb(GtkWidget *menu, void * blah)
-{
-	gpsm_grp_t *grp, *deleted;
-	GlameTreeItem *grpw;
-
-	if ((deleted = gpsm_find_grp_label(gpsm_root(), NULL, GPSM_GRP_DELETED_LABEL)))
-		gpsm_item_remove((gpsm_item_t *)deleted);
+static void create_new_project_cb(GtkWidget *menu, void * blah) 
+{ 
+	gpsm_grp_t *grp; 
 
 	/* Create new gpsm group. */
-	grp = gpsm_newgrp(_("Unnamed"));
-	if (gpsm_item_place(gpsm_root(), (gpsm_item_t *)grp,
-			    0, gpsm_item_vsize(gpsm_root())) == -1
-	    && gpsm_item_place(gpsm_root(), (gpsm_item_t *)grp,
-			       0, gpsm_item_vsize(gpsm_root())+1) == -1)
+ 	grp = gpsm_newgrp(_("New project")); 
+ 	if (gpsm_item_place(gpsm_root(), (gpsm_item_t *)grp, 
+ 			    0, gpsm_item_vsize(gpsm_root())) == -1 
+ 	    && gpsm_item_place(gpsm_root(), (gpsm_item_t *)grp, 
+ 			       0, gpsm_item_vsize(gpsm_root())+1) == -1) 
 		DPRINTF("Cannot insert new group!?\n");
-	if (deleted)
-		gpsm_item_place(gpsm_root(), (gpsm_item_t *)deleted,
-				0, GPSM_GRP_DELETED_VPOS);
+} 
 
-	/* Find out which widget it got. */
-	grpw = glame_tree_find_gpsm_item(
-		GTK_OBJECT(SWAPFILE_GUI(swapfile)->tree), (gpsm_item_t *)grp);
-	if (!grpw) {
-		DPRINTF("Umm, cant find widget for new project.\n");
-		return;
+static void edit_file_cleanup_cb(GtkObject *widget, gpsm_item_t *file) 
+{ 
+ 	gpsm_item_destroy(file); 
+} 
+static void edit_file_mark_modified_cb(glsig_handler_t *h, long sig, va_list va) 
+{ 
+ 	WaveeditGui *we = WAVEEDIT_GUI(glsig_handler_private(h)); 
+ 	we->modified = 1; 
+} 
+static void edit_file_cb(GtkWidget *menu, void *data) 
+{ 
+ 	gpsm_item_t *file, *item; 
+ 	WaveeditGui *we; 
+
+/* Run the import dialog. */
+ 	file = glame_import_dialog(GTK_WINDOW(app)); 
+	if (!file) 
+ 		return; 
+
+ 	/* Open the waveedit window and register a handler for gpsm 
+ 	 * deletion after widget destroy. Also register handlers for 
+ 	 * track modification to update bool. */
+ 	we = glame_waveedit_gui_new(gpsm_item_label(file), file); 
+ 	if (!we) { 
+		glame_error_dialog(_("Cannot open wave editor"), NULL);
+ 		gpsm_item_destroy(file); 
+ 		return; 
+ 	} 
+ 	gtk_signal_connect(GTK_OBJECT(we), "destroy", 
+ 			   (GtkSignalFunc)edit_file_cleanup_cb, file); 
+ 	gpsm_grp_foreach_item(file, item) 
+ 		glsig_add_handler(gpsm_item_emitter(item), GPSM_SIG_SWFILE_CHANGED|GPSM_SIG_SWFILE_CUT|GPSM_SIG_SWFILE_INSERT, 
+ 				  edit_file_mark_modified_cb, we); 
+ 	gtk_quit_add_destroy(1, GTK_OBJECT(we)); 
+ 	gtk_widget_show_all(GTK_WIDGET(we)); 
+} 
+
+static void record_wave_cb(GtkWidget *menu, void *data) 
+{ 
+	gpsm_grp_t *grp;
+	gpsm_swfile_t *left, *right;
+ 	gpsm_item_t *item; 
+ 	WaveeditGui *we; 
+
+	/* Create a new stereo track.  */
+	grp = gpsm_newgrp(_("New wave"));
+	left = gpsm_newswfile(_("left"));
+	gpsm_swfile_set_position(left, FILTER_PIPEPOS_LEFT);
+	right = gpsm_newswfile(_("right"));
+	gpsm_swfile_set_position(right, FILTER_PIPEPOS_RIGHT);
+	gpsm_vbox_insert(grp, (gpsm_item_t *)left, 0, 0);
+	gpsm_vbox_insert(grp, (gpsm_item_t *)right, 0, 1);
+
+ 	/* Open the waveedit window and register a handler for gpsm 
+ 	 * deletion after widget destroy. Also register handlers for 
+ 	 * track modification to update bool. */
+ 	we = glame_waveedit_gui_new("New wave", grp); 
+ 	if (!we) { 
+		glame_error_dialog(_("Cannot open wave editor"), NULL);
+ 		gpsm_item_destroy(grp); 
+ 		return; 
+ 	} 
+ 	gtk_signal_connect(GTK_OBJECT(we), "destroy", 
+ 			   (GtkSignalFunc)edit_file_cleanup_cb, grp); 
+ 	gpsm_grp_foreach_item(grp, item) 
+ 		glsig_add_handler(gpsm_item_emitter(item), GPSM_SIG_SWFILE_CHANGED|GPSM_SIG_SWFILE_CUT|GPSM_SIG_SWFILE_INSERT, 
+ 				  edit_file_mark_modified_cb, we); 
+ 	gtk_quit_add_destroy(1, GTK_OBJECT(we)); 
+ 	gtk_widget_show_all(GTK_WIDGET(we)); 
+} 
+
+static void import_cb(GtkWidget *menu, void *data) 
+{ 
+ 	gpsm_item_t *file; 
+ 	gpsm_grp_t *deleted; 
+ 	int res; 
+
+ 	/* Run the import dialog. */ 
+ 	file = glame_import_dialog(GTK_WINDOW(app)); 
+ 	if (!file) 
+ 		return; 
+
+ 	if ((deleted = gpsm_find_grp_label(gpsm_root(), NULL, GPSM_GRP_DELETED_LABEL))) 
+ 		gpsm_item_remove((gpsm_item_t *)deleted); 
+ 	res = gpsm_vbox_insert(gpsm_root(), file, 
+ 			       0, gpsm_item_vsize(gpsm_root())); 
+ 	if (deleted) 
+ 		gpsm_item_place(gpsm_root(), (gpsm_item_t *)deleted, 
+ 				0, GPSM_GRP_DELETED_VPOS); 
+ 	if (res == -1) { 
+ 		glame_error_dialog(_("Cannot place imported wave"), NULL); 
+ 		gpsm_item_destroy(file); 
+ 	} 
+ } 
+
+static void load_plugin_cb(GtkWidget*bla,void*blu) 
+{ 
+ 	GtkWidget *dialog; 
+ 	char *filename;
+
+	dialog = gtk_file_chooser_dialog_new(
+		_("Load Plugin"), NULL,
+		GTK_FILE_CHOOSER_ACTION_OPEN,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
+		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		if (glame_load_plugin(filename) == -1) 
+			glame_error_dialog(_("Error loading plugin"), NULL);
+		g_free(filename);
 	}
-	edit_tree_label(grpw);
-}
-
-static void edit_file_cleanup_cb(GtkObject *widget, gpsm_item_t *file)
-{
-	gpsm_item_destroy(file);
-}
-static void edit_file_mark_modified_cb(glsig_handler_t *h, long sig, va_list va)
-{
-	WaveeditGui *we = WAVEEDIT_GUI(glsig_handler_private(h));
-	we->modified = 1;
-}
-static void edit_file_cb(GtkWidget *menu, void *data)
-{
-	gpsm_item_t *file, *item;
-	WaveeditGui *we;
-
-	/* Run the import dialog. */
-	file = glame_import_dialog(GTK_WINDOW(app));
-	if (!file)
-		return;
-
-	/* Open the waveedit window and register a handler for gpsm
-	 * deletion after widget destroy. Also register handlers for
-	 * track modification to update bool. */
-	we = glame_waveedit_gui_new(gpsm_item_label(file), file);
-	if (!we) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("Cannot open wave editor"))));
-		gpsm_item_destroy(file);
-		return;
-	}
-	gtk_signal_connect(GTK_OBJECT(we), "destroy",
-			   (GtkSignalFunc)edit_file_cleanup_cb, file);
-	gpsm_grp_foreach_item(file, item)
-		glsig_add_handler(gpsm_item_emitter(item), GPSM_SIG_SWFILE_CHANGED|GPSM_SIG_SWFILE_CUT|GPSM_SIG_SWFILE_INSERT,
-				  edit_file_mark_modified_cb, we);
-	gtk_quit_add_destroy(1, GTK_OBJECT(we));
-	gtk_widget_show_all(GTK_WIDGET(we));
-}
-
-static void import_cb(GtkWidget *menu, void *data)
-{
-	gpsm_item_t *file;
-	gpsm_grp_t *deleted;
-	int res;
-
-	/* Run the import dialog. */
-	file = glame_import_dialog(GTK_WINDOW(app));
-	if (!file)
-		return;
-
-	if ((deleted = gpsm_find_grp_label(gpsm_root(), NULL, GPSM_GRP_DELETED_LABEL)))
-		gpsm_item_remove((gpsm_item_t *)deleted);
-	res = gpsm_vbox_insert(gpsm_root(), file,
-			       0, gpsm_item_vsize(gpsm_root()));
-	if (deleted)
-		gpsm_item_place(gpsm_root(), (gpsm_item_t *)deleted,
-				0, GPSM_GRP_DELETED_VPOS);
-	if (res == -1) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(_("Cannot place imported wave"))));
-		gpsm_item_destroy(file);
-	}
-}
-
-static void load_plugin_cb(GtkWidget*bla,void*blu)
-{
-	GtkWidget *dialog;
-	char filenamebuffer[256];
-
-	filenamebuffer[0] = '\0';
-	dialog = glame_dialog_file_request(_("Load Plugin"),
-					   "main:load_plugin", _("Filename"),
-					   NULL, filenamebuffer);
-	if (!gnome_dialog_run_and_close(GNOME_DIALOG(dialog))
-	    || !*filenamebuffer)
-		return;
-
-	if (glame_load_plugin(filenamebuffer) == -1)
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("Error loading plugin"))));
-}
+	gtk_widget_destroy(dialog);
+} 
 
 
 
@@ -368,15 +385,13 @@ static int update_preferences()
 	DPRINTF(
 "Preferences:\n"
 "\tSwapfile directory %s\n"
-"\tVM parameters are: lru %li, fds %li, maps %li, vm %li\n"
 "\tUndo stack depth is %li\n"
 "\tAudio input plugin %s, device \"%s\", rate %li\n"
 "\tAudio output plugin %s, device \"%s\"\n"
 "\tGLAME_WBUFSIZE %i\n"
 "\tPopup timeout is %lims\n"
 "\tMac mode is %s\n",
-                swappath, maxlru, maxfds, maxmaps, maxvm,
-		maxundo, ainplugin, aindev, rate, aoutplugin, aoutdev, _GLAME_WBUFSIZE,
+                swappath, maxundo, ainplugin, aindev, rate, aoutplugin, aoutdev, _GLAME_WBUFSIZE,
                 nPopupTimeout, bMac ? "on" : "off");
 
 	/* Free temp. storage. */
@@ -403,83 +418,6 @@ toggle_cb(GtkWidget*foo, gboolean *bar)
 	*bar = (*bar)?FALSE:TRUE;
 }
 
-#if 0
-/* This widget should build a widget with a list of the current
- * (configured) audio I/O plugins and allow to add custom ones
- * and configure existing ones. */
-static GtkWidget *audio_io_preferences_widget_new()
-{
-	GtkWidget *vbox;
-	GtkWidget *hbox1;
-	GtkWidget *vbox2;
-	GtkWidget *label1;
-	GtkWidget *list1;
-	GtkWidget *vbox1;
-	GtkWidget *hbox2;
-	GtkWidget *entry1;
-	GtkWidget *combo_entry2;
-	GtkWidget *combo1;
-	GtkWidget *combo_entry1;
-	GtkWidget *hbox3;
-	GtkWidget *button1;
-	GtkWidget *button2;
-
-	vbox = gtk_vbox_new(FALSE, 0);
-
-	hbox1 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox1);
-	gtk_container_add (GTK_CONTAINER (vbox), hbox1);
-
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox2);
-	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
-
-	label1 = gtk_label_new (_("Audio plugins"));
-	gtk_widget_show (label1);
-	gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);
-
-	list1 = gtk_list_new ();
-	gtk_widget_show (list1);
-	gtk_box_pack_start (GTK_BOX (vbox2), list1, TRUE, TRUE, 0);
-
-	vbox1 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox1);
-	gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);
-
-	hbox2 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox2);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);
-
-	entry1 = gnome_entry_new (NULL);
-	gtk_widget_show (entry1);
-	gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 0);
-
-	combo_entry2 = gnome_entry_gtk_entry (GNOME_ENTRY (entry1));
-	gtk_widget_show (combo_entry2);
-
-	combo1 = gtk_combo_new ();
-	gtk_widget_show (combo1);
-	gtk_box_pack_start (GTK_BOX (hbox2), combo1, TRUE, TRUE, 0);
-
-	combo_entry1 = GTK_COMBO (combo1)->entry;
-	gtk_widget_show (combo_entry1);
-
-	hbox3 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox3);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox3, TRUE, TRUE, 0);
-
-	button1 = gtk_button_new_with_label (_("Add"));
-	gtk_widget_show (button1);
-	gtk_box_pack_start (GTK_BOX (hbox3), button1, TRUE, TRUE, 0);
-
-	button2 = gtk_button_new_with_label (_("Apply"));
-	gtk_widget_show (button2);
-	gtk_box_pack_start (GTK_BOX (hbox3), button2, TRUE, TRUE, 0);
-
-	return vbox;
-}
-#endif
-
 static GtkWidget *preferences_tab_filteredit(long *fe_popup_timeout, long *fe_mac)
 {
 	GtkWidget *vbox, *macMode;
@@ -553,48 +491,55 @@ static GtkWidget *preferences_tab_swapfile(char *sw_path,
 	return vbox;
 }
 
+static gint changeCombo(GtkComboBox *combo, char *string)
+{
+	GtkTreeModel *model;
+	GtkTreeIter i;
+	GValue val = { 0, };
+
+	if (!gtk_combo_box_get_active_iter(combo, &i))
+		return FALSE;
+	model = gtk_combo_box_get_model(combo);
+	gtk_tree_model_get_value(model, &i, 0, &val);
+	strcpy(string, g_value_get_string(&val));
+
+	return TRUE;
+}
+
 static GtkWidget *preferences_tab_audioio(char *ainplugin, char *aindev,
 					  char *aoutplugin, char *aoutdev,
 					  long *rate, long *wbufsize)
 {
 	GtkWidget *vbox, *combo;
-	GList *combo_items;
 	char *cfg;
 
 	vbox = gtk_vbox_new(FALSE, 1);
 
 	/* input filter */
-	combo_items = NULL;
+	combo = gtk_combo_box_new_text();
+	gtk_widget_show(combo);
+	glame_config_get_string("audio_io/input_plugin", &cfg);
+	strncpy(ainplugin, cfg, 255);
+	g_free(cfg);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), ainplugin);
 	if (plugin_get("oss_audio_in"))
-		combo_items = g_list_append(combo_items, "oss_audio_in");
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "oss_audio_in");
 	if (plugin_get("esd_audio_in"))
-		combo_items = g_list_append(combo_items, "esd_audio_in");
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "esd_audio_in");
 	if (plugin_get("alsa_audio_in"))
-		combo_items = g_list_append(combo_items, "alsa_audio_in");
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "alsa_audio_in");
 	if (plugin_get("sgi_audio_in"))
-		combo_items = g_list_append(combo_items, "sgi_audio_in");
-	if (combo_items) {
-		combo = gtk_combo_new();
-		gtk_combo_set_popdown_strings(GTK_COMBO(combo), combo_items);
-		g_list_free(combo_items);
-		gtk_widget_show(combo);
-		glame_config_get_string("audio_io/input_plugin", &cfg);
-		strncpy(ainplugin, cfg, 255);
-		g_free(cfg);
-		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), ainplugin);
-		gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed", (GtkSignalFunc)changeString, ainplugin);
-		gtk_widget_show(GTK_COMBO(combo)->entry);
-		create_label_widget_pair(vbox, _("Default input plugin (audio_in)"), combo);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "sgi_audio_in");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+	gtk_signal_connect(GTK_OBJECT(combo), "changed", (GtkSignalFunc)changeCombo, ainplugin);
+	create_label_widget_pair(vbox, _("Default input plugin (audio_in)"), combo);
+
+	/* input device */
+	glame_config_get_string("audio_io/input_dev", &cfg);
+	strncpy(aindev, cfg, 255);
+	g_free(cfg);
+	create_label_edit_pair(vbox, _("Default input device"), "aindev", aindev);
 
-		/* input device */
-		glame_config_get_string("audio_io/input_dev", &cfg);
-		strncpy(aindev, cfg, 255);
-		g_free(cfg);
-		create_label_edit_pair(vbox, _("Default input device"), "aindev", aindev);
-	} else {
-		gtk_container_add(GTK_CONTAINER(vbox),
-				  gtk_label_new(_("No audio input plugin")));
-	}
 	/* sample rate */
 	glame_config_get_long("audio_io/input_rate", rate);
 	create_label_long_pair(vbox, _("Default input sample rate"), rate, 11025, 48000);
@@ -602,36 +547,29 @@ static GtkWidget *preferences_tab_audioio(char *ainplugin, char *aindev,
 	gtk_container_add(GTK_CONTAINER(vbox), gtk_hseparator_new());
 
 	/* output filter */
-	combo_items = NULL;
+	combo = gtk_combo_box_new_text();
+	glame_config_get_string("audio_io/output_plugin", &cfg);
+	strncpy(aoutplugin, cfg, 255);
+	g_free(cfg);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), aoutplugin);
 	if (plugin_get("oss_audio_out"))
-		combo_items = g_list_append(combo_items, "oss_audio_out");
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "oss_audio_out");
 	if (plugin_get("esd_audio_out"))
-		combo_items = g_list_append(combo_items, "esd_audio_out");
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "esd_audio_out");
 	if (plugin_get("alsa_audio_out"))
-		combo_items = g_list_append(combo_items, "alsa_audio_out");
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "alsa_audio_out");
 	if (plugin_get("sgi_audio_out"))
-		combo_items = g_list_append(combo_items, "sgi_audio_out");
-	if (combo_items) {
-		combo = gtk_combo_new();
-		gtk_combo_set_popdown_strings(GTK_COMBO(combo), combo_items);
-		g_list_free(combo_items);
-		glame_config_get_string("audio_io/output_plugin", &cfg);
-		strncpy(aoutplugin, cfg, 255);
-		g_free(cfg);
-		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), aoutplugin);
-		gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed", (GtkSignalFunc)changeString, aoutplugin);
-		create_label_widget_pair(vbox, _("Default output plugin (audio_out)"), combo);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "sgi_audio_out");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+	gtk_signal_connect(GTK_OBJECT(combo), "changed", (GtkSignalFunc)changeCombo, aoutplugin);
+	create_label_widget_pair(vbox, _("Default output plugin (audio_out)"), combo);
 
-		/* output device */
-		glame_config_get_string("audio_io/output_dev", &cfg);
-		strncpy(aoutdev, cfg, 255);
-		g_free(cfg);
-		create_label_edit_pair(vbox, _("Default output device"),
-				       "aoutdev", aoutdev);
-	} else {
-		gtk_container_add(GTK_CONTAINER(vbox),
-				  gtk_label_new(_("No audio output plugin")));
-	}
+	/* output device */
+	glame_config_get_string("audio_io/output_dev", &cfg);
+	strncpy(aoutdev, cfg, 255);
+	g_free(cfg);
+	create_label_edit_pair(vbox, _("Default output device"),
+			       "aoutdev", aoutdev);
 
 	gtk_container_add(GTK_CONTAINER(vbox), gtk_hseparator_new());
 
@@ -646,7 +584,7 @@ static GtkWidget *preferences_tab_audioio(char *ainplugin, char *aindev,
 
 static void pref_help_cb(GnomePropertyBox *box, gint page)
 {
-	gnome_help_goto(NULL, "info:glame#The_Preferences_Dialog");
+	glame_help_goto(NULL, "info:glame#The_Preferences_Dialog");
 }
 
 static int
@@ -713,12 +651,9 @@ preferences_cb(GtkWidget * wid, void * bla)
 	 */
 	glame_config_set_string("swapfile/defaultpath", sw_path);
 	glame_config_set_long("swapfile/maxundo", sw_maxundo);
-	/* Do some magic with sw_maxvm number to derive the rest.
-	 * This is now more safe, but also too low for 64bit large
-	 * memory machines. */
-	sw_maxvm = MIN(3072, sw_maxvm);
+	/* Do some magic with sw_maxvm number to derive the rest. */
 	sw_maxlru = 2048 * sw_maxvm / 256;
-	sw_maxfds = MIN(256, 128 * sw_maxvm / 256);
+	sw_maxfds = 128 * sw_maxvm / 256;
 	sw_maxmaps = 256 * sw_maxvm / 256;
 	sw_maxvm *= 1024*1024;
 	glame_config_set_long("swapfile/maxlru", sw_maxlru);
@@ -761,37 +696,48 @@ static GtkWidget* glame_about(void)
 	 * issue by allocating dynamically. But it sucks. Leave it
 	 * untranslated for now.
 	 */
+	GdkPixbuf* log;
 	const gchar *authors[]={
-		N_("Richard Guenther [Richi]"),
-		N_("Alexander Ehlert [Mag]"),
-		N_("Daniel Kobras [*nold]"),
-		N_("Johannes Hirche [XWolf]"),
+		N_("Richard Guenther"),
+		N_("Alexander Ehlert"),
+		N_("Daniel Kobras"),
+		N_("Johannes Hirche"),
 		N_("and others"),
 		NULL
 	};
+	const gchar *documenters[]={
+		N_("Richard Guenther"),
+		N_("Daniel Kobras"),
+		N_("Alexander Ehlert"),
+		NULL
+	};
 	GtkWidget *about;
 	char *logo;
-
+	
 	logo = GLAME_LOGO;
+
 	if (access(logo, R_OK) == -1)
 		logo = "../data/pixmaps/glame-logo.jpg";
-	about = gnome_about_new ("GLAME", VERSION, 
-				 _("Copyright (C) 1999-2001 R. Guenther, A. Ehlert, J. Hirche, D. Kobras"),
-				 authors,
-
+	log = gdk_pixbuf_new_from_file(logo,NULL);
+	about = gnome_about_new ("GLAME", 
+				 VERSION, 
+				 _("Copyright (C) 1999-2004 R. Guenther, A. Ehlert, J. Hirche, D. Kobras and others"),
 				 _(
-"This program is free software; you can redistribute it and/or modify"
-"it under the terms of the GNU General Public License as published by"
-"the Free Software Foundation; either version 2 of the License, or"
-"(at your option) any later version.\n"
-"This program is distributed in the hope that it will be useful,"
-"but WITHOUT ANY WARRANTY; without even the implied warranty of"
-"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
-"GNU General Public License for more details.\n"
-"You should have received a copy of the GNU General Public License"
-"along with this program; if not, write to the Free Software"
-"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
-				 logo);
+				   "This program is free software; you can redistribute it and/or modify "
+				   "it under the terms of the GNU General Public License as published by "
+				   "the Free Software Foundation; either version 2 of the License, or "
+				   "(at your option) any later version.\n"
+				   "This program is distributed in the hope that it will be useful, "
+				   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+				   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+				   "GNU General Public License for more details.\n"
+				   "You should have received a copy of the GNU General Public License "
+				   "along with this program; if not, write to the Free Software "
+				   "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
+				 authors,
+				 documenters,
+				 _("Daniel Kobras\nLaurent Georget\nBenno Senoner\nAllan Lyckegaard"),
+				 log);
 	gtk_object_set_data (GTK_OBJECT (about), "about", about);
 	gtk_window_set_modal (GTK_WINDOW (about), FALSE);
 	gtk_window_set_wmclass (GTK_WINDOW (about), "Glameabout", "Glame");
@@ -800,29 +746,37 @@ static GtkWidget* glame_about(void)
 	return about;
 }
 
-static gint glame_splash_timeout(GtkWidget *about)
+static GtkWidget *glame_splash(void)
 {
-	gtk_object_destroy(GTK_OBJECT(about));
-	return FALSE;
-}
-static void glame_splash_destroy_cb(GtkWidget *about, gpointer tid)
-{
-	gtk_timeout_remove((guint)tid);
-}
-static void glame_splash(void)
-{
-	GtkWidget *about;
-	guint tid;
+	GtkWidget *a;
 
-	about = glame_about();
+	/* We may "specialize" the splash somewhat in the future.
+	 * For now just use the about dialog with disabled close. */
+	a = glame_about();
+	gtk_dialog_set_default_response(GTK_DIALOG(a), GTK_RESPONSE_NONE);
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(a), GTK_RESPONSE_CLOSE, FALSE);
 
-#ifdef DEBUG
-	tid = gtk_timeout_add(1000, (GtkFunction)glame_splash_timeout, about);
-#else
-	tid = gtk_timeout_add(5000, (GtkFunction)glame_splash_timeout, about);
-#endif
-	gtk_signal_connect(GTK_OBJECT(about), "destroy",
-			   (GtkSignalFunc)glame_splash_destroy_cb, (gpointer)tid);
+	return a;
+}
+
+
+static long maxundo;
+static pthread_mutex_t giw_mx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t giw_condmx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t giw_cond = PTHREAD_COND_INITIALIZER;
+static void *gpsm_init_wrapper(void *path_)
+{
+	int res;
+	pthread_mutex_lock(&giw_mx);
+	pthread_mutex_lock(&giw_condmx);
+	pthread_cond_broadcast(&giw_cond);
+	pthread_mutex_unlock(&giw_condmx);
+	res = gpsm_init(path_, maxundo);
+	pthread_mutex_unlock(&giw_mx);
+	if (res == -1)
+		return NULL;
+	else
+		return (void *)-1;
 }
 
 
@@ -845,7 +799,7 @@ static void on_swapfile_panic(const char *msg)
 		 "an internal GLAME error, too).\n\n"
 		 "-- BYE BYE.\n"),
 		 msg, strerror(errno));
-	gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(message)));
+	glame_error_dialog(message, NULL);
 
 	/* Try to save the current gpsm tree. May even lock up. Ugh. */
 	gpsm_sync();
@@ -871,53 +825,64 @@ static void gui_quit(GtkWidget *widget, gpointer data)
 	gtk_main_quit();
 }
 
+static GtkWidget *splash;
 static void gui_main()
 {
 	GtkWidget *dock, *scrollview;
 	char *path;
+	void *gpsm_init_result;
+	pthread_t gpsm_init_thread;
 
 	/* Init GUI dependend subsystems. */
 	glame_accel_init();
-	glame_swapfilegui_init();
 	glame_waveeditgui_init();
 	glame_filtereditgui_init();
-#ifdef ENABLE_EXPERIMENTAL
-	glame_timeline_init();
-#endif
 
 	/* Update preferences. */
 	if (update_preferences() == -1) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog(
+		glame_info_dialog(
 _("Welcome first-time user of GLAME.\n"
 "We need to do some basic setup stuff. Please run through\n"
 "the preferences dialog and check the \"Swapfile Path\" and\n"
-"\"Audio IO\" settings.\n"))));
+"\"Audio IO\" settings.\n"), NULL);
 	run_prefs:
 		if (!preferences_cb(NULL, NULL)) {
-			gnome_dialog_run_and_close(
-				GNOME_DIALOG(gnome_error_dialog(_("You didnt change/check the configuration.\nGLAME is exiting now.\n"))));
+			glame_error_dialog(_("You didnt change/check the configuration.\nGLAME is exiting now.\n"), NULL);
 			exit(1);
 		}
 	}
 
 	glame_config_get_string("swapfile/defaultpath", &path);
 	DPRINTF("path: %s\n",path);
-	if (!g_file_test(path,G_FILE_TEST_ISDIR)) {
-		if (swapfile_creat(path, -1)) {
+	if (!g_file_test(path,G_FILE_TEST_IS_DIR)) {
+		if (swapfile_creat(path, 0)) {
 			char msg[256];
 			char *errmsg = strerror(errno);
 			snprintf(msg, 255, _("GLAME was unable to create its swapfile\nbecause of \"%s\".\nPlease check the configuration.\n"), errmsg);
-			gnome_dialog_run_and_close(
-				GNOME_DIALOG(gnome_error_dialog(msg)));
+			glame_error_dialog(msg, NULL);
 			goto run_prefs;
 		}
 	}
-	if (gpsm_init(path) == -1) {
+
+	/* Asynchronly try to initialize the gpsm. */
+	glame_config_get_long("swapfile/maxundo", &maxundo);
+	pthread_mutex_lock(&giw_condmx);
+	pthread_create(&gpsm_init_thread, NULL, gpsm_init_wrapper, path);
+	pthread_cond_wait(&giw_cond, &giw_condmx);
+
+	/* Poll for thread completion. */
+	while (pthread_mutex_trylock(&giw_mx)) {
+		gtk_main_iteration_do(FALSE);
+		sched_yield();
+	}
+	pthread_join(gpsm_init_thread, &gpsm_init_result);
+	pthread_mutex_unlock(&giw_mx);
+
+	if (gpsm_init_result == NULL) {
 		char msg[256];
 		char *errmsg = strerror(errno);
 		snprintf(msg, 255, "GLAME was unable to open/init its swapfile\nbecause of \"%s\".\nPlease check the configuration and/or check for\nGLAME messages on the console.\n", errmsg);
-		gnome_dialog_run_and_close(
-			GNOME_DIALOG(gnome_error_dialog(msg)));
+		glame_error_dialog(msg, NULL);
 		goto run_prefs;
 	}
 	g_free(path);
@@ -926,7 +891,7 @@ _("Welcome first-time user of GLAME.\n"
 	swapfile_register_panic_handler(on_swapfile_panic);
 
 	/* create swapfile gui - in a scrolled window */
-	swapfile = GTK_WIDGET(glame_swapfile_widget_new(gpsm_root()));
+	swapfile = glame_gltree_init(gpsm_root());
 	if (!swapfile)
 		return;
 	scrollview = gtk_scrolled_window_new(NULL, NULL);
@@ -965,22 +930,14 @@ _("Welcome first-time user of GLAME.\n"
 	gtk_signal_connect(GTK_OBJECT(swapfile), "size_request",
 			   (GtkSignalFunc)resize_horiz_cb, app);
 
-	/* Register accelerators. */
-	SWAPFILE_GUI(swapfile)->accel_handler =
-		glame_accel_install(app, "swapfile", NULL);
-	SWAPFILE_GUI(swapfile)->app = app;
-
-	/* Pop up splash screen. */
-	glame_splash();
-
 	/* Pop up console. */
 	glame_console_init();
 	glame_console_printf("%s%s%s",
-_("    GLAME version "), VERSION, _(", Copyright (C) 1999-2001 by\n"
+_("    GLAME version "), VERSION, _(", Copyright (C) 1999-2004 by\n"
 "    Alexander Ehlert, Richard Guenther, Johannes Hirche,\n"
-"    Daniel Kobras. GLAME comes with ABSOLUTELY NO\n"
-"    WARRANTY. This is free software, and you are welcome to\n"
-"    redistribute it under certain conditions.\n\n"));
+"    Daniel Kobras and others.\n"
+"    GLAME comes with ABSOLUTELY NO WARRANTY. This is free software,\n"
+"    and you are welcome to redistribute it under certain conditions.\n\n"));
 #ifndef HAVE_LADSPA
 	glame_console_printf("WARNING! This copy of GLAME was compiled without LADSPA plugin support\n");
 #endif
@@ -988,6 +945,9 @@ _("    GLAME version "), VERSION, _(", Copyright (C) 1999-2001 by\n"
 	glame_console_printf("WARNING! This copy of GLAME was compiled without libglade support\n");
 #endif
 
+	/* Finally, destroy splash. */
+	gtk_object_destroy(GTK_OBJECT(splash));
+
 	/* Main event loop */
        	gtk_main();
 
@@ -1002,10 +962,17 @@ _("    GLAME version "), VERSION, _(", Copyright (C) 1999-2001 by\n"
 
 int main(int argc, char **argv)
 {
+        bind_textdomain_codeset("glame", "UTF-8");
 	textdomain("glame");
 
 	/* setup gnome/gtk  */
-	gnome_init("glame0.7", VERSION, argc, argv);
+	gnome_program_init("glame2", VERSION, LIBGNOMEUI_MODULE,
+			   argc, argv, GNOME_PARAM_NONE);
+
+	/* Pop up splash screen. */
+	splash = glame_splash();
+	while (gtk_events_pending())
+		gtk_main_iteration_do(FALSE) ;
 
 #ifdef HAVE_LIBGLADE
 	glade_init();
