@@ -1,7 +1,7 @@
 /*
  * glame_gui_utils.c
  *
- * $Id: glame_gui_utils.c,v 1.27.2.3 2004/10/23 13:14:22 richi Exp $
+ * $Id: glame_gui_utils.c,v 1.40 2005/03/11 20:29:43 richi Exp $
  *
  * Copyright (C) 2001, 2002, 2003 Johannes Hirche
  *
@@ -85,9 +85,10 @@ void create_label_edit_pair(GtkWidget *vbox,
 	wlabel = gtk_label_new(label);
 	gtk_misc_set_alignment(GTK_MISC(wlabel), 1.0, 0.5);
 
-	wentry = gnome_entry_new(history);
-	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(wentry))), result);
-	gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(wentry))), "changed", update_string_from_editable_cb, result);
+	wentry = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(wentry), result);
+	gtk_signal_connect(GTK_OBJECT(wentry), "changed",
+			   GTK_SIGNAL_FUNC(update_string_from_editable_cb), result);
 
 	gtk_container_add(GTK_CONTAINER(whbox), wlabel);
 	gtk_container_add(GTK_CONTAINER(whbox), wentry);
@@ -142,39 +143,6 @@ void create_label_double_pair(GtkWidget *vbox,
 }
 
 
-static void glame_dialog_file_request_browse_cb(GnomeFileEntry *fileentry,
-						const char *pattern)
-{
-	/* Well, this f*** widget is NULL here... */
-	gtk_file_selection_complete(GTK_FILE_SELECTION(fileentry->fsw),
-				    pattern);
-}
-GtkWidget *glame_dialog_file_request(const char *windowtitle,
-				     const char *history_id, const char *label,
-				     const char *pattern,
-				     char *returnbuffer)
-{
-	GtkWidget *dialog;
-	GtkWidget *fileEntry;
-	GtkWidget *dialogVbox;	
-	
-	dialog = gnome_dialog_new(windowtitle, GNOME_STOCK_BUTTON_CANCEL,
-				  GNOME_STOCK_BUTTON_OK, NULL);
-        dialogVbox = GTK_WIDGET(GTK_VBOX(GNOME_DIALOG(dialog)->vbox));
-
-        fileEntry = gnome_file_entry_new(history_id, label);
-	gnome_file_entry_set_modal(GNOME_FILE_ENTRY(fileEntry), TRUE);
-        gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fileEntry))),
-                           "changed", update_string_from_editable_cb,
-			   returnbuffer);
-	if (pattern)
-		gtk_signal_connect_after(GTK_OBJECT(fileEntry), "browse-clicked",
-					 (GtkSignalFunc)glame_dialog_file_request_browse_cb,
-					 (gpointer)pattern);
-        create_label_widget_pair(dialogVbox, "Filename", fileEntry);
-	return dialog;
-}
-
 
 /*
  * Network error dialog
@@ -192,7 +160,7 @@ void glame_network_error_dialog(filter_t *net, const char *header)
 			snprintf(msg+cnt, 4096-cnt, " %s: %s\n",
 				 filter_name(node), filter_errstr(node));
 	}
-	gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(msg)));
+	glame_error_dialog(msg, NULL);
 }
 
 
@@ -317,7 +285,7 @@ glame_gui_build_plugin_menu_genmenu(struct glame_list_head *entries, GtkMenu *me
 			if (gtksighand)
 				gtk_signal_connect(
 					GTK_OBJECT(mitem), "activate",
-					gtksighand, entry->u.p.plugin);
+					GTK_SIGNAL_FUNC(gtksighand), entry->u.p.plugin);
 			gtk_widget_show(mitem);
 		}
 		glame_list_del(&entry->list);
@@ -436,7 +404,7 @@ void glame_help_cb(GtkWidget *bla, const char *helppath)
 {
 	char path[256];
 	snprintf(path, 255, "info:glame#%s", helppath);
-	gnome_help_goto(NULL, path);
+	glame_help_goto(NULL, path);
 }
 
 GtkWidget *
@@ -458,7 +426,7 @@ glame_gui_filter_properties(filter_paramdb_t *pdb, const char *caption, const ch
 
 	gtk_object_destroy(GTK_OBJECT(GNOME_PROPERTY_BOX(propBox)->apply_button));
 	if (helppath)
-		gtk_signal_connect(GTK_OBJECT(GNOME_PROPERTY_BOX(propBox)->help_button), "clicked", glame_help_cb, (gpointer)helppath);
+		gtk_signal_connect(GTK_OBJECT(GNOME_PROPERTY_BOX(propBox)->help_button), "clicked", GTK_SIGNAL_FUNC(glame_help_cb), (gpointer)helppath);
 	else
 		gtk_object_destroy(GTK_OBJECT(GNOME_PROPERTY_BOX(propBox)->help_button));
 	gtk_object_destroy(GTK_OBJECT(GNOME_PROPERTY_BOX(propBox)->ok_button));
@@ -474,7 +442,7 @@ glame_gui_filter_properties(filter_paramdb_t *pdb, const char *caption, const ch
 					    GLSIG_PARAM_DELETED,
 					    fprop_kill_prop, propBox);
 		gtk_signal_connect(GTK_OBJECT(propBox), "destroy",
-				   fprop_kill_handler, handler);
+				   GTK_SIGNAL_FUNC(fprop_kill_handler), handler);
 	}
 
 	return propBox;
@@ -501,10 +469,10 @@ GtkWidget *glame_gui_from_paramdb(filter_paramdb_t *pdb)
 
 
 
-GdkImlibImage* glame_load_icon(const char *filename, int x, int y)
+GdkPixbuf* glame_load_icon(const char *filename, int x, int y)
 {
-	GdkImlibImage* image = NULL;
-	GdkImlibImage* imagetmp = NULL;
+	GdkPixbuf* image = NULL;
+	GdkPixbuf* imagetmp = NULL;
 	const char * file;
 	char * filepath;
 
@@ -516,30 +484,35 @@ GdkImlibImage* glame_load_icon(const char *filename, int x, int y)
 
 	/* check if stock gnome */
 	if ((filepath = gnome_pixmap_file(file))) {
-		image = gdk_imlib_load_image(filepath);
+		imagetmp = gdk_pixbuf_new_from_file(filepath,NULL);
 		g_free(filepath);
 	}
-
-	/* maybe in Glamepixmappath? */
-	filepath = g_concat_dir_and_file(GLAME_PIXMAP_PATH,file);
-	if (!g_file_test(filepath, G_FILE_TEST_ISFILE)) {
+	if(!imagetmp){
+		/* maybe in Glamepixmappath? */
+		filepath = g_concat_dir_and_file(GLAME_PIXMAP_PATH,file);
+		if (!g_file_test(filepath, G_FILE_TEST_IS_REGULAR)) {
+			g_free(filepath);
+			/* maybe cvs? */
+			filepath = g_concat_dir_and_file("../data/pixmaps", file);
+		}
+		if (!g_file_test(filepath, G_FILE_TEST_IS_REGULAR)) {
+			g_free(filepath);
+			/* default! */
+			filepath = gnome_pixmap_file(GLAME_DEFAULT_ICON);
+		}
+		if(!filepath)
+			filepath = g_concat_dir_and_file("../data/pixmaps", GLAME_DEFAULT_ICON);
+		
+		imagetmp = gdk_pixbuf_new_from_file(filepath,NULL);
+		if(!imagetmp)
+			DPRINTF("Failed to load image %s to pixbuf\n",filepath);
 		g_free(filepath);
-		/* maybe cvs? */
-		filepath = g_concat_dir_and_file("../data/pixmaps", file);
 	}
-	if (!g_file_test(filepath, G_FILE_TEST_ISFILE)) {
-		g_free(filepath);
-		/* default! */
-		filepath = gnome_pixmap_file(GLAME_DEFAULT_ICON);
-	}
-	if(!filepath)
-		filepath = g_concat_dir_and_file("../data/pixmaps", GLAME_DEFAULT_ICON);
-	
-	imagetmp = gdk_imlib_load_image(filepath);
-	g_free(filepath);
 	if(x && y){
-		image = gdk_imlib_clone_scaled_image(imagetmp, x,y);
-		gdk_imlib_destroy_image(imagetmp);
+		if(!imagetmp)
+			DPRINTF("pixbuf == NULL\n");
+		image = gdk_pixbuf_scale_simple(imagetmp,x,y,GDK_INTERP_BILINEAR);
+		g_object_unref(imagetmp);
 		return image;
 	}else{
 		return imagetmp;
@@ -548,14 +521,14 @@ GdkImlibImage* glame_load_icon(const char *filename, int x, int y)
 
 GtkWidget* glame_load_icon_widget(const char* filename,int x, int y)
 {
-	GdkImlibImage *icon;
-	GtkWidget *pixmap;
+	GdkPixbuf* img;
 
-	icon = glame_load_icon(filename,x,y);
-	pixmap = gnome_pixmap_new_from_imlib(icon);
-	gdk_imlib_destroy_image(icon);
-
-	return pixmap;
+	img = glame_load_icon(filename,x,y);
+	if (!img) {
+		DPRINTF("Error opening %s\n", filename);
+		return NULL;
+	}
+	return gtk_image_new_from_pixbuf(img);
 }
 
 
@@ -687,6 +660,9 @@ gint glame_menu_get_active_index(GtkMenu *menu)
 	GList *list;
 	int val;
 
+	if (!menu)
+		return -1;
+
 	act = gtk_menu_get_active(menu);
 	list = gtk_container_children(GTK_CONTAINER(menu));
 	val = 0;
@@ -699,3 +675,43 @@ gint glame_menu_get_active_index(GtkMenu *menu)
 
 	return list ? val : -1;
 }
+
+void glame_help_goto(void *ignore, const char *url)
+{
+        GError *ret = NULL;
+        gnome_url_show(url, &ret);
+}
+
+void glame_error_dialog(const char *message, GtkWindow *parent)
+{
+	GtkWidget *dialog;
+	dialog = gtk_message_dialog_new(parent,
+			GTK_DIALOG_MODAL,
+			GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+			message);
+	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+}
+
+void glame_warning_dialog(const char *message, GtkWindow *parent)
+{
+	GtkWidget *dialog;
+	dialog = gtk_message_dialog_new(parent,
+			GTK_DIALOG_MODAL,
+			GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+			message);
+	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+}
+
+void glame_info_dialog(const char *message, GtkWindow *parent)
+{
+	GtkWidget *dialog;
+	dialog = gtk_message_dialog_new(parent,
+			GTK_DIALOG_MODAL,
+			GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+			message);
+	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+}
+
