@@ -93,8 +93,10 @@ glame_hruler_init (GlameHRuler *hruler)
   GtkWidget *widget;
 
   widget = GTK_WIDGET (hruler);
-  widget->requisition.width = widget->style->klass->xthickness * 2 + 1;
-  widget->requisition.height = widget->style->klass->ythickness * 2 + RULER_HEIGHT;
+  widget->requisition.width = GTK_STYLE(widget->style)->xthickness * 2 + 1;
+  widget->requisition.height = GTK_STYLE(widget->style)->ythickness * 2 + RULER_HEIGHT;
+
+  hruler->layout = gtk_widget_create_pango_layout (widget, NULL);
 }
 
 
@@ -147,15 +149,14 @@ glame_hruler_draw_ticks (GlameRuler *ruler)
   gint scale;			/* Number of units per major unit */
   gdouble subd_incr;
   gdouble start, end, cur;
-  gchar *unit_str;
+  gchar unit_str[32];
+  gint unit_str_len;
   gint digit_height;
   gint digit_offset;
   gint text_width;
   gint pos;
-#if 0
   PangoLayout *layout;
   PangoRectangle logical_rect, ink_rect;
-#endif
 
   g_return_if_fail (ruler != NULL);
   g_return_if_fail (GLAME_IS_HRULER (ruler));
@@ -167,20 +168,12 @@ glame_hruler_draw_ticks (GlameRuler *ruler)
 
   gc = widget->style->fg_gc[GTK_STATE_NORMAL];
   bg_gc = widget->style->bg_gc[GTK_STATE_NORMAL];
-  font = widget->style->font;
+  font = gtk_style_get_font(widget->style);
 
-  xthickness = widget->style->klass->xthickness;
-  ythickness = widget->style->klass->ythickness;
+  xthickness = GTK_STYLE(widget->style)->xthickness;
+  ythickness = GTK_STYLE(widget->style)->ythickness;
 
-#if 0
-  layout = gtk_widget_create_pango_layout (widget, "012456789");
-  pango_layout_get_extents (layout, &ink_rect, &logical_rect);
-  
-  digit_height = PANGO_PIXELS (ink_rect.height) + 1;
-  digit_offset = ink_rect.y;
-#endif
-  digit_height = 10 /* FIXME */;
-  digit_offset = 2 /* FIXME */;
+  layout = GLAME_HRULER (ruler)->layout;
 
   width = widget->allocation.width;
   height = widget->allocation.height - ythickness * 2;
@@ -201,7 +194,7 @@ glame_hruler_draw_ticks (GlameRuler *ruler)
   upper = ruler->upper / ruler->metric->pixels_per_unit;
   lower = ruler->lower / ruler->metric->pixels_per_unit;
 
-  if ((upper - lower) == 0) 
+  if ((upper - lower) <= 0) 
     return;
   increment = (gdouble) width / (upper - lower);
 
@@ -211,12 +204,15 @@ glame_hruler_draw_ticks (GlameRuler *ruler)
    *  for the scale looks consistent with an accompanying vruler
    */
   scale = ceil (ruler->max_size / ruler->metric->pixels_per_unit);
-  unit_str = ruler->metric->translate (scale);
-  text_width = strlen (unit_str) * digit_height + 1;
-  g_free (unit_str);
+  unit_str_len = ruler->metric->translate (scale, unit_str, sizeof(unit_str)-1);
+  pango_layout_set_text (layout, unit_str, unit_str_len);
+  pango_layout_get_extents (layout, &ink_rect, &logical_rect);
+  digit_height = PANGO_PIXELS (ink_rect.height) + 1;
+  digit_offset = ink_rect.y;
+  text_width = unit_str_len * digit_height + 1;
 
   for (scale = 0; scale < ruler->metric->nr_ruler_scale; scale++)
-    if (ruler->metric->ruler_scale[scale] * fabs(increment) > 2 * text_width)
+    if (ruler->metric->ruler_scale[scale] > 2 * text_width / increment)
       break;
 
   if (scale == ruler->metric->nr_ruler_scale)
@@ -228,7 +224,7 @@ glame_hruler_draw_ticks (GlameRuler *ruler)
     {
       subd_incr = (gdouble) ruler->metric->ruler_scale[scale] / 
 	          (gdouble) ruler->metric->subdivide[i];
-      if (subd_incr * fabs(increment) <= MINIMUM_INCR) 
+      if (subd_incr * increment <= MINIMUM_INCR) 
 	continue;
 
       /* Calculate the length of the tickmarks. Make sure that
@@ -261,29 +257,17 @@ glame_hruler_draw_ticks (GlameRuler *ruler)
 	  /* draw label */
 	  if (i == 0)
 	    {
-#if 0
-	      unit_str = ruler->metric->translate (cur);
-	      pango_layout_set_text (layout, unit_str, -1);
-              g_free (unit_str);
+	      unit_str_len = ruler->metric->translate (cur, unit_str, sizeof(unit_str)-1);
+	      pango_layout_set_text (layout, unit_str, unit_str_len);
 
 	      pango_layout_get_extents (layout, &logical_rect, NULL);
 
 	      gdk_draw_layout (ruler->backing_store, gc,
-			       pos + 2, ythickness + PANGO_PIXELS (logical_rect.y - digit_offset),
+			       pos + 2, ythickness + PANGO_PIXELS (logical_rect.y - digit_offset)-2,
 			       layout);
-#endif
-	      unit_str = ruler->metric->translate (cur);
-	      gdk_draw_string(ruler->backing_store, font, gc,
-			      pos + 2, ythickness + font->ascent -1,
-			      unit_str);
-              g_free (unit_str);
 	    }
 	}
     }
-
-#if 0
-  gtk_object_unref (GTK_OBJECT (layout));
-#endif
 }
 
 static void
@@ -307,8 +291,8 @@ glame_hruler_draw_pos (GlameRuler *ruler)
       widget = GTK_WIDGET (ruler);
 
       gc = widget->style->fg_gc[GTK_STATE_NORMAL];
-      xthickness = widget->style->klass->xthickness;
-      ythickness = widget->style->klass->ythickness;
+      xthickness = GTK_STYLE(widget->style)->xthickness;
+      ythickness = GTK_STYLE(widget->style)->ythickness;
       width = widget->allocation.width;
       height = widget->allocation.height - ythickness * 2;
 
@@ -355,13 +339,12 @@ glame_hruler_get_stride (GlameRuler *ruler)
   gdouble increment;		/* Number of pixels per unit */
   gint scale;			/* Number of units per major unit */
   gdouble subd_incr;
-  gchar *unit_str;
+  gchar unit_str[32];
+  gint unit_str_len;
   gint digit_height;
   gint text_width;
-#if 0
   PangoLayout *layout;
   PangoRectangle logical_rect, ink_rect;
-#endif
 
   if (!ruler || !GLAME_IS_HRULER (ruler))
     return 0.0;
@@ -371,17 +354,11 @@ glame_hruler_get_stride (GlameRuler *ruler)
 
   widget = GTK_WIDGET (ruler);
 
-  font = widget->style->font;
+  font = gtk_style_get_font(widget->style);
 
-  ythickness = widget->style->klass->ythickness;
+  ythickness = GTK_STYLE(GTK_OBJECT_GET_CLASS(widget->style))->ythickness;
 
-#if 0
-  layout = gtk_widget_create_pango_layout (widget, "012456789");
-  pango_layout_get_extents (layout, &ink_rect, &logical_rect);
-  
-  digit_height = PANGO_PIXELS (ink_rect.height) + 1;
-#endif
-  digit_height = 10 /* FIXME */;
+  layout = GLAME_HRULER (ruler)->layout;
 
   width = widget->allocation.width;
    
@@ -398,9 +375,11 @@ glame_hruler_get_stride (GlameRuler *ruler)
    *  for the scale looks consistent with an accompanying vruler
    */
   scale = ceil (ruler->max_size / ruler->metric->pixels_per_unit);
-  unit_str = ruler->metric->translate (scale);
-  text_width = strlen (unit_str) * digit_height + 1;
-  g_free (unit_str);
+  unit_str_len = ruler->metric->translate (scale, unit_str, sizeof(unit_str)-1);
+  pango_layout_set_text (layout, unit_str, unit_str_len);
+  pango_layout_get_extents (layout, &ink_rect, &logical_rect);
+  digit_height = PANGO_PIXELS (ink_rect.height) + 1;
+  text_width = unit_str_len * digit_height + 1;
 
   for (scale = 0; scale < ruler->metric->nr_ruler_scale; scale++)
     if (ruler->metric->ruler_scale[scale] * fabs(increment) > 2 * text_width)
