@@ -1,7 +1,7 @@
 /*
  * gtknob.c
  *
- * $Id: gtknob.c,v 1.13 2002/04/25 21:17:34 richi Exp $
+ * $Id: gtknob.c,v 1.19 2003/06/15 13:02:24 richi Exp $
  *
  * Copyright (C) 2000 timecop@japan.co.jp
  * Copyright (C) 2002 Richard Guenther, Laurent Georget
@@ -113,7 +113,7 @@ static void gtk_knob_class_init(GtkKnobClass * klass)
     object_class = (GtkObjectClass *) klass;
     widget_class = (GtkWidgetClass *) klass;
 
-    parent_class = gtk_type_class(gtk_widget_get_type());
+    parent_class = GTK_OBJECT_CLASS(gtk_type_class(gtk_widget_get_type()));
 
     gtk_object_add_arg_type("GtkKnob::adjustment",
 			    GTK_TYPE_ADJUSTMENT,
@@ -134,9 +134,9 @@ static void gtk_knob_class_init(GtkKnobClass * klass)
     widget_class->button_release_event = gtk_knob_button_release;
     widget_class->motion_notify_event = gtk_knob_motion_notify;
     widget_class->expose_event = gtk_knob_expose;
-
-    widget_class->draw = gtk_knob_draw;
-    widget_class->draw_focus = gtk_knob_draw_focus;
+    //FIXME
+    //    widget_class->draw = gtk_knob_draw;
+    //    widget_class->draw_focus = gtk_knob_draw_focus;
 
     widget_class->focus_in_event = gtk_knob_focus_in;
     widget_class->focus_out_event = gtk_knob_focus_out;
@@ -151,7 +151,7 @@ static void gtk_knob_set_arg(GtkObject * object, GtkArg * arg,
 
     switch (arg_id) {
     case ARG_ADJUSTMENT:
-	gtk_knob_set_adjustment(GTK_KNOB(knob), GTK_VALUE_POINTER(*arg));
+	gtk_knob_set_adjustment(GTK_KNOB(knob), GTK_ADJUSTMENT(GTK_VALUE_POINTER(*arg)));
 	break;
     default:
 	break;
@@ -213,9 +213,12 @@ static void gtk_knob_destroy(GtkObject * object)
 
     knob = GTK_KNOB(object);
 
-    if (knob->adjustment)
+    if (knob->adjustment) {
 	gtk_signal_disconnect_by_data(GTK_OBJECT(knob->adjustment),
 				      (gpointer) knob);
+	g_object_unref(knob->adjustment);
+	knob->adjustment = NULL;
+    }
 
     /* set value back to 0 */
     knob->value = 0;
@@ -498,7 +501,7 @@ static gint gtk_knob_focus_in(GtkWidget * widget, GdkEventFocus * event)
     g_return_val_if_fail(GTK_IS_KNOB(widget), FALSE);
 
     GTK_WIDGET_SET_FLAGS(widget, GTK_HAS_FOCUS);
-    gtk_widget_draw_focus(widget);
+    //FIXME    gtk_widget_draw_focus(widget);
 
     return FALSE;
 }
@@ -509,7 +512,7 @@ static gint gtk_knob_focus_out(GtkWidget * widget, GdkEventFocus * event)
     g_return_val_if_fail(GTK_IS_KNOB(widget), FALSE);
 
     GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
-    gtk_widget_draw_focus(widget);
+    //FIXME gtk_widget_draw_focus(widget);
 
     return FALSE;
 }
@@ -531,7 +534,7 @@ static void gtk_knob_paint(GtkKnob * knob, GdkRectangle * area)
 		    knob->value * 31, 0, 15, 10, 30, 30);
 
     if (GTK_WIDGET_HAS_FOCUS(widget)) {
-	gtk_paint_focus(widget->style, widget->window,
+	    gtk_paint_focus(widget->style, widget->window,GTK_STATE_NORMAL,
 			area, widget, "knob",
 			widget->allocation.x, widget->allocation.y,
 			widget->allocation.width - 1,
@@ -596,7 +599,7 @@ static void gtk_knob_adjustment_value_changed(GtkAdjustment * adjustment,
 }
 
 
-#ifdef HAVE_LIBGLADE
+#if defined HAVE_LIBGLADE
 #include "glscript.h"
 
 static gchar *gtk_knob_scheme_formatter(gfloat value, char *code)
@@ -615,16 +618,17 @@ static gchar *gtk_knob_scheme_formatter(gfloat value, char *code)
 	return res;
 }
 
-static GtkWidget *gtk_knob_glade_new(GladeXML *xml, GladeWidgetInfo *info)
+static GtkWidget *gtk_knob_glade_new(GladeXML *xml, GType widget_type,
+				     GladeWidgetInfo *info)
 {
 	GtkWidget *knob;
 	GtkObject *adj;
-	GList *tmp;
 	float value = 0.0, lower = 0.0, upper = 1.0, tick;
 	char *formatter = NULL;
+	int i;
 
-	for (tmp = info->attributes; tmp; tmp = tmp->next) {
-		GladeAttribute *attr = tmp->data;
+	for (i = 0; i< info->n_properties; ++i) {
+		GladeProperty *attr = &info->properties[i];
 		if (strcmp(attr->name, "value") == 0)
 			sscanf(attr->value, "%f", &value);
 		if (strcmp(attr->name, "lower") == 0)
@@ -638,8 +642,8 @@ static GtkWidget *gtk_knob_glade_new(GladeXML *xml, GladeWidgetInfo *info)
 	knob = gtk_knob_new(GTK_ADJUSTMENT(adj));
 	if (formatter)
 		gtk_knob_set_formatter(GTK_KNOB(knob), (GtkKnobFormatter)gtk_knob_scheme_formatter, g_strdup(formatter));
-	for (tmp = info->attributes; tmp; tmp = tmp->next) {
-		GladeAttribute *attr = tmp->data;
+	for (i = 0; i< info->n_properties; ++i) {
+		GladeProperty *attr = &info->properties[i];
 		if (strcmp(attr->name, "tick") == 0) {
 			sscanf(attr->value, "%f", &tick);
 			gtk_knob_add_tick(GTK_KNOB(knob), tick);
@@ -751,15 +755,10 @@ void gtk_knob_add_tick(GtkKnob *knob, gfloat tick)
 }
 
 
-
-#ifdef HAVE_LIBGLADE
+#if defined HAVE_LIBGLADE
 void gtk_knob_glade_register()
 {
-	static GladeWidgetBuildData widgets[] = {
-		{ "GtkKnob", gtk_knob_glade_new, NULL },
-		{ NULL, NULL, NULL }
-	};
-	glade_register_widgets(widgets);
+	glade_register_widget(gtk_knob_get_type(), gtk_knob_glade_new, NULL, NULL);
 }
 #else
 void gtk_knob_glade_register()
