@@ -1,6 +1,8 @@
 /*
  * waveeditgui.c
  *
+ * $Id: waveeditgui.c,v 1.152 2005/03/11 17:02:39 richi Exp $
+ *
  * Copyright (C) 2001, 2002, 2003 Richard Guenther
  *
  * This program is free software; you can redistribute it and/or modify
@@ -187,6 +189,7 @@ static void copy_cb(GtkWidget *bla, GtkWaveView *waveview);
 static void paste_cb(GtkWidget *bla, GtkWaveView *waveview);
 static void replace_cb(GtkWidget *bla, GtkWaveView *waveview);
 static void mix_cb(GtkWidget *bla, GtkWaveView *waveview);
+static void crop_cb(GtkWidget *bla, GtkWaveView *waveview);
 static void cut_cb(GtkWidget *bla, GtkWaveView *waveview);
 static void delete_cb(GtkWidget *bla, GtkWaveView *waveview);
 static void undo_cb(GtkWidget *bla, GtkWaveView *waveview);
@@ -198,6 +201,7 @@ static GnomeUIInfo edit_menu[] = {
 	GNOMEUIINFO_ITEM(N_("Paste"), NULL, paste_cb, NULL),
 	GNOMEUIINFO_ITEM(N_("Replace"), NULL, replace_cb, NULL),
 	GNOMEUIINFO_ITEM(N_("Mix"), NULL, mix_cb, NULL),
+	GNOMEUIINFO_ITEM(N_("Crop to selection"), NULL, crop_cb, NULL),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_ITEM(N_("Delete"), NULL, delete_cb, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -210,9 +214,10 @@ static GnomeUIInfo edit_menu[] = {
 #define EDIT_MENU_PASTE_INDEX 2
 #define EDIT_MENU_REPLACE_INDEX 3
 #define EDIT_MENU_MIX_INDEX 4
-#define EDIT_MENU_DELETE_INDEX 6
-#define EDIT_MENU_UNDO_INDEX 8
-#define EDIT_MENU_REDO_INDEX 9
+#define EDIT_MENU_CROP_INDEX 5
+#define EDIT_MENU_DELETE_INDEX 7
+#define EDIT_MENU_UNDO_INDEX 9
+#define EDIT_MENU_REDO_INDEX 10
 
 /* Menu event - Copy. */
 static void copy_cb(GtkWidget *bla, GtkWaveView *waveview)
@@ -228,9 +233,11 @@ static void copy_cb(GtkWidget *bla, GtkWaveView *waveview)
 	if (length <= 0)
 		return;
 
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	if (clipboard_copy(item, start, length) == -1)
 		DPRINTF("Failed copying\n");
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Paste. */
@@ -246,15 +253,19 @@ static void paste_cb(GtkWidget *bla, GtkWaveView *waveview)
 	pos = gtk_wave_view_get_marker (waveview);
 	if (pos < 0)
 		return;
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	oldsize = gpsm_item_hsize(item);
-	if (!clipboard_can_paste(item))
+	if (!clipboard_can_paste(item)) {
+		gpsm_item_destroy(item);
 		return;
+	}
 	if (gpsm_op_prepare(item) == -1)
 		DPRINTF("Error preparing for undo\n");
 	if (clipboard_paste(item, pos) == -1)
 		DPRINTF("Error pasting\n");
 	gtk_wave_view_set_selection(waveview, pos, gpsm_item_hsize(item) - oldsize);
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Replace. */
@@ -270,15 +281,19 @@ static void replace_cb(GtkWidget *bla, GtkWaveView *waveview)
 	pos = gtk_wave_view_get_marker (waveview);
 	if (pos < 0)
 		return;
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	size = clipboard_hsize();
-	if (!clipboard_can_paste(item))
+	if (!clipboard_can_paste(item)) {
+		gpsm_item_destroy(item);
 		return;
+	}
 	if (gpsm_op_prepare(item) == -1)
 		DPRINTF("Error preparing for undo\n");
 	if (clipboard_replace(item, pos) == -1)
 		DPRINTF("Error replacing\n");
 	gtk_wave_view_set_selection(waveview, pos, size);
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Mix. */
@@ -294,15 +309,19 @@ static void mix_cb(GtkWidget *bla, GtkWaveView *waveview)
 	pos = gtk_wave_view_get_marker (waveview);
 	if (pos < 0)
 		return;
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	size = clipboard_hsize();
-	if (!clipboard_can_paste(item))
+	if (!clipboard_can_paste(item)) {
+		gpsm_item_destroy(item);
 		return;
+	}
 	if (gpsm_op_prepare(item) == -1)
 		DPRINTF("Error preparing for undo\n");
 	if (clipboard_mix(item, pos) == -1)
 		DPRINTF("Error mixing\n");
 	gtk_wave_view_set_selection(waveview, pos, size);
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Cut. */
@@ -319,7 +338,8 @@ static void cut_cb(GtkWidget *bla, GtkWaveView *waveview)
 	if (length <= 0)
 		return;
 
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	if (gpsm_op_prepare(item) == -1)
 		DPRINTF("Error preparing for undo\n");
 	if (clipboard_cut(item, start, length) == -1)
@@ -327,6 +347,7 @@ static void cut_cb(GtkWidget *bla, GtkWaveView *waveview)
 
 	/* Remove the selection. */
 	selectnone_cb(bla, waveview);
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Delete. */
@@ -343,7 +364,8 @@ static void delete_cb(GtkWidget *bla, GtkWaveView *waveview)
 	if (length <= 0)
 		return;
 
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	if (gpsm_op_prepare(item) == -1)
 		DPRINTF("Error preparing for undo\n");
 	if (clipboard_delete(item, start, length) == -1)
@@ -351,6 +373,33 @@ static void delete_cb(GtkWidget *bla, GtkWaveView *waveview)
 
 	/* Remove the selection. */
 	selectnone_cb(bla, waveview);
+	gpsm_item_destroy(item);
+}
+
+/* Menu event - Crop to selection. */
+static void crop_cb(GtkWidget *bla, GtkWaveView *waveview)
+{
+	GtkWaveBuffer *wavebuffer;
+	GtkSwapfileBuffer *swapfile;
+	gint32 start, length, total_size;
+	gpsm_item_t *item;
+
+	wavebuffer = gtk_wave_view_get_buffer(waveview);
+	swapfile = GTK_SWAPFILE_BUFFER(wavebuffer);
+	gtk_wave_view_get_selection (waveview, &start, &length);
+	if (length <= 0)
+		return;
+
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
+	total_size = gpsm_item_hsize(item);
+	if (gpsm_op_prepare(item) == -1)
+		DPRINTF("Error preparing for undo\n");
+	if (clipboard_delete(item, 0, start-1) == -1
+	    || clipboard_delete(item, length, total_size-start-1) == -1)
+		DPRINTF("Error cropping\n");
+
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Undo. */
@@ -360,11 +409,12 @@ static void undo_cb(GtkWidget *bla, GtkWaveView *waveview)
 	gpsm_item_t *item;
 
 	swapfile = GTK_SWAPFILE_BUFFER(gtk_wave_view_get_buffer(waveview));
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 
 	if (gpsm_op_undo(item) == -1)
-		gnome_dialog_run_and_close(
-			GNOME_DIALOG(gnome_error_dialog(_("Error during undo"))));
+		glame_error_dialog(_("Error during undo"), NULL);
+	gpsm_item_destroy(item);
 }
 
 /* Menu event - Redo. */
@@ -374,11 +424,12 @@ static void redo_cb(GtkWidget *bla, GtkWaveView *waveview)
 	gpsm_item_t *item;
 
 	swapfile = GTK_SWAPFILE_BUFFER(gtk_wave_view_get_buffer(waveview));
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 
 	if (gpsm_op_redo(item) == -1)
-		gnome_dialog_run_and_close(
-			GNOME_DIALOG(gnome_error_dialog(_("Error during redo"))));
+		glame_error_dialog(_("Error during redo"), NULL);
+	gpsm_item_destroy(item);
 }
 
 
@@ -389,7 +440,11 @@ static void redo_cb(GtkWidget *bla, GtkWaveView *waveview)
  * Complex stuff like apply, play, record, etc.
  */
 
-static void playrecordtoolbar_cb(GtkWidget *bla, GtkWaveView *waveview);
+#define TOOLBAR_PLAY_DEL 6
+#define TOOLBAR_PLAY_INS 9
+
+static void recordtoolbar_cb(GtkWidget *bla, GtkWaveView *waveview);
+static void playtoolbar_cb(GtkWidget *bla, GtkWaveView *waveview);
 
 /* Menu event - Apply operation. */
 static void applyop_cb(GtkWidget *bla, plugin_t *plugin)
@@ -399,12 +454,12 @@ static void applyop_cb(GtkWidget *bla, plugin_t *plugin)
 	GtkSwapfileBuffer *swapfile = GTK_SWAPFILE_BUFFER(wavebuffer);
 	gint32 start, length;
 	gpsm_grp_t *grp;
-	int (*operation)(gpsm_item_t *, long, long);
+	gpsmop_func_t operation;
 
 	gtk_wave_view_get_selection (waveview, &start, &length);
 	grp = gtk_swapfile_buffer_get_item(swapfile);
 
-	if (!(operation = plugin_query(plugin, PLUGIN_GPSMOP))) {
+	if (!(operation = (gpsmop_func_t)plugin_query(plugin, PLUGIN_GPSMOP))) {
 		DPRINTF("No such operation %s\n", plugin_name(plugin));
 		return;
 	}
@@ -413,8 +468,7 @@ static void applyop_cb(GtkWidget *bla, plugin_t *plugin)
 		(long)start, (long)start+length);
 
 	if (operation((gpsm_item_t *)grp, start, length) == -1)
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("Error executing"))));
+		glame_error_dialog(_("Error executing"), NULL);
 
 	DPRINTF("%s finished.\n", plugin_name(plugin));
 }
@@ -431,8 +485,10 @@ static void apply_cb(GtkWidget *bla, plugin_t *plugin)
 	gpsm_item_t *item;
 	
 	gtk_wave_view_get_selection (waveview, &start, &length);
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+				gtk_wave_view_get_select_channels(waveview));
 	gpsmop_apply_plugin(item, plugin, start, length);
+	gpsm_item_destroy(item);
 }
 
 
@@ -450,13 +506,13 @@ static void play_cleanup(glsig_handler_t *handler,
 		gtk_wave_view_set_marker(GTK_WAVE_VIEW(waveedit->waveview),
 					 waveedit->pm_marker);
 
-	/* restore normal play button -- wheee, gtk suxx. */
+	/* Restore normal play button.  */
 	gtk_widget_destroy(g_list_nth(gtk_container_children(
-		GTK_CONTAINER(waveedit->toolbar)), 6)->data);
-	gtk_toolbar_insert_item(GTK_TOOLBAR(waveedit->toolbar),
-				_("Play/Record"), _("PlayRecord"), _("PlayRecord"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_FORWARD),
-				playrecordtoolbar_cb, waveedit->waveview, 9);
+		GTK_CONTAINER(waveedit->toolbar)), TOOLBAR_PLAY_DEL)->data);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(waveedit->toolbar),
+ 				 GTK_STOCK_MEDIA_PLAY, _("Play"), _("Play"),
+ 				 GTK_SIGNAL_FUNC(playtoolbar_cb), waveedit->waveview,
+ 				 TOOLBAR_PLAY_INS);
 
         /* Scan network for swapfile_out nodes and issue gpsm invalidate
          * signals. */
@@ -488,7 +544,7 @@ static void play_cleanup(glsig_handler_t *handler,
 			if (xruns > 0) {
 				char msg[64];
 				snprintf(msg, 64, "Had %li XRUNs", xruns);
-				gnome_dialog_run_and_close(GNOME_DIALOG(gnome_warning_dialog(msg)));
+				glame_warning_dialog(msg, NULL);
 			}
 		}
 		waveedit->pm_ain = NULL;
@@ -530,6 +586,8 @@ static void play(GtkWaveView *waveview,
 	glsig_emitter_t *emitter;
 	gboolean *flg_rec, *flg_mute;
 
+	DPRINTF("play: start %i, end %i, rec_start %i, restore_marker %i, loop %i, extend %i, enable_record %i\n", start, end, rec_start, restore_marker, loop, extend, enable_record);
+
 	/* A simple state machine with two states:
 	 * - not playing, a button press will start playing
 	 * - playing, a button press will stop playing
@@ -559,17 +617,22 @@ static void play(GtkWaveView *waveview,
 		if (flg_rec[i-1])
 			rec_cnt++;
 	}
-	if (play_cnt == 0 && rec_cnt == 0)
-		return;
 
 	if (play_cnt != 0 && !plugin_get("audio_out")) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("No audio output support"))));
+		glame_error_dialog(_("No audio output support"), NULL);
 		return;
 	}
 	if (rec_cnt != 0 && !plugin_get("audio_in")) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(
-			gnome_error_dialog(_("No audio input support"))));
+		glame_error_dialog(_("No audio input support"), NULL);
+		return;
+	}
+
+	if (play_cnt == 0 && !enable_record) {
+		glame_error_dialog(_("You did not mark any track for playback?"), NULL);
+		return;
+	}
+	if (rec_cnt == 0 && enable_record) {
+		glame_error_dialog(_("You did not mark any track for recording?"), NULL);
 		return;
 	}
 
@@ -683,13 +746,13 @@ static void play(GtkWaveView *waveview,
 		return;
 	}
 
-	/* exchange play for stop button -- wheee, gtk suxx. */
+	/* Exchange play for stop button.  */
 	gtk_widget_destroy(g_list_nth(gtk_container_children(
-		GTK_CONTAINER(active_waveedit->toolbar)), 6)->data);
-	gtk_toolbar_insert_item(GTK_TOOLBAR(active_waveedit->toolbar),
-				_("Stop"), _("Stop"), _("Stop"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_STOP),
-				playrecordtoolbar_cb, active_waveedit->waveview, 9);
+		GTK_CONTAINER(active_waveedit->toolbar)), TOOLBAR_PLAY_DEL)->data);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(active_waveedit->toolbar),
+ 				 GTK_STOCK_MEDIA_STOP, _("Stop"), _("Stop"),
+ 				 GTK_SIGNAL_FUNC(playtoolbar_cb), active_waveedit->waveview,
+ 				 TOOLBAR_PLAY_INS);
 	active_waveedit->locked = 1;
 
 	return;
@@ -697,20 +760,36 @@ static void play(GtkWaveView *waveview,
 fail_ain:
 	filter_delete(net);
 	gpsm_item_destroy((gpsm_item_t *)grp);
-	gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(
-		"Your audio device cannot record to so much tracks")));
+	glame_error_dialog(_("Your audio device cannot record to so much tracks"), NULL);
 	return;
 
 fail_other:
 	filter_delete(net);
 	gpsm_item_destroy((gpsm_item_t *)grp);
-	gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(
-		"Error constructing recording/playing network")));
+	glame_error_dialog(_("Error constructing recording/playing network"), NULL);
 	return;
 }
 
 
-static void playrecordtoolbar_cb(GtkWidget *bla, GtkWaveView *waveview)
+static void playtoolbar_cb(GtkWidget *bla, GtkWaveView *waveview)
+{
+	GtkWaveBuffer *wavebuffer = gtk_wave_view_get_buffer(waveview);
+	gint32 start, end;
+
+	/* Choose between playing the actual selection and from
+	 * the current marker position (where the first one restores
+	 * marker position after play). */
+	gtk_wave_view_get_selection(waveview, &start, &end);
+	end = start + end;
+	if (end - start <= 0) {
+		start = MAX(0, gtk_wave_view_get_marker (waveview));
+		end = gtk_wave_buffer_get_length(wavebuffer);
+		play(waveview, start, end, start, FALSE, FALSE, TRUE, FALSE);
+	} else
+		play(waveview, start, end, start, TRUE, FALSE, FALSE, FALSE);
+}
+
+static void recordtoolbar_cb(GtkWidget *bla, GtkWaveView *waveview)
 {
 	GtkWaveBuffer *wavebuffer = gtk_wave_view_get_buffer(waveview);
 	gint32 start, end;
@@ -747,8 +826,7 @@ static void playselection_cb(GtkWidget *widget, GtkWaveView *waveview)
 	gtk_wave_view_get_selection(waveview, &start, &end);
 	end = start + end;
 	if (end - start <= 0) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(
-			_("Nothing selected"))));
+		glame_error_dialog(_("Nothing selected"), NULL);
 		return;
 	}
 	play(waveview, start, end, start, TRUE, FALSE, FALSE, FALSE);
@@ -785,8 +863,7 @@ static void recordselection_cb(GtkWidget *widget, GtkWaveView *waveview)
 	end = start + end;
 	rec_start = start;
 	if (end - start <= 0) {
-		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(
-			_("Nothing selected"))));
+		glame_error_dialog(_("Nothing selected"), NULL);
 		return;
 	}
 	/* check for play-before-record
@@ -807,11 +884,10 @@ static void apply_custom_cb(GtkWidget * foo, GtkWaveView *waveview)
 	GtkSwapfileBuffer *swapfile = GTK_SWAPFILE_BUFFER(wavebuffer);
 	GtkWidget *canvas;
 	gint32 start, length, marker, wavesize;
-	long nrtracks;
-	gpsm_swfile_t **files;
-	gpsm_item_t *item;
+	gpsm_grp_t *files;
+	gpsm_item_t *item, *it;
 	filter_t *net;
-	int rate, i;
+	int rate;
 	float y_position = 20.0;
 	char position_buffer[20];
 
@@ -821,18 +897,19 @@ static void apply_custom_cb(GtkWidget * foo, GtkWaveView *waveview)
 	if (length <= 0 && marker < 0)
 		return;
 
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
-	nrtracks = gtk_swapfile_buffer_get_swfiles(swapfile, &files);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
+	files = gpsm_collect_swfiles(item);
 	rate = gtk_wave_buffer_get_rate(wavebuffer);
 	DPRINTF("Applying to [%li, +%li]\n", (long)start, (long)length);
 
 	/* Create the network, add nrtracks instances of swapfile_in/out */
 	net = filter_creat(NULL);
-	for (i=0; i<nrtracks; i++) {
+	gpsm_grp_foreach_item(files, it) {
 		filter_t *swin, *swout;
 		if (length <= 0 && wavesize - marker <= 0)
 			goto no_swin;
-		swin = net_add_gpsm_input(net, files[i],
+		swin = net_add_gpsm_input(net, (gpsm_swfile_t *)it,
 					  length > 0 ? start : marker,
 					  length > 0 ? length : -1, 0);
 		if (!swin)
@@ -844,7 +921,7 @@ static void apply_custom_cb(GtkWidget * foo, GtkWaveView *waveview)
 		filter_set_property(swin,"canvas_y",position_buffer);
 
 	no_swin:
-		swout = net_add_gpsm_output(net, files[i],
+		swout = net_add_gpsm_output(net, (gpsm_swfile_t *)it,
 					    length > 0 ? start : marker,
 					    length > 0 ? length : -1,
 					    0);
@@ -858,10 +935,12 @@ static void apply_custom_cb(GtkWidget * foo, GtkWaveView *waveview)
 
 		y_position += 100;
 	}
+	gpsm_item_destroy((gpsm_item_t *)files);
 
 	/* Prepare for undo -- NO(!?). FIXME -- Yes, still FIXME. */
 	if (gpsm_op_prepare(item) == -1)
 		DPRINTF("Error preparing for undo\n");
+	gpsm_item_destroy(item);
 
 	/* Pop up the custom generated canvas - the wave widget is
 	 * updated by filtereditgui, if changed. */
@@ -869,8 +948,9 @@ static void apply_custom_cb(GtkWidget * foo, GtkWaveView *waveview)
 	return;
 
  fail:
-	gnome_dialog_run_and_close(GNOME_DIALOG(
-		gnome_error_dialog(_("Failed to create network"))));
+	glame_error_dialog(_("Failed to create network"), NULL);
+	gpsm_item_destroy(item);
+	gpsm_item_destroy((gpsm_item_t *)files);
 	filter_delete(net);
 }
 
@@ -884,18 +964,33 @@ static void wave_export_cb(GtkWidget *foo, void *bar)
 
 static void wave_help_cb(GtkWidget *foo, void*bar)
 {
-	gnome_help_goto(NULL,"info:glame#The_Wave_Editor");
+	glame_help_goto(NULL,"info:glame#The_Wave_Editor");
 }
 
+static void wave_close_if_yes_cb(GtkDialog *dialog, gint response,
+				 GtkObject *waveedit)
+{
+	if (response == GTK_RESPONSE_YES)
+		gtk_object_destroy(GTK_OBJECT(waveedit));
+}
 static void wave_close_cb(GtkWidget *foo, GtkObject *window)
 {
 	if (!active_waveedit
 	    || active_waveedit->locked
 	    || GTK_WAVE_VIEW(active_waveedit->waveview)->drawing)
 		return;
-	if (active_waveedit->modified)
-		if (gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_cancel_dialog_modal_parented("Wave modified.\nDo you really want to close?", NULL, NULL, GTK_WINDOW(active_waveedit)))) == 1)
-			return;
+	if (active_waveedit->modified) {
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new(GTK_WINDOW(active_waveedit),
+			GTK_DIALOG_MODAL,
+			GTK_MESSAGE_QUESTION,
+			GTK_BUTTONS_YES_NO,
+			_("Wave modified.\nDo you really want to close without saving?"));
+		g_signal_connect (dialog, "response", G_CALLBACK (wave_close_if_yes_cb), active_waveedit);
+		gtk_dialog_run(GTK_DIALOG(dialog));
+		gtk_widget_destroy(dialog);
+		return;
+	}
 	gtk_object_destroy(GTK_OBJECT(active_waveedit));
 }
 
@@ -948,6 +1043,8 @@ static int choose_effects(plugin_t *plugin)
 	/* We need input and output port(s) of the same count, if
          * greater than one it needs to match the channel count. */
 	filterportdb_foreach_port(filter_portdb(filter), port) {
+		if (filterport_get_property(port, "!CONTROL")) /* ignore LADSPA control ports */
+			continue;
 		if (!FILTER_PORTS_ARE_COMPATIBLE(filterport_type(port), FILTER_PORTTYPE_SAMPLE))
 			return 0;
 		if (filterport_is_input(port))
@@ -991,13 +1088,12 @@ static GtkWidget *waveedit_build_menu(GtkWaveView *waveview)
 	GtkMenu *filter_menu, *op_menu;
 	gpsm_item_t *item;
 	gint32 sel_start, sel_length, marker_pos;
-	guint32 nrtracks;
 
 	/* Get stuff we need for enabling/disabling items. */
 	wavebuffer = gtk_wave_view_get_buffer (waveview);
 	swapfile = GTK_SWAPFILE_BUFFER(wavebuffer);
-	item = (gpsm_item_t *)gtk_swapfile_buffer_get_item(swapfile);
-	nrtracks = gtk_wave_buffer_get_num_channels(wavebuffer);
+	item = (gpsm_item_t *)gtk_swapfile_buffer_get_active(swapfile,
+			gtk_wave_view_get_select_channels(waveview));
 	gtk_wave_view_get_selection(waveview, &sel_start, &sel_length);
 	marker_pos = gtk_wave_view_get_marker(waveview);
 
@@ -1035,6 +1131,8 @@ static GtkWidget *waveedit_build_menu(GtkWaveView *waveview)
 	gtk_widget_set_sensitive(edit_menu[EDIT_MENU_MIX_INDEX].widget,
 				 clipboard_can_paste(item)
 				 && (marker_pos >= 0) ? TRUE : FALSE);
+	gtk_widget_set_sensitive(edit_menu[EDIT_MENU_CROP_INDEX].widget,
+				 (sel_length > 0) ? TRUE : FALSE);
 	gtk_widget_set_sensitive(edit_menu[EDIT_MENU_DELETE_INDEX].widget,
 				 (sel_length > 0) ? TRUE : FALSE);
 	gtk_widget_set_sensitive(edit_menu[EDIT_MENU_UNDO_INDEX].widget,
@@ -1042,6 +1140,7 @@ static GtkWidget *waveedit_build_menu(GtkWaveView *waveview)
 	gtk_widget_set_sensitive(edit_menu[EDIT_MENU_REDO_INDEX].widget,
 				 gpsm_op_can_redo(item) ? TRUE : FALSE);
 
+	gpsm_item_destroy(item);
 	return menu;
 }
 
@@ -1056,16 +1155,15 @@ static gint waveedit_rmb_cb(GtkWidget *widget, GdkEventButton *event,
 		return FALSE;
 
 	menu = waveedit_build_menu(waveview);
-	gnome_popup_menu_do_popup(menu, NULL, NULL, event, waveview);
+	gnome_popup_menu_do_popup(menu, NULL, NULL, event, waveview,GTK_WIDGET(waveview));
 	return FALSE;
 }
 
-static gint handle_enter(GtkWidget *tree, GdkEventCrossing *event,
+static void handle_enter(GtkWidget *tree, GdkEventCrossing *event,
 			 WaveeditGui *waveedit)
 {
 	if (event->type == GDK_ENTER_NOTIFY)
 		active_waveedit = waveedit;
-	return FALSE;
 }
 
 static gint event_block(GtkObject *object, GdkEventAny *event, gpointer data)
@@ -1309,14 +1407,16 @@ static void waveedit_gui_destroy(GtkObject *waveedit)
 	GnomeAppClass* parent_class;
 	parent_class = gtk_type_class(gnome_app_get_type());
 	GTK_OBJECT_CLASS(parent_class)->destroy(waveedit);
-	if (WAVEEDIT_GUI(waveedit)->swfiles)
+	if (WAVEEDIT_GUI(waveedit)->swfiles){
 		gpsm_item_destroy((gpsm_item_t *)WAVEEDIT_GUI(waveedit)->swfiles);
+		WAVEEDIT_GUI(waveedit)->swfiles = NULL;
+	}
 }
 
-static void waveedit_gui_class_init(WaveeditGuiClass *class)
+static void waveedit_gui_class_init(WaveeditGuiClass *klass)
 {
 	GtkObjectClass *object_class;
-	object_class = GTK_OBJECT_CLASS(class);
+	object_class = GTK_OBJECT_CLASS(klass);
 	object_class->destroy = waveedit_gui_destroy;
 }
 
@@ -1348,7 +1448,6 @@ GtkType waveedit_gui_get_type(void)
 			NULL,NULL,(GtkClassInitFunc)NULL,};
 		waveedit_gui_type = gtk_type_unique(
 			gnome_app_get_type(), &waveedit_gui_info);
-		gtk_type_set_chunk_alloc(waveedit_gui_type, 8);
 	}
 
 	return waveedit_gui_type;
@@ -1356,7 +1455,7 @@ GtkType waveedit_gui_get_type(void)
 
 static void help_cb(GtkWidget *menu, void *blah)
 {
-	gnome_help_goto(NULL, "info:glame#The_Wave_Editor");
+	glame_help_goto(NULL, "info:glame#The_Wave_Editor");
 }
 static void list_keybindings_cb(GtkWidget *menu, void *blah)
 {
@@ -1471,52 +1570,51 @@ WaveeditGui *glame_waveedit_gui_new(const char *title, gpsm_item_t *item)
 
 
 	/* Add the toolbar. */
-	window->toolbar = gtk_toolbar_new(GTK_ORIENTATION_VERTICAL,
-					  GTK_TOOLBAR_ICONS);
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("Export"), _("Export"), _("Export"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_SAVE),
-				wave_export_cb, window);
+	window->toolbar = gtk_toolbar_new();
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_SAVE,
+				 _("Export"), _("Export"),
+				 GTK_SIGNAL_FUNC(wave_export_cb), window,-1);
 	gtk_toolbar_append_space(GTK_TOOLBAR(window->toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("Zoom in"), _("Zoom in"), _("Zoom in"),
-				glame_load_icon_widget("zoom_in.png",24,24),
-				zoomin_cb, window->waveview);
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("Zoom out"), _("Zoom out"), _("Zoom out"),
-				glame_load_icon_widget("zoom_out.png",24,24),
-				zoomout_cb, window->waveview);
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("View all"), _("View all"), _("View all"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_REFRESH),
-				zoomfull_cb, window->waveview);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_ZOOM_IN,
+				 _("Zoom in"), _("Zoom in"), 
+				 GTK_SIGNAL_FUNC(zoomin_cb), window->waveview,-1);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_ZOOM_OUT,
+				 _("Zoom out"), _("Zoom out"), 
+				 GTK_SIGNAL_FUNC(zoomout_cb), window->waveview,-1);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_ZOOM_FIT,
+				 _("View all"), _("View all"), 
+				 GTK_SIGNAL_FUNC(zoomfull_cb), window->waveview,-1);
 	gtk_toolbar_append_space(GTK_TOOLBAR(window->toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
 				_("Select all"), _("Select all"), _("Select all"),
 				glame_load_icon_widget("select_all.png",24,24),
-				selectall_cb, window->waveview);
+				GTK_SIGNAL_FUNC(selectall_cb), window->waveview);
 	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
 				_("Select none"), _("Select none"), _("Select none"),
 				glame_load_icon_widget("select_none.png",24,24),
-				selectnone_cb, window->waveview);
-	/* Play button that should change to Stop if pressed, different
-	 * callback than "Play all"/"Play selection" - play from marker.
-	 * FIXME. */
+				GTK_SIGNAL_FUNC(selectnone_cb), window->waveview);
 	gtk_toolbar_append_space(GTK_TOOLBAR(window->toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("Play/Record"), _("Play/Record"), _("Play/Record"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_FORWARD),
-				playrecordtoolbar_cb, window->waveview);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_MEDIA_PLAY, _("Play"), _("Play"),
+				 GTK_SIGNAL_FUNC(playtoolbar_cb), window->waveview, -1);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_MEDIA_RECORD, _("Record"), _("Record"),
+				 GTK_SIGNAL_FUNC(recordtoolbar_cb), window->waveview, -1);
+
 	/* Keep last. */
 	gtk_toolbar_append_space(GTK_TOOLBAR(window->toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("Close"), _("Close"), _("Close"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_CLOSE),
-				wave_close_cb, window);
-	gtk_toolbar_append_item(GTK_TOOLBAR(window->toolbar),
-				_("Help"), _("Help"), _("Help"),
-				gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_HELP),
-				wave_help_cb, window->waveview);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_CLOSE,
+				 _("Close"), _("Close"),
+				 GTK_SIGNAL_FUNC(wave_close_cb), window, -1);
+	gtk_toolbar_insert_stock(GTK_TOOLBAR(window->toolbar),
+				 GTK_STOCK_HELP,
+				 _("Help"), _("Help"),
+				 GTK_SIGNAL_FUNC(wave_help_cb), window->waveview, -1);
 
 	/* Create menubar - FIXME copy all uiinfos, restructure to
 	 * match nice menu layout, etc.
@@ -1525,12 +1623,6 @@ WaveeditGui *glame_waveedit_gui_new(const char *title, gpsm_item_t *item)
 	//gnome_app_install_menu_hints(GNOME_APP(window), window_menu);
 
 	gnome_app_set_toolbar(GNOME_APP(window), GTK_TOOLBAR(window->toolbar));
-#if 0
-	gnome_app_add_toolbar(GNOME_APP(window), GTK_TOOLBAR(window->toolbar),
-			      "waveedit::toolbar",
-			      /*GNOME_DOCK_ITEM_BEH_EXCLUSIVE|*/GNOME_DOCK_ITEM_BEH_NEVER_FLOATING,
-			      GNOME_DOCK_TOP, 0, 0, 0);
-#endif
 
 	/* Install the rmb menu and enter/leave callbacks. */
 	gtk_signal_connect(GTK_OBJECT(window->waveview), "button_press_event",
