@@ -75,7 +75,7 @@
 */
 
 #define scm_c_define_gsubr(func_name, req, opt, rest, func) gh_new_procedure(func_name, func, req, opt, rest)
-#define scm_c_export(sym, foo) /* nothing */
+#define scm_c_export(sym, ...) /* nothing */
 #define scm_c_define(sym, val) gh_define (sym, val)
 #define scm_long2num(num) gh_long2scm (num)
 #define scm_double2num(num) gh_double2scm (num)
