@@ -4,7 +4,7 @@
 /*
  * list.h
  *
- * $Id: list.h,v 1.21.6.2 2004/10/23 13:14:23 richi Exp $
+ * $Id: list.h,v 1.26 2004/12/25 19:17:50 richi Exp $
  * 
  * Copyright (C) 1999, 2000, 2001, 2004 Richard Guenther
  *
@@ -31,6 +31,7 @@
 #include <config.h>
 #endif
 
+#include <stddef.h>
 #include "util.h"
 
 /*
@@ -203,7 +204,7 @@ static inline void glame_list_unsplice(struct glame_list_head *list,
 /* Get a pointer to the specified type out of a ptr which points to the
  * struct glame_list_head inside type at struct member member. */
 #define glame_list_entry(ptr, type, member) \
-	((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))
+	((type *)((char *)(ptr)-offsetof(type, member)))
 
 /* Get head/tail entry of list like glame_list_entry(), but NULL if list is empty. */
 #define glame_list_gethead(listptr, type, member) ((glame_list_empty(listptr)) ? ((type *)NULL) : \
@@ -224,7 +225,11 @@ static inline void glame_list_unsplice(struct glame_list_head *list,
 /* Iterate through all list items using entryvar. It is safe to remove
  * entryvar, but _only_ entryvar. Provide an additional dummy pointer
  * for internal use. */
+#ifdef __cplusplus
+#define glame_list_safe_foreach(listptr, type, member, dummy_ignored, entryvar) for (glame_list_head *dummy = (entryvar = glame_list_entry((listptr)->next, type, member), (listptr)->next->next); (&entryvar->member != (listptr) && (!glame_list_empty(&entryvar->member) || DERROR_eval("glame_list_safe_foreach remove error"))) || (entryvar = NULL); entryvar = glame_list_entry(dummy, type, member), dummy = ((struct glame_list_head *)dummy)->next) 
+#else
 #define glame_list_safe_foreach(listptr, type, member, dummy, entryvar) for (entryvar = glame_list_entry((listptr)->next, type, member), dummy = (void *)(listptr)->next->next; (&entryvar->member != (listptr) && (!glame_list_empty(&entryvar->member) || DERROR_eval("glame_list_safe_foreach remove error"))) || (entryvar = NULL); entryvar = glame_list_entry((struct glame_list_head *)dummy, type, member), dummy = (void *)((struct glame_list_head *)dummy)->next) 
+#endif
 
 /* Count the number of items in the list. */
 static inline int glame_list_count(struct glame_list_head *l)
