@@ -17,5 +17,5 @@
 	(if (filter? plugin)
 	   plugin
 	   (begin
-		(plugin_set plugin PLUGIN_DESCRIPTION "Bass Boost macro filter")
-		(plugin_set plugin PLUGIN_CATEGORY "Effects"))))
+		(plugin-set! plugin PLUGIN_DESCRIPTION "Bass Boost macro filter")
+		(plugin-set! plugin PLUGIN_CATEGORY "Effects"))))
