@@ -1,6 +1,6 @@
 /*
  * normalize.c
- * $Id: normalize.c,v 1.18.4.2 2004/10/23 13:14:23 richi Exp $
+ * $Id: normalize.c,v 1.24 2004/11/07 22:50:17 richi Exp $
  *
  * Copyright (C) 2001, 2002, 2003 Alexander Ehlert
  *
@@ -77,10 +77,10 @@ static void cleanup_task_list(struct normalize_s *ns) {
 	ns->head = NULL;
 }
 
-static gint normmode_cb(GtkMenu *menu, struct normalize_s* ns)
+static gint normmode_cb(GtkComboBox *menu, struct normalize_s* ns)
 {
 	int oldmode = ns->mode;
-	ns->mode =  glame_menu_get_active_index(menu);
+	ns->mode =  gtk_combo_box_get_active(menu);
 	DPRINTF("Selected normalize mode %d\n", ns->mode);
 	switch (ns->mode) {
 	case 0 : 
@@ -163,8 +163,6 @@ void normalize_dialog(struct normalize_s* norms)
   GtkObject *spinbutton2_adj;
   GtkWidget *spinbutton2;
   GtkWidget *optionmenu1;
-  GtkWidget *optionmenu1_menu;
-  GtkWidget *glade_menuitem;
   GtkWidget *label7;
   GtkObject *spinbutton3_adj;
   GtkWidget *spinbutton3;
@@ -291,7 +289,7 @@ void normalize_dialog(struct normalize_s* norms)
 		     "changed",
 		     (GtkSignalFunc)ampl_db_cb, norms);
 
-  optionmenu1 = gtk_option_menu_new ();
+  optionmenu1 = gtk_combo_box_new_text();
   gtk_widget_ref (optionmenu1);
   gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu1", optionmenu1,
                             (GtkDestroyNotify) gtk_widget_unref);
@@ -299,20 +297,13 @@ void normalize_dialog(struct normalize_s* norms)
   gtk_table_attach (GTK_TABLE (table1), optionmenu1, 1, 2, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
-  optionmenu1_menu = gtk_menu_new ();
-  glade_menuitem = gtk_menu_item_new_with_label (_("Peak"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
-  glade_menuitem = gtk_menu_item_new_with_label (_("Volume"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
-  glade_menuitem = gtk_menu_item_new_with_label (_("Volume/Frequency"));
-  gtk_widget_show (glade_menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);
+  gtk_combo_box_append_text(GTK_COMBO_BOX(optionmenu1), _("Peak"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(optionmenu1), _("Volume"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(optionmenu1), _("Volume/Frequency"));
+  gtk_combo_box_set_active(GTK_COMBO_BOX(optionmenu1), 0);
 
-  gtk_signal_connect(GTK_OBJECT(optionmenu1_menu),
-		     "selection_done",
+  gtk_signal_connect(GTK_OBJECT(optionmenu1),
+		     "changed",
 		     (GtkSignalFunc)normmode_cb, norms);
 
   norms->mode = 0;
@@ -363,7 +354,7 @@ void normalize_dialog(struct normalize_s* norms)
   gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 3);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
-  norms->text = text1 = gtk_text_new (NULL, NULL);
+  norms->text = text1 = gtk_text_view_new ();
   gtk_widget_ref (text1);
   gtk_object_set_data_full (GTK_OBJECT (dialog1), "text1", text1,
                             (GtkDestroyNotify) gtk_widget_unref);
@@ -431,7 +422,7 @@ void normalize_dialog(struct normalize_s* norms)
 
   gnome_dialog_append_button(GNOME_DIALOG(dialog1), GNOME_STOCK_BUTTON_HELP);
   gnome_dialog_button_connect(GNOME_DIALOG(dialog1), 2,
-			      glame_help_cb, "Normalize");
+			      (GtkSignalFunc)glame_help_cb, "Normalize");
 
 
   /* paranoia */
@@ -530,7 +521,7 @@ static void analyze_rms(struct normalize_s *ns) {
 			if (ns->running==0)
 				goto cancel_cleanup;
 
-			gnome_appbar_set_progress(GNOME_APPBAR(ns->appbar),
+			gnome_appbar_set_progress_percentage(GNOME_APPBAR(ns->appbar),
 						  percentage);
 		}
 		filter_launchcontext_unref(&context);
@@ -558,7 +549,7 @@ static void analyze_rms(struct normalize_s *ns) {
 	ns->maxrms = mrms = get_max_rms(te, &item);
 	snprintf(label, 127, "Found maximum rms = %.3f(%.3f dB) in track %s.\n\n", mrms, GAIN2DB(mrms), gpsm_item_label(item));
 	strcat(string, label);
-	gtk_text_insert(GTK_TEXT(ns->text), NULL, NULL, NULL, string, strlen(string));
+	gtk_text_buffer_insert_at_cursor(gtk_text_view_get_buffer(GTK_TEXT_VIEW(ns->text)), string, strlen(string));
 
 	ns->changed = 0;
 	return;
@@ -635,8 +626,8 @@ static void normalize_do_task(struct normalize_s *ns) {
 			if (ns->running==0)
 				goto cancel_cleanup;
 
-			gnome_appbar_set_progress(GNOME_APPBAR(ns->appbar),
-						  percentage);
+			gnome_appbar_set_progress_percentage(GNOME_APPBAR(ns->appbar),
+						  	     percentage);
 		}
 		ns->running = 0;
 		filter_launchcontext_unref(&context);
