@@ -283,14 +283,14 @@ int swapfile_open(const char *name, int flags)
 	return 0;
 }
 
-void swapfile_sync(void)
+void swapfile_sync()
 {
 	file_sync();
 }
 
 /* Closes and updates a previously opened swap file/partition
  * and marks it clean. */
-void swapfile_close(void)
+void swapfile_close()
 {
 	struct swfd *f;
 	char s[256];
@@ -773,7 +773,7 @@ int sw_unlink(long name)
 /* Open the (flat) swapfile directory for reading. The stream
  * is positioned at the first file. Like opendir(3), but w/o
  * directory specification for obvious reason. */
-SWDIR *sw_opendir(void)
+SWDIR *sw_opendir()
 {
 	if (!SWAPFILE_OK()) {
 		errno = EINVAL;
