@@ -204,7 +204,7 @@ static int cluster_init(int maxlru, int maxfds,
 	return 0;
 }
 
-static void cluster_cleanup(void)
+static void cluster_cleanup()
 {
 	struct swcluster *c;
 	int i;
@@ -484,7 +484,7 @@ static void _cluster_killmap(struct swcluster *c)
 	UNLOCKMAPPINGS;
 }
 
-static inline void __cluster_shrinkmaps(void)
+static inline void __cluster_shrinkmaps()
 {
 	while (clusters_mappedsize > clusters_maxmappedsize
 	       || clusters_maplru_cnt > clusters_maplru_maxcnt) {
@@ -803,7 +803,7 @@ static ssize_t cluster_write(struct swcluster *c, const void *buf,
  */
 
 /* Shrink the fdlru to honour the max size set. Caller needs to hold FD lock. */
-static void __cluster_fdlru_shrink(void)
+static void __cluster_fdlru_shrink()
 {
 	while (clusters_fdlru_cnt > clusters_fdlru_maxcnt) {
 		struct swcluster *c2;
@@ -838,7 +838,7 @@ static int __cluster_meta_open(long name, int flags)
 }
 
 /* Alloc and init a cluster struct. */
-static struct swcluster *_cluster_new(void)
+static struct swcluster *_cluster_new()
 {
 	struct swcluster *c;
 
