@@ -179,7 +179,7 @@ static void file_put(struct swfile *f, int flags)
 						     CSIZE(f->clusters, i)))) {
 					if (cluster_delfileref(c, f->name) == -1)
 						SWAPFILE_MARK_UNCLEAN("Cannot del fileref");
-					cluster_put(c, 0);
+					cluster_put(c, CLUSTERPUT_FREE);
 				} else
 					SWAPFILE_MARK_UNCLEAN("Cannot get cluster");
 			}
@@ -205,7 +205,7 @@ static void file_put(struct swfile *f, int flags)
 	file_check(f);
 }
 
-static void file_sync(void)
+static void file_sync()
 {
 	int i;
 
